/*
 * Amiga MUD
 *
 * Copyright (c) 1999 by Chris Gray
 */

/*
 * Common/baseProp.m - the most basic properties used in the scenario.
 */

public t_baseProp CreateTable()$
use t_baseProp

/* Property nameing convention:
    p_o - property on an object
    p_p - property on a player
    p_m - property on a machine/monster
    p_l - property on a location (e.g. room or map area)
    p_r - property on a room, when in a room-based world
    p_lm - property on a location model (mode specific stuff)
 */

/* For now, we simply have a count on how many objects can be carried. */

public MAX_CARRY 7$

/* These properties are made public, so that a non-builder can make code
   to use them in the playpen. These are special in that they are
   referenced by the automatically generated code. We want that code to
   compile properly, even if the builder hasn't @use'd any tables such
   as t_baseProp. */

public p_lName CreateStringProp()$		/* the name of a "room" */
public p_lContents CreateThingListProp()$	/* the "room"'s contents */
public p_pCarrying CreateThingListProp()$	/* player's inventory */
public p_oName CreateStringProp()$		/* encoded name of object */
public p_oCarryer CreateThingProp()$		/* who is carrying it */
public p_oCreator CreateThingProp()$		/* who "created" it */
public p_oContents CreateThingListProp()$	/* contents of an object */

define t_baseProp p_Image CreateStringProp()$	/* name of image file */

/* some properties for players */

define t_baseProp p_pDesc CreateStringProp()$	/* the player's description */
define t_baseProp p_pDescCheck CreateActionProp()$	/* check if look OK */
define t_baseProp p_pDescAction CreateActionProp()$	/* routine instead */
define t_baseProp p_pDescMore CreateActionListProp()$	/* more desc stuff */
define t_baseProp p_pMoney CreateIntProp()$ 	/* how much money player has */
define t_baseProp p_pVerbose CreateBoolProp()$	/* wants verbose desc */
define t_baseProp p_pSuperBrief CreateBoolProp()$	/* wants superbrief */
define t_baseProp p_pEchoPose CreateBoolProp()$	/* echo pose, say to them */
define t_baseProp p_pHidden CreateBoolProp()$	/* for special tricks */
define t_baseProp p_pStandard CreateBoolProp()$	/* not on fight monsters */
define t_baseProp p_pPosition CreateIntProp()$	/* special codes */
define t_baseProp p_pWhere CreateThingProp()$	/* e.g. on a bench */
define t_baseProp p_pCursor CreateIntListProp()$	/* the chosen cursor */
define t_baseProp p_pCursorColour CreateIntProp()$	/* colour they want */
define t_baseProp p_pIconColour CreateIntProp()$	/* colour for these */
define t_baseProp p_pPrivileged CreateBoolProp()$
define t_baseProp p_pBuilder CreateBoolProp()$	/* used in the 'build' code */
define t_baseProp p_pGivePre CreateActionProp()$	/* before obj check */
define t_baseProp p_pGivePost CreateActionProp()$	/* after obj check */
define t_baseProp p_pEnterActions CreateActionListProp()$   /* do on startup */
define t_baseProp p_pToRoomActions CreateActionListProp()$  /* other => room */
define t_baseProp p_pExitActions CreateActionListProp()$    /* do on idle */
define t_baseProp p_pHiddenList CreateThingListProp()$	    /* whatever */
define t_baseProp p_pAliases CreateThingListProp()$	/* top level aliases */
define t_baseProp p_sAliasKey CreateStringProp()$	/* the alias key */
define t_baseProp p_sAliasValue CreateStringProp()$	/* aliased command */
define t_baseProp p_pFollowing CreateThingProp()$	/* who I follow */
define t_baseProp p_pFollowers CreateThingListProp()$	/* those following */

/* these are set by provided Add/Del routines, so can be private */

define tp_common p_pLeaveChecks CreateActionListProp()$
define tp_common p_pEnterChecks CreateActionListProp()$

/* some properties for objects */

define t_baseProp p_oDesc CreateStringProp()$	/* long description of it */
define t_baseProp p_oDescCheck CreateActionProp()$	/* check if look OK */
define t_baseProp p_oDescAction CreateActionProp()$	/* call proc instead */
define t_baseProp p_oDrawAction CreateActionProp()$	/* for drawn stuff */
define t_baseProp p_oInvisible CreateBoolProp()$	/* don't show on look*/
define t_baseProp p_oHome CreateThingProp()$	/* its home location */
define t_baseProp p_oWhere CreateThingProp()$	/* "room" it is now in */
define t_baseProp p_oLight CreateBoolProp()$	/* also on players/machines */
define t_baseProp p_oReadString CreateStringProp()$ /* stuff on a sign, etc. */
define t_baseProp p_oReadAction CreateActionProp()$
define t_baseProp p_pReadChecker CreateActionProp()$	/* can he read? */
define t_baseProp p_oState CreateIntProp()$ 	/* handy for lots of things */
define t_baseProp p_oCanSitIn CreateIntProp()$	/* # + 1 who can sit in it */
define t_baseProp p_oCanSitOn CreateIntProp()$	/* # + 1 who can sit on it */
define t_baseProp p_oCanLieIn CreateIntProp()$	/* # + 1 who can lie in it */
define t_baseProp p_oCanLieOn CreateIntProp()$	/* # + 1 who can lie on it */
define t_baseProp p_oCanStandIn CreateIntProp()$/* # + 1 who can stand in it */
define t_baseProp p_oCanStandOn CreateIntProp()$/* # + 1 who can stand on it */
define t_baseProp p_oPositionChecker CreateActionProp()$/* checker for above */
define t_baseProp p_oGiveChecker CreateActionProp()$	/* when try give it */
define t_baseProp p_oPutMeInChecker CreateActionProp()$
define t_baseProp p_oPutInMeChecker CreateActionProp()$
define t_baseProp p_oTakeMeFromChecker CreateActionProp()$
define t_baseProp p_oTakeFromMeChecker CreateActionProp()$
define t_baseProp p_oFillMeWithChecker CreateActionProp()$
define t_baseProp p_oFillWithMeChecker CreateActionProp()$
define t_baseProp p_oEmptyChecker CreateActionProp()$
define t_baseProp p_oLookInString CreateStringProp()$
define t_baseProp p_oLookInAction CreateActionProp()$
define t_baseProp p_oNotUnlockString CreateStringProp()$
define t_baseProp p_oNotLockString CreateStringProp()$
define t_baseProp p_oUnlockMeWithChecker CreateActionProp()$
define t_baseProp p_oUnlockWithMeChecker CreateActionProp()$
define t_baseProp p_oLockMeWithChecker CreateActionProp()$
define t_baseProp p_oLockWithMeChecker CreateActionProp()$
define t_baseProp p_oNotLockable CreateBoolProp()$
define t_baseProp p_oLocked CreateBoolProp()$
define t_baseProp p_oActWord CreateStringProp()$	/* word to 'act' obj */
define t_baseProp p_oActString CreateStringProp()$	/* string to print */
define t_baseProp p_oActAction CreateActionProp()$	/* routine to call */
define t_baseProp p_oSpecialGrammar CreateGrammarProp()$    /* for specials */
define t_baseProp p_oCapacity CreateIntProp()$		/* max contents */
define t_baseProp p_oConsumer CreateThingProp()$	/* for potions */

define tp_common POS_NONE 0$
define tp_common POS_SIT_IN 1$
define tp_common POS_SIT_ON 2$
define tp_common POS_LIE_IN 3$
define tp_common POS_LIE_ON 4$
define tp_common POS_STAND_IN 5$
define tp_common POS_STAND_ON 6$

define t_baseProp p_oNotGettable CreateBoolProp()$	/* prevent get it */
define t_baseProp p_oGetChecker CreateActionProp()$	/* passed the object */
define t_baseProp p_oDropChecker CreateActionProp()$	/* passed the object */
define t_baseProp p_oUnGetChecker CreateActionProp()$	/* passed the object */
define t_baseProp p_oBuyChecker CreateActionProp()$	/* how to buy item */
define t_baseProp p_oPrice CreateIntProp()$		/* price of it */

/* some properties for locations */

define t_baseProp p_lNameAction CreateActionProp()$	/* call proc instead */
define t_baseProp p_lName1 CreateStringProp()$		/* for name box */
define t_baseProp p_lName2 CreateStringProp()$
define t_baseProp p_lDark CreateBoolProp()$		/* its dark in here */
define t_baseProp p_lNoMachines CreateBoolProp()$	/* keep machines out */
define t_baseProp p_lLocked CreateBoolProp()$		/* only owner go in */
define t_baseProp p_lScenery CreateStringProp()$	/* scenery names */

define t_baseProp p_lGetChecks CreateActionListProp()$	/* passed the object */
define t_baseProp p_lDropChecks CreateActionListProp()$	/* passed the object */

define t_baseProp p_lSayChecks CreateActionListProp()$	/* passed stuff said */
define t_baseProp p_lLookChecks CreateActionListProp()$	/* instead of/with */

define t_baseProp p_lLightChecks CreateActionListProp()$/* light introduced */

/* kind codes for effects */

define t_baseProp EFFECT_SOUND		0$
define t_baseProp EFFECT_SPEECH		1$
define t_baseProp EFFECT_MUSIC		2$

/* kind codes for EditString */

define t_baseProp EDIT_COOKED		0$
define t_baseProp EDIT_RAW		1$
define t_baseProp EDIT_CODE		2$

/* comparison codes for FindIntOnList */

define t_baseProp COMP_EQUAL		0$
define t_baseProp COMP_NOT_EQUAL	1$
define t_baseProp COMP_LESS_THAN	2$
define t_baseProp COMP_LESS_OR_EQUAL	3$
define t_baseProp COMP_GREATER_THAN 	4$
define t_baseProp COMP_GREATER_OR_EQUAL	5$
