/*
 * Amiga MUD
 *
 * Copyright (c) 1999 by Chris Gray
 */

/*
 * baseVerbs.m - define the standard basic verbs (commands).
 *	This set is the ones not dependent on location types.
 */

public t_baseVerbs CreateTable()$
use t_baseVerbs

private tp_baseVerbs CreateTable()$
use tp_baseVerbs

/* All of the separator words that we will use */

ignore Word(G, "to")$
ignore Word(G, "at")$
ignore Word(G, "around")$
ignore Word(G, "from")$
ignore Word(G, "on")$
ignore Word(G, "onto")$
ignore Word(G, "do")$

/* Also used, but are defined as verbs.

    "in"/"into"/"inside"/"inwards", "out"/"outside",
    "up", "down", "with", "off"

*/

/* Need to define 'off' early */

define tp_baseVerbs proc v_quit()bool:
    /* Bit of a nuisance - the server handles the messages for coming and
       going from the world, and it knows nothing of p_pHidden. Oh well. */
    Quit();
    ClearFollowers(Me());
    false
corp;

Verb0(G, "quit", 0, v_quit)$
Synonym(G, "quit", "bye")$
Verb0(G, "off", 0, v_quit)$	/* used separately */


/* This verb is defined early, so we can use "with" for other verbs. */

define tp_baseVerbs proc v_with()bool:
    string what, whatName;
    status st;
    thing object;
    grammar g;

    what := GetNounPhrase(G, GetTail(), FindAnyWord(G, "do"));
    if what = "" then
	Print("You must specify what you want to do something with.\n");
	false
    else
	if GetWord() ~= "do" then
	    Print("Use the form: with <object> do <command>\n");
	    false
	else
	    whatName := FormatName(what);
	    st := FindName(Me()@p_pCarrying, p_oName, what);
	    if st = succeed then
		object := FindResult();
		g := object@p_oSpecialGrammar;
		if g = nil then
		    Print("The " + whatName + " has no special actions.\n");
		    false
		else
		    what := GetTail();
		    if what = "" then
			Print("You must specify what you want to do with the "+
			    whatName + ".\n");
			false
		    else
			/* This is the magic here. Take the remainder of
			   the player's command line ("GetTail()"), and
			   parse it against the grammar retrieved from the
			   special object. */
			SetIt(object);
			what := GetTail();
			if Punctuation() ~= " " then
			    what := what + Punctuation();
			fi;
			Parse(g, what) ~= 0
		    fi
		fi
	    elif st = continue then
		Print(Capitalize(whatName) + " is ambiguous.\n");
		false
	    else
		Print(AAn("You are not carrying", whatName) + ".\n");
		false
	    fi
	fi
    fi
corp;

VerbTail(G, "with", v_with)$


define t_baseVerbs proc v_owner(string what)bool:
    string whatName;
    status st;
    thing obj;

    if what = "" then
	Print("Use is: owner <object>\n");
	false
    else
	whatName := FormatName(what);
	st := FindName(Me()@p_pCarrying, p_oName, what);
	if st = succeed then
	    obj := FindResult();
	    if Owner(obj) = SysAdmin then
		obj := obj@p_oCreator;
		if obj ~= nil then
		    Print("The " + whatName + " is owned by " +
			obj@p_pName + ".\n");
		else
		    Print("The " + whatName + " is owned by SysAdmin.\n");
		fi;
	    else
		Print("The " + whatName + " is owned by " +
		    CharacterThing(Owner(obj))@p_pName + ".\n");
		fi;
	    true
	elif st = continue then
	    Print(Capitalize(whatName) + " is ambiguous.\n");
	    false
	else
	    Print(AAn("You are not carrying", whatName) + ".\n");
	    false
	fi
    fi
corp;

Verb1(G, "owner", 0, v_owner)$


define t_baseVerbs proc v_inventory()bool:
    int cash;
    thing me;

    me := Me();
    cash := me@p_pMoney;
    if cash = 0 then
	Print("You are broke.\n");
    else
	Print("You have ");
	IPrint(cash);
	if cash = 1 then
	    Print(" bluto.\n");
	else
	    Print(" blutos.\n");
	fi;
    fi;
    if ShowList(me@p_pCarrying, "You are carrying:\n") then
	Print("You are not carrying anything.\n");
    fi;
    MaybeShowMeDo("takes inventory");
    true
corp;

Verb0(G, "inventory", 0, v_inventory)$
Synonym(G, "inventory", "inv")$
Synonym(G, "inventory", "i")$

define tp_baseVerbs proc v_score()bool:

    Print(
	"There is no 'score' command in this game. Instead, use 'inv' "
	"or 'take inventory', etc. to see what you are carrying; use "
	"'quests' to see what quests you have completed; and, if you have "
	"setup combat, use 'status' or 'st' to see your (or someone else's "
	"if you are a wizard) combat stats and equipment.\n"
    );
    true
corp;

Verb0(G, "score", 0, v_score)$


/* a few utility-type commands */

define tp_baseVerbs proc v_time()bool:
    Print("The time and date at the server is: " + Date() + ".\n");
    true
corp;

Verb0(G, "time", 0, v_time)$
Verb0(G, "date", 0, v_time)$

define tp_baseVerbs proc v_verbose()bool:
    thing me;

    me := Me();
    if me@p_pVerbose then
	Print("You are already getting verbose descriptions!\n");
	false
    else
	me@p_pVerbose := true;
	me@p_pSuperBrief := false;
	Print("Verbose desciptions turned on.\n");
	true
    fi
corp;

Verb0(G, "verbose", 0, v_verbose)$

define tp_baseVerbs proc v_terse()bool:
    thing me;

    me := Me();
    if me@p_pSuperBrief then
	me@p_pSuperBrief := false;
	Print("Superterse mode turned off.\n");
	true
    elif me@p_pVerbose then
	me@p_pVerbose := false;
	Print("Verbose descriptions turned off.\n");
	true
    else
	Print("You are already in terse mode!\n");
	false
    fi
corp;

Verb0(G, "terse", 0, v_terse)$
Synonym(G, "terse", "brief")$

define tp_baseVerbs proc v_superterse()bool:
    thing me;

    me := Me();
    if me@p_pSuperBrief then
	Print("You are already in superterse mode!\n");
	false
    else
	me@p_pVerbose := false;
	me@p_pSuperBrief := true;
	Print("Superterse mode enabled.\n");
	true
    fi
corp;

Verb0(G, "superterse", 0, v_superterse)$
Synonym(G, "superterse", "superbrief")$

define tp_baseVerbs proc v_echo()bool:
    thing me;

    me := Me();
    if me@p_pEchoPose then
	me@p_pEchoPose := false;
	Print("Say/whisper/pose/emits will no longer be echoed to you.\n");
    else
	me@p_pEchoPose := true;
	Print("Say/whisper/pose/emits will now be echoed to you.\n");
    fi;
    true
corp;

Verb0(G, "echo", 0, v_echo)$

define tp_baseVerbs proc v_ats()bool:
    bool oldValue;

    oldValue := PrintNoAts(true);
    if oldValue then
	Print("Non-wizard output will now be identified by leading '@'s.\n");
	ignore PrintNoAts(false);
    else
	Print(
"Non-wizard output will no longer be identified by leading '@'s. You are "
"warned that unscrupulous apprentices may attempt to trick you by printing "
"false messages indistinguishable from normal messages. E.g.\n\n"
"SysAdmin gives you 10000 blutos.\n\n"
"Beware of this type of \"spoofing\".\n");
	ignore PrintNoAts(true);
    fi;
    true
corp;

Verb0(G, "ats", 0, v_ats)$

define tp_baseVerbs proc v_wizard()bool:
    if not WizardMode() then
	Print("Sorry, you can't go into wizard mode.\n");
	false
    else
	true
    fi
corp;

Verb0(G, "wizard", 0, v_wizard)$


define tp_baseVerbs proc v_password()bool:
    NewCharacterPassword();
    true
corp;

Verb0(G, "password", 0, v_password)$

define tp_baseVerbs proc v_prompt()bool:
    string w;

    w := GetWord();
    Print("Setting prompt to " + w + ".\n");
    ignore SetPrompt(w);
    true
corp;

VerbTail(G, "prompt", v_prompt)$

define tp_baseVerbs proc v_name(string newName)bool:
    thing me;
    string oldName, desc;
    int len;

    me := Me();
    if Character(me@p_pName) = nil then
	OPrint(Capitalize(CharacterNameS(me)) + " is confused.\n");
	false
    elif newName = "" then
	Print("You must give the new name you want to use.\n");
	false
    else
	SetTail(newName);
	ignore GetWord();
	if GetWord() ~= "" then
	    Print("Sorry, no spaces in character names.\n");
	    false
	elif Character(newName) ~= nil then
	    Print("Sorry, character name '" + newName +
		"' is already in use.\n");
	    false
	else
	    oldName := Capitalize(me@p_pName);
	    ChangeName(newName);
	    Print("Your name is now " + newName + ".\n");
	    APrint(oldName + " is now called " + newName + ".\n");
	    desc := me@p_pDesc;
	    len := Length(oldName);
	    if SubString(desc, 0, len) == oldName then
		me@p_pDesc := Capitalize(newName) +
		    SubString(desc, len, Length(desc) - len);
	    fi;
	    true
	fi
    fi
corp;

Verb1(G, "name", 0, v_name)$


/*
 * showAliases - used in 'v_alias' and in v_aliases.
 */

define tp_baseVerbs proc showAliases(list thing aliases)void:
    thing alias;
    int count;

    if aliases = nil then
	Print("You have no command aliases.\n");
    else
	count := Count(aliases);
	Print("Command aliases:\n");
	while count ~= 0 do
	    count := count - 1;
	    alias := aliases[count];
	    Print("  ");
	    Print(alias@p_sAliasKey);
	    Print(" => ");
	    Print(alias@p_sAliasValue);
	    Print("\n");
	od;
    fi;
corp;

define tp_baseVerbs proc v_aliases()bool:

    showAliases(Me()@p_pAliases);
    true
corp;

define tp_baseVerbs proc v_alias()bool:
    list thing aliases;
    thing alias;
    string word, contents;
    int count;
    bool found;

    word := GetWord();
    aliases := Me()@p_pAliases;
    if word = "" then
	showAliases(aliases);
    else
	Print("Command alias '");
	Print(word);
	Print("' ");
	found := false;
	if aliases ~= nil then
	    count := Count(aliases);
	    while count ~= 0 and not found do
		count := count - 1;
		alias := aliases[count];
		if alias@p_sAliasKey == word then
		    found := true;
		fi;
	    od;
	fi;
	contents := GetTail();
	if contents = "" then
	    if found then
		DelElement(aliases, alias);
		Print("removed.\n");
	    else
		Print("does not exist.\n");
	    fi;
	else
	    if SubString(contents, 0, 1) = "\"" then
		contents := SubString(contents, 1, Length(contents) - 2);
	    fi;
	    if found then
		alias@p_sAliasValue := contents;
		Print("updated.\n");
	    else
		if aliases = nil then
		    aliases := CreateThingList();
		    Me()@p_pAliases := aliases;
		fi;
		alias := CreateThing(nil);
		alias@p_sAliasKey := word;
		alias@p_sAliasValue := contents;
		AddTail(aliases, alias);
		Print("added.\n");
	    fi;
	fi;
    fi;
    true
corp;

Verb0(G, "aliases", 0, v_aliases)$
VerbTail(G, "alias", v_alias)$
Synonym(G, "alias", "a")$


define tp_baseVerbs proc DoBackup(thing obj)void:
    int saveLimit;

    /* We want to schedule the next backup before we actually do this one,
       so that the timekeeper will have us on its list in the backup! */
    if GlobalThing@p_BackupInterval ~= 0 then
	DoAfter(GlobalThing@p_BackupInterval * 60, GlobalThing, DoBackup);
    fi;
    if GlobalThing@p_FlushNeeded then
	saveLimit := RunLimit(100);
	/* clear flag *before* doing backup, so are clean after restore */
	GlobalThing@p_FlushNeeded := false;
	Log("Starting automatic backup, interval = " +
	    IntToString(GlobalThing@p_BackupInterval) + " minutes\n");
	APrint("\n * Timed database backup starting - please wait. *\n");
	Flush();
/*	Execute("copy MUD.#? backup"); */
	Execute("cp MUD.* backup");
	APrint(" * Backup complete. *\n\n");
	ignore RunLimit(saveLimit);
    fi;
corp;

define tp_baseVerbs proc v_backup(string arg)bool:
    int n;

    if Me() ~= CharacterThing(SysAdmin) then
	Print("Only SysAdmin can use the 'backup' command.\n");
	false
    else
	if arg = "" then
	    /* do it unconditionally */
	    n := RunLimit(100);
	    Print("Doing one-shot backup.\n");
	    GlobalThing@p_FlushNeeded := false;
	    Log("Starting one-shot backup.\n");
	    APrint("\n * Database being backed up - please wait. *\n");
	    Flush();
/*	    Execute("copy MUD.#? backup");*/
	    Execute("cp MUD.* backup");
	    APrint(" * Backup complete. *\n\n");
	    ignore RunLimit(n);
	    true
	else
	    n := StringToInt(arg);
	    if n = 0 then
		GlobalThing@p_BackupInterval := 0;
		ignore CancelDoAfter(GlobalThing, DoBackup);
		Print("Automated backups cancelled.\n");
		true
	    elif n > 0 then
		if n > 0x7fffffff / 60 then
		    n := 0x7fffffff / 60;
		fi;
		GlobalThing@p_BackupInterval := n;
		Print("Backup interval set to " + IntToString(n) +
		    " minutes.\n");
		ignore CancelDoAfter(GlobalThing, DoBackup);
		DoAfter(n * 60, GlobalThing, DoBackup);
		Print("Next backup will occur that interval from now.\n");
		true
	    else
		Print("Expecting minute count with 'backup'.\n");
		false
	    fi
	fi
    fi
corp;

Verb1(G, "backup", 0, v_backup)$


define tp_baseVerbs proc v_shutdown()bool:
    string message, m;

    if Me() ~= CharacterThing(SysAdmin) then
	Print("Only SysAdmin can use the 'shutdown' command.\n");
	false
    else
	message := "\n\n**** ";
	m := GetTail();
	if m ~= "" then
	    message := message + m + "\n";
	fi;
	message := message +
	    "SysAdmin has started shutdown of the MUD. ****\n\n\n";
	APrint(message);
	ignore ShutDown(true);
	true
    fi
corp;

VerbTail(G, "shutdown", v_shutdown)$


define tp_baseVerbs proc v_freeposes()bool:
    string errorMessage, yesNo;

    if Me() ~= CharacterThing(SysAdmin) then
	Print("Only SysAdmin can use the 'freeposes' command.\n");
	false
    else
	yesNo := GetWord();
	if yesNo = "" then
	    yesNo := "y";
	fi;
	errorMessage := "Use is: freeposes [y|n]\n";
	if GetWord() ~= "" then
	    Print(errorMessage);
	    false
	elif IsYes(yesNo) then
	    GlobalThing@p_FreePoses := true;
	    Print("Free user poses enabled.\n");
	    true
	elif IsNo(yesNo) then
	    GlobalThing@p_FreePoses := false;
	    Print("Free user poses disabled.\n");
	    true
	else
	    Print(errorMessage);
	    false
	fi
    fi
corp;

define tp_baseVerbs proc v_freebuilding()bool:
    string errorMessage, yesNo;

    if Me() ~= CharacterThing(SysAdmin) then
	Print("Only SysAdmin can use the 'freebuilding' command.\n");
	false
    else
	yesNo := GetWord();
	if yesNo = "" then
	    yesNo := "y";
	fi;
	errorMessage := "Use is: freebuilding [y|n]\n";
	if GetWord() ~= "" then
	    Print(errorMessage);
	    false
	elif IsYes(yesNo) then
	    GlobalThing@p_FreeBuilding := true;
	    Print("All new characters will now be builders.\n");
	    true
	elif IsNo(yesNo) then
	    GlobalThing@p_FreeBuilding := false;
	    Print("New characters will no longer be builders.\n");
	    true
	else
	    Print(errorMessage);
	    false
	fi
    fi
corp;

define tp_baseVerbs proc v_makeapprentice()bool:
    string name;
    character who;

    if Me() ~= CharacterThing(SysAdmin) then
	Print("Only SysAdmin can use the 'makeapprentice' command.\n");
	false
    else
	name := GetWord();
	if name = "" or GetWord() ~= "" then
	    Print("Use is: makeapprentice <player-name>\n");
	    false
	else
	    who := Character(name);
	    if who = nil then
		Print("'" + name + "' is not a character.\n");
		false
	    else
		MakeApprentice(who, false);
		Print("'" + name + "' is now an apprentice.\n");
		true
	    fi
	fi
    fi
corp;

VerbTail(G, "freeposes", v_freeposes)$
VerbTail(G, "freebuilding", v_freebuilding)$
VerbTail(G, "makeapprentice", v_makeapprentice)$


define tp_baseVerbs proc v_shout()bool:
    string tail;

    if GlobalThing@p_AllowShout then
	tail := GetTail();
	if tail = "" then
	    Print("You must say what you want to shout.\n");
	    false
	else
	    APrint(CharacterNameS(Me()) + " shouts: " + tail + "\n");
	    true
	fi
    else
	Print("Sorry, the sysadmin had not enabled shouting.\n");
	false
    fi
corp;

define tp_baseVerbs proc v_allowshout()bool:
    string errorMessage, yesNo;

    if Me() ~= CharacterThing(SysAdmin) then
	Print("Only SysAdmin can use the 'allowshout' command.\n");
	false
    else
	yesNo := GetWord();
	if yesNo = "" then
	    yesNo := "y";
	fi;
	errorMessage := "Use is: allowshout [y|n]\n";
	if GetWord() ~= "" then
	    Print(errorMessage);
	    false
	elif IsYes(yesNo) then
	    GlobalThing@p_AllowShout := true;
	    Print("Shouting is now allowed.\n");
	    true
	elif IsNo(yesNo) then
	    GlobalThing@p_AllowShout := false;
	    Print("Shouting is no longer allowed.\n");
	    true
	else
	    Print(errorMessage);
	    false
	fi
    fi
corp;

VerbTail(G, "shout", v_shout)$
VerbTail(G, "allowshout", v_allowshout)$


define tp_baseVerbs proc v_characters(string form)bool:

    if form == "long" or form == "l" then
	ignore ShowCharacters(true);
	true
    elif form = "" or form == "short" or form == "s" then
	ignore ShowCharacters(false);
	true
    else
	Print("You must use either \"short\" or \"long\".\n");
	false
    fi
corp;

Verb1(G, "characters", 0, v_characters)$
Synonym(G, "characters", "chars")$
Synonym(G, "characters", "ch")$
Synonym(G, "characters", "players")$
Synonym(G, "characters", "pl")$

define tp_baseVerbs proc v_clients(string form)bool:

    if form == "long" or form == "l" then
	ignore ShowClients(true);
	true
    elif form = "" or form == "short" or form == "s" then
	ignore ShowClients(false);
	true
    else
	Print("You must use either \"short\" or \"long\".\n");
	false
    fi
corp;

Verb1(G, "clients", 0, v_clients)$
Synonym(G, "clients", "cl")$
Synonym(G, "clients", "users")$

define tp_baseVerbs proc v_character(string who)bool:
    character ch;

    if who = "" then
	Print("You must specify a character to check on.\n");
	false
    else
	ch := Character(who);
	if ch = nil then
	    ch := Character(Capitalize(who));
	fi;
	if ch = nil then
	    Print(who + " is not a character. Make sure you have the spelling "
		"and capitalization correct.\n");
	else
	    ShowCharacter(ch);
	fi;
	true
    fi
corp;

Verb1(G, "character", 0, v_character)$
Synonym(G, "character", "player")$
Synonym(G, "character", "char")$
Synonym(G, "character", "user")$

define tp_baseVerbs proc v_who()bool:
    Print("Use 'CHaracters [long|short]' to see existing characters.\n"
	"Use 'CLients [long|short]' to see current clients.\n"
	"Use 'CHARacter <name>, ...' to see given characters.\n");
    true
corp;

Verb0(G, "who", 0, v_who)$

define tp_baseVerbs proc v_width(string newWidth)bool:
    int width;

    if newWidth = "" then
	width := TextWidth(0);
	Print("Current display width is " + IntToString(width) + ".\n");
    else
	Print("Old width was " +
		IntToString(TextWidth(StringToInt(newWidth))) + ".\n");
    fi;
    true
corp;

Verb1(G, "width", 0, v_width)$
Synonym(G, "width", "columns")$
Synonym(G, "width", "cols")$

define tp_baseVerbs proc v_height(string newHeight)bool:
    int height;

    if newHeight = "" then
	height := TextHeight(0);
	Print("Current display height is " + IntToString(height) + ".\n");
    else
	Print("Old height was " +
		IntToString(TextHeight(StringToInt(newHeight))) + ".\n");
    fi;
    true
corp;

Verb1(G, "height", 0, v_height)$
Synonym(G, "height", "rows")$
Synonym(G, "height", "lines")$

define tp_baseVerbs proc v_volume()bool:
    string error, which, volString;
    int newVolume;

    error := "Use is: volume {sound | voice | music} <value:0-100>\n";
    which := GetWord();
    volString := GetWord();
    newVolume := StringToInt(volString);
    if newVolume >= 0 and newVolume <= 100 then
	newVolume := newVolume * 100;
	if which == "sound" then
	    SVolume(nil, newVolume);
	    true
	elif which == "voice" then
	    VVolume(nil, newVolume);
	    true
	elif which == "music" then
	    MVolume(nil, newVolume);
	    true
	else
	    Print(error);
	    false
	fi
    else
	Print(error);
	false
    fi
corp;

VerbTail(G, "volume", v_volume)$

define tp_baseVerbs proc showHelpHelp()void:
    Print(
"Use 'help <topic>' or '? <topic>' for specific help.\n"
"(Do not type the apostrophes (') or the < and >.)\n\n"
"Help topics: mouse, move, graphics, objects\n"
    );
corp;

define tp_baseVerbs proc v_helphelp()bool:

    showHelpHelp();
    true
corp;

Verb0(G, "helphelp", 0, v_helphelp)$
Synonym(G, "helphelp", "??")$

define tp_baseVerbs proc v_help(string what)bool:

    if what = "" then
	Print(
"Use 'help help' or '??' to see more help topics.\n\n"
"Standard commands available in this starter game:\n"
"    [go] north/south.../n/s/...enter/exit/up/down...\n"
"    look [around]; look at XXX, YYY, ...; examine XXX; look <direction>\n"
"    inventory/inv/i\n"
"    pick up XXX, YYY, ...; get/g XXX, YYY, ...\n"
"    put/p [down] XXX, YYY, ...; drop XXX, YYY, ...\n"
"    say XXX; \"xxx; smile, wave, ...; quests [who]\n"
"    quit; verbose; terse; password; prompt; name; who; height; width;\n"
"    play; erase; eat; use; wear; read; touch; smell; open; close;\n"
"    push; pull; turn; chat; pose; echo; follow; etc.\n"
"plus others as the game features require.\n"
	);
    elif what == "help" or what = "?" or what == "topics" or what == "topic"
    then
	showHelpHelp();
    elif what == "mouse" then
	if GOn(nil) then
	    Print(
"The mouse can be used instead of typing to enter some commands. The right "
"half of the graphics area usually contains a number of \"buttons\" that you "
"can click on. For example, clicking on the 'L' button is equivalent to "
"typing \"look around\" or just \"l\". You can also click in the left hand "
"half of the graphics area. Doing so will attempt to generate a command to "
"move your character one room towards where you click. Note that you can only "
"move in the 8 compass directions doing this. You cannot drag your character "
"around - you must move in single-room steps, in order to properly allow "
"interaction with other characters in the game. You also cannot just point to "
"something on the display and try to move to it - the movement is only in "
"single-room steps in the compass direction closest to what you select.\n"
	    );
	else
	    Print(
"Since you do not currently have graphics enabled, it is likely that you are "
"not using the custom AmigaMUD client program. In that case, you cannot use "
"your mouse for anything specific to playing in AmigaMUD, other than normal "
"use within your normal user interface.\n"
	    );
	fi;
    elif what == "move" then
	if GOn(nil) then
	    Print(
"There are several ways to move around. The quickest, once you get used to "
"it, is to use the numeric keypad. If you imagine your character cursor on "
"top of the '5' key, then the 1, 2, 3, 4, 6, 7, 8, and 9 keys will try to "
"move your character one room in the corresponding compass direction. For "
"example, typing the '7' key on the numeric keypad will attempt to move your "
"character one room to the north-west. This is equivalent to typing the "
"command   northwest   or just   nw. Typing the '5' key itself is the same as "
"using the   look around   command. You can also use '+' and '-' keys if you "
"have them, to move up and down.\n\n"
"Movement in this game is through a set of distinct \"rooms\". You cannot "
"move to arbitrary positions, only to the few positions that the game has "
"predefined. The graphics you see on your screen are only pictures - they do "
"not define what you can do. To see how the simpler graphics work, try going "
"into the playpen in the Builder's Guild, and building your own rooms. You "
"can do this by clicking on the '@' button that will appear, and then on the "
"'Rooms' button that will appear.\n"
	    );
	else
	    Print(
"Since you do not currently have graphics enabled, it is likely that you are "
"not using the custom AmigaMUD client program. In that case, you must move "
"around by typing individual movement commands. These commands are the eight "
"compass directions: north, northeast, east, southeast, south, southwest, "
"west and northwest, or their abbreviations: n, ne, e, se, s, sw, w, nw. You "
"can also move up (u) or down (d) and in or out.\n"
	    );
	fi;
    elif what == "graphics" or what == "graphic" then
	if GOn(nil) then
	    Print(
"The graphics displayed by AmigaMUD should not be interpreted literally. They "
"are sketches only. Even when you see a bitmap image, it is only meant to "
"guide your imagination as to what your character can currently see. The fact "
"that the picture shows a tree or fountain or road, does not mean that the "
"game has such a thing that you can do things to. In a game of this type, you "
"can only move among a fixed set of \"rooms\" - you cannot walk up to "
"everything you see in pictures, since some of them are scenery only. Pay "
"close attention to the textual description of things - it will more "
"accurately show what things have actually been built in the world, and will "
"also contains hints about things you can do. The words are the key to the "
"game - the pictures and sounds are usually only decorations.\n"
	    );
	else
	    Print(
"Since you do not currently have graphics enabled, graphics is not relevant "
"to you. Graphics are only available when you are using the custom AmigaMUD "
"client program.\n"
	    );
	fi;
    elif what == "objects" or what == "object" then
	Print(
"Objects are very important in games of this type. When the text you see when "
"you enter a room shows something being there, it is a good idea to look at "
"that object and remember where you saw it - it may be important to you. You "
"can carry objects around with you, but only so many of them. Also, some "
"objects cannot be picked up. When you see an object named, you can refer to "
"it in commands by that name, but you can also usually leave off the "
"adjectives - they are only needed if you have to choose from more than one "
"object of the same kind. Some commands useful with objects include get, "
"drop, look at, look in, fill, empty, read, etc. For example, if the room "
"you have just walked into contains a universal carryall and a delicious red "
"apple, you can probably do:\n\n"
"    get carryall\n"
"    g apple\n"
"    l in carryall\n"
"    put apple in carryall\n\n"
"etc. The game accepts simple abbreviations for most command words, and can "
"also handle longer, more grammatically correct forms of commands. It can "
"also work with multiple objects at once. For example:\n\n"
"    Pick up the universal carryall and the delicious red apple.\n"
"    Look inside the univeral carryall.\n"
"    Put the delicious red apple into the universal carryall.\n\n"
"Most people don't have the patience for such long forms, however!\n\n"
"Sometimes, the objects in a room will be \"hidden\" from you, in that they "
"will not explicitly show up. You have to read the room description, and find "
"out what is real, and what is just scenery.\n"
	);
    else
	Print("Unknown help topic. Type 'help help' or '??' for topics.\n");
    fi;
    true
corp;

Verb1(G, "help", 0, v_help)$
Synonym(G, "help", "?")$

define tp_baseVerbs proc v_words()bool:
    ShowWords(G);
    true
corp;

Verb0(G, "words", 0, v_words)$


/* some commands for folks to complain to SysAdmin */

define tp_baseVerbs proc complain(string kind)bool:
    thing location;
    string locName, ownerName;

    location := call(Me()@p_pCurrentMode@p_lmGetLocation, thing)();
    if location = nil then
	locName := "<no location>";
	ownerName := "<unknown>";
    else
	locName := location@p_lName;
	ownerName := CharacterThing(Owner(location))@p_pName;
    fi;
    Log(kind + ": '" + locName + "' owner " +
	ownerName + ": " + GetTail() + "\n");
    Print(kind + " logged.\n");
    true
corp;

define tp_baseVerbs proc v_typo()bool:
    complain("Typo")
corp;

define tp_baseVerbs proc v_bug()bool:
    complain("Bug")
corp;

define tp_baseVerbs proc v_gripe()bool:
    complain("Gripe")
corp;

VerbTail(G, "typo", v_typo)$
VerbTail(G, "bug", v_bug)$
VerbTail(G, "gripe", v_gripe)$
Synonym(G, "gripe", "complain")$
Synonym(G, "gripe", "bitch")$


define tp_baseVerbs proc v_cursor(string colour)bool:
    int which;

    if colour = "" then
	Print("Use is: cursor <colour-name>\n");
	ShowKnownColours();
	false
    else
	which := ColourMatch(colour);
	if which = -1 then
	    Print("Colour " + FormatName(colour) + " not known.\n");
	    false
	else
	    Me()@p_pCursorColour := which;
	    SetCursorPen(which);
	    true
	fi
    fi	
corp;

define tp_baseVerbs proc v_icons(string colour)bool:
    int which;

    if colour = "" then
	Print("Use is: icons <colour-name>\n");
	ShowKnownColours();
	false
    else
	which := ColourMatch(colour);
	if which = -1 then
	    Print("Colour " + FormatName(colour) + " not known.\n");
	    false
	else
	    Me()@p_pIconColour := which;
	    GSetIconPen(nil, which);
	    true
	fi
    fi
corp;

Verb1(G, "cursor", 0, v_cursor)$
Verb1(G, "icon", 0, v_icons)$
Synonym(G, "icon", "icons")$


unuse tp_baseVerbs
