/*
 * Amiga MUD
 *
 * Copyright (c) 1999 by Chris Gray
 */

/*
 * Common/charUtil.m - utility routines related to characters.
 */

public t_charUtil CreateTable()$
use t_charUtil

private tp_charUtil CreateTable()$
use tp_charUtil

/* routines to add/delete checkers on players */

define t_charUtil proc utility AddPlayerEnterChecker(thing player;
	action a; bool front)void:
    addChecker(player, p_pEnterChecks, a, front);
corp;

define t_charUtil proc utility DelPlayerEnterChecker(thing player;
	action a)void:
    delChecker(player, p_pEnterChecks, a);
corp;

define t_charUtil proc utility AddPlayerLeaveChecker(thing player;
	action a; bool front)void:
    addChecker(player, p_pLeaveChecks, a, front);
corp;

define t_charUtil proc utility DelPlayerLeaveChecker(thing player;
	action a)void:
    delChecker(player, p_pLeaveChecks, a);
corp;


/* routines to do some of the private player checks */

define t_charUtil proc utility DoPlayerLeaveChecks(thing who; int dir)status:
    DoIntChecks(who@p_pLeaveChecks, dir)
corp;

define t_charUtil proc utility DoPlayerEnterChecks(thing who)status:
    DoChecks(who@p_pEnterChecks)
corp;

/*
 * CharacterDescription - return the normal description of the passed character
 */

define t_charUtil proc CharacterDescription(thing who)string:
    string s;
    action a;

    SetIt(who);
    a := who@p_pDescAction;
    if a = nil then
	s := who@p_pDesc;
    else
	s := call(a, string)();
    fi;
    s + DoActionsString(who@p_pDescMore)
corp;

/*
 * LookAtCharacter - take a detailed look at the given player/monster.
 */

define t_charUtil proc LookAtCharacter(thing who)bool:
    action a;
    status st;
    string name, s;

    st := continue;
    a := who@p_pDescCheck;
    if a ~= nil then
	/* if p_pDescCheck returns 'continue', all is well and we
	   will continue looking at the player, and around. If it
	   returns 'succeed', then we are done looking at the
	   player, but can look around. If it returns 'fail', then
	   we cannot look anymore (or some such). */
	SetIt(who);
	st := call(a, status)();
    fi;
    if st ~= fail then
	if who@p_Image ~= "" then
	    ShowImage(who@p_Image);
	    a := Me()@p_pCurrentMode@p_lmClearMap;
	    if a ~= nil then
		call(a, void)();
	    fi;
	elif who@p_oDrawAction ~= nil then
	    ShowDrawing(who@p_oDrawAction, who);
	    a := Me()@p_pCurrentMode@p_lmClearMap;
	    if a ~= nil then
		call(a, void)();
	    fi;
	fi;
    fi;
    if st = continue then
	s := CharacterDescription(who);
	name := Capitalize(CharacterNameS(who));
	if s = "" then
	    s := name + " has no description - bug him/her about it."
	fi;
	Print(s + "\n");
	if who = Me() then
	    s := "You are carrying:\n";
	else
	    s := name + " is carrying:\n";
	fi;
	ignore ShowList(who@p_pCarrying, s);
	true
    else
	st = succeed
    fi
corp;

/* some code to aid the 'follow' verb. */

define tp_charUtil p_pMeFollowDir CreateIntProp()$
define tp_charUtil p_pFollowMeDir CreateIntProp()$

define t_charUtil proc Follow(thing leader)bool:
    list thing followers;
    thing me, lm;
    string name;

    name := CharacterNameS(leader);
    me := Me();
    lm := me@p_pCurrentMode;
    if call(lm@p_lmCanFollow, bool)(leader, me) then
	followers := leader@p_pFollowers;
	if followers = nil then
	    followers := CreateThingList();
	    leader@p_pFollowers := followers;
	fi;
	AddTail(followers, me);
	me@p_pFollowing := leader;
	call(lm@p_lmStartFollowing, void)(leader, me);
	call(lm@p_lmFollowSaveLocation, void)(me);
	Print("You are now following " + name + ".\n");
	true
    else
	Print("Sorry, you can't follow " + name + " from here.\n");
	false
    fi
corp;

define tp_charUtil proc doFollow()status:
    thing me;
    int dir;

    me := Me();
    dir := me@p_pMeFollowDir;
    Print("You follow " + CharacterNameS(me@p_pFollowing) + " to " +
	DirName(dir) + ".\n");
    if call(me@p_pCurrentMode@p_lmDoMove, bool)(dir) then
	call(me@p_pCurrentMode@p_lmFollowSaveLocation, void)(me);
	continue
    else
	fail
    fi
corp;

define tp_charUtil proc clearLeader(thing leader)void:

    if leader@p_pFollowing = nil then
	call(leader@p_pCurrentMode@p_lmStopFollowing, void)(leader);
    fi;
    leader -- p_pFollowers;
    leader -- p_pFollowMeDir;
corp;

define tp_charUtil proc doCheckFollowers()void:
    list thing followers;
    thing leader, follower, lm;
    int dir, i;

    leader := Me();
    lm := leader@p_pCurrentMode;
    followers := leader@p_pFollowers;
    if followers ~= nil then
	dir := leader@p_pFollowMeDir;
	i := Count(followers);
	while i ~= 0 do
	    i := i - 1;
	    follower := followers[i];
	    if follower@p_pFollowing = leader and
		call(lm@p_lmAgentStillFollowing, bool)(leader, follower)
	    then
		follower@p_pMeFollowDir :=
		    call(lm@p_lmFollowingStep, int)(follower, dir);
		if ForceAction(follower, doFollow) ~= continue
		then
		    /* the follow failed - remove this follower */
		    /* note that we specifically do not inform the
		       follower! */
		    call(follower@p_pCurrentMode@p_lmStopFollowing, void)
			(follower);
		    follower -- p_pFollowing;
		    follower -- p_pMeFollowDir;
		    DelElement(followers, follower);
		fi;
	    else
		/* Again, we do not inform the follower - that could tell
		   him that the one he was following has moved, when he
		   should not normally know that. */
		call(follower@p_pCurrentMode@p_lmStopFollowing, void)
		    (follower);
		follower -- p_pFollowing;
		follower -- p_pMeFollowDir;
		DelElement(followers, follower);
	    fi;
	od;
	if Count(followers) = 0 then
	    clearLeader(leader);
	fi;
    fi;
corp;

define t_charUtil proc CheckFollowers(int dir)void:
    thing me;

    me := Me();
    if me@p_pFollowers ~= nil then
	me@p_pFollowMeDir := dir;
	After(0.0, doCheckFollowers);
    fi;
corp;

define t_charUtil proc ClearFollowers(thing leader)void:
    list thing followers;
    thing follower;
    int i;
    string name;

    followers := leader@p_pFollowers;
    if followers ~= nil then
	name := CharacterNameS(leader);
	for i from 0 upto Count(followers) - 1 do
	    follower := followers[i];
	    if follower@p_pFollowing = leader then
		follower -- p_pFollowing;
		follower -- p_pMeFollowDir;
		SPrint(follower,"You are no longer following " + name + ".\n");
	    fi;
	od;
	clearLeader(leader);
    fi;
corp;

define t_charUtil proc UnFollow()void:
    list thing followers;
    thing me, leader;

    me := Me();
    leader := me@p_pFollowing;
    if leader ~= nil then
	call(me@p_pCurrentMode@p_lmStopFollowing, void)(me);
	me -- p_pFollowing;
	me -- p_pMeFollowDir;
	Print("You stop following " + CharacterNameS(leader) + ".\n");
	followers := leader@p_pFollowers;
	if followers ~= nil then
	    DelElement(followers, me);
	    if Count(followers) = 0 then
		clearLeader(leader);
	    fi;
	fi;
    fi;
corp;

/*
 * UserMove - the user is explicitly moving.
 */

define t_charUtil proc UserMove(int dir)bool:

    UnFollow();
    call(Me()@p_pCurrentMode@p_lmDoMove, bool)(dir)
corp;


unuse tp_charUtil
