/*
 * Amiga MUD
 *
 * Copyright (c) 1999 by Chris Gray
 */

/*
 * Common/directions.m - general stuff dealing with directions.
 */

public t_directions CreateTable()$
use t_directions

/* Define the 12 standard directions. */

define t_directions D_NORTH		0 $
define t_directions D_NORTHEAST		1 $
define t_directions D_EAST		2 $
define t_directions D_SOUTHEAST		3 $
define t_directions D_SOUTH		4 $
define t_directions D_SOUTHWEST		5 $
define t_directions D_WEST		6 $
define t_directions D_NORTHWEST		7 $
define t_directions D_UP		8 $
define t_directions D_DOWN		9 $
define t_directions D_ENTER		10 $
define t_directions D_EXIT		11 $
define t_directions D_HERE		12 $

/* Define bit values for them. */

define t_directions B_NORTH		1 << D_NORTH $
define t_directions B_NORTHEAST		1 << D_NORTHEAST $
define t_directions B_EAST		1 << D_EAST $
define t_directions B_SOUTHEAST		1 << D_SOUTHEAST $
define t_directions B_SOUTH		1 << D_SOUTH $
define t_directions B_SOUTHWEST		1 << D_SOUTHWEST $
define t_directions B_WEST		1 << D_WEST $
define t_directions B_NORTHWEST		1 << D_NORTHWEST $
define t_directions B_UP		1 << D_UP $
define t_directions B_DOWN		1 << D_DOWN $
define t_directions B_ENTER		1 << D_ENTER $
define t_directions B_EXIT		1 << D_EXIT $
define t_directions B_HERE		1 << D_HERE $


/* Define the properties by which rooms (or pseudo-rooms or whatever) link
   in specific directions to each other. */

define t_directions p_lNorth		CreateThingProp()$
define t_directions p_lSouth		CreateThingProp()$
define t_directions p_lEast		CreateThingProp()$
define t_directions p_lWest		CreateThingProp()$
define t_directions p_lNorthEast	CreateThingProp()$
define t_directions p_lNorthWest	CreateThingProp()$
define t_directions p_lSouthEast	CreateThingProp()$
define t_directions p_lSouthWest	CreateThingProp()$
define t_directions p_lUp		CreateThingProp()$
define t_directions p_lDown		CreateThingProp()$
define t_directions p_lEnter		CreateThingProp()$
define t_directions p_lExit		CreateThingProp()$


/* Properties pointing to pictures for viewing in given directions. */

define t_directions p_lNorthPicture	CreateStringProp()$
define t_directions p_lNorthEastPicture	CreateStringProp()$
define t_directions p_lEastPicture	CreateStringProp()$
define t_directions p_lSouthEastPicture	CreateStringProp()$
define t_directions p_lSouthPicture	CreateStringProp()$
define t_directions p_lSouthWestPicture	CreateStringProp()$
define t_directions p_lWestPicture	CreateStringProp()$
define t_directions p_lNorthWestPicture	CreateStringProp()$
define t_directions p_lUpPicture	CreateStringProp()$
define t_directions p_lDownPicture	CreateStringProp()$
define t_directions p_lEnterPicture	CreateStringProp()$
define t_directions p_lExitPicture	CreateStringProp()$
define t_directions p_lHerePicture	CreateStringProp()$


/*
 * DirBack - yield the reverse of the passed direction.
 */

define t_directions proc DirBack(int dir)int:

    if dir < 4 then
	dir + 4
    elif dir < 8 then
	dir - 4
    else
	dir >< 1
    fi
corp;

/*
 * DirName - yield a string name for the direction. This form is suitable
 *	in places like "XXX arives from <dirname>".
 */

define t_directions proc utility public DirName(int dir)string:
    case dir
    incase D_NORTH:
	"the north"
    incase D_NORTHEAST:
	"the north-east"
    incase D_EAST:
	"the east"
    incase D_SOUTHEAST:
	"the south-east"
    incase D_SOUTH:
	"the south"
    incase D_SOUTHWEST:
	"the south-west"
    incase D_WEST:
	"the west"
    incase D_NORTHWEST:
	"the north-west"
    incase D_UP:
	"above"
    incase D_DOWN:
	"below"
    incase D_ENTER:
	"the inside"
    incase D_EXIT:
	"the outside"
    incase D_HERE:
	"here"
    default:
	""
    esac
corp;

/*
 * ExitName - the most direct mapping of directions to their names.
 */

define t_directions proc utility public ExitName(int dir)string:
    case dir
    incase D_NORTH:
	"north"
    incase D_NORTHEAST:
	"northeast"
    incase D_EAST:
	"east"
    incase D_SOUTHEAST:
	"southeast"
    incase D_SOUTH:
	"south"
    incase D_SOUTHWEST:
	"southwest"
    incase D_WEST:
	"west"
    incase D_NORTHWEST:
	"northwest"
    incase D_UP:
	"up"
    incase D_DOWN:
	"down"
    incase D_ENTER:
	"in"
    incase D_EXIT:
	"out"
    incase D_HERE:
	"here"
    default:
	""
    esac
corp;

/*
 * DirMatch - turn a string into a direction number. Return -1 if the string
 *	is not a valid direction name.
 */

define t_directions proc utility public DirMatch(string name)int:

    MatchName(
	"north,n.northeast,north-east,ne.east,e.southeast,south-east,se."
	"south,s.southwest,south-west,sw.west,w.northwest,north-west,nw."
	"up,u.down,d.enter,in,inside.exit,out,outside,leave.here", name)
corp;

/*
 * PairToDir - decode a character pair to a direction number.
 */

define t_directions proc utility public PairToDir(string ch)int:

    if ch == "n " then
	D_NORTH
    elif ch == "ne" then
	D_NORTHEAST
    elif ch == "e " then
	D_EAST
    elif ch == "se" then
	D_SOUTHEAST
    elif ch == "s " then
	D_SOUTH
    elif ch == "sw" then
	D_SOUTHWEST
    elif ch == "w " then
	D_WEST
    elif ch == "nw" then
	D_NORTHWEST
    elif ch == "u " then
	D_UP
    elif ch == "d " then
	D_DOWN
    elif ch == "in" then
	D_ENTER
    elif ch == "ex" then
	D_EXIT
    else
	-1
    fi
corp;

/*
 * DirProp - given the direction, return the linking property.
 */

define t_directions proc DirProp(int dir)property thing:
    case dir
    incase D_NORTH:
	p_lNorth
    incase D_NORTHEAST:
	p_lNorthEast
    incase D_EAST:
	p_lEast
    incase D_SOUTHEAST:
	p_lSouthEast
    incase D_SOUTH:
	p_lSouth
    incase D_SOUTHWEST:
	p_lSouthWest
    incase D_WEST:
	p_lWest
    incase D_NORTHWEST:
	p_lNorthWest
    incase D_UP:
	p_lUp
    incase D_DOWN:
	p_lDown
    incase D_ENTER:
	p_lEnter
    incase D_EXIT:
	p_lExit
    default:
	Print("*** invalid direction number ");
	IPrint(dir);
	Print(" to 'DirProp' ***\n");
	p_lNorth
    esac
corp;

/*
 * DirPictureProp - given a direction, return the property that would
 *	give the name of a picture for that direction, if any.
 */

define t_directions proc DirPictureProp(int dir)property string:
    case dir
    incase D_NORTH:
	p_lNorthPicture
    incase D_NORTHEAST:
	p_lNorthEastPicture
    incase D_EAST:
	p_lEastPicture
    incase D_SOUTHEAST:
	p_lSouthEastPicture
    incase D_SOUTH:
	p_lSouthPicture
    incase D_SOUTHWEST:
	p_lSouthWestPicture
    incase D_WEST:
	p_lWestPicture
    incase D_NORTHWEST:
	p_lNorthWestPicture
    incase D_UP:
	p_lUpPicture
    incase D_DOWN:
	p_lDownPicture
    incase D_ENTER:
	p_lEnterPicture
    incase D_EXIT:
	p_lExitPicture
    incase D_HERE:
	p_lHerePicture
    default:
	Print("*** invalid direction number ");
	IPrint(dir);
	Print(" to 'DirPictureProp' ***\n");
	p_lNorthPicture
    esac
corp;
