/*
 * Amiga MUD
 *
 * Copyright (c) 1999 by Chris Gray
 */

/*
 * Common/graphics.m - general graphics stuff.
 */

public t_graphics CreateTable()$
use t_graphics

private tp_graphics CreateTable()$
use tp_graphics


define tp_graphics GraphicsThing CreateThing(nil)$
define tp_graphics p_ColourMap CreateIntListProp()$

define tp_graphics proc initColourMap()void:
    list int li;

    li := CreateIntArray(32);
    GraphicsThing@p_ColourMap := li;
    li[ 0] := C_BLACK;
    li[ 1] := C_DARK_GREY;
    li[ 2] := C_MEDIUM_GREY;
    li[ 3] := C_LIGHT_GREY;
    li[ 4] := C_WHITE;
    li[ 5] := C_BRICK_RED;
    li[ 6] := C_RED;
    li[ 7] := C_RED_ORANGE;
    li[ 8] := C_ORANGE;
    li[ 9] := C_GOLD;
    li[10] := C_CADMIUM_YELLOW;
    li[11] := C_LEMON_YELLOW;
    li[12] := C_LIME_GREEN;
    li[13] := C_GREEN;
    li[14] := C_LIGHT_GREEN;
    li[15] := C_DARK_GREEN;
    li[16] := C_FOREST_GREEN;
    li[17] := C_BLUE_GREEN;
    li[18] := C_AQUA;
    li[19] := C_LIGHT_AQUA;
    li[20] := C_SKY_BLUE;
    li[21] := C_LIGHT_BLUE;
    li[22] := C_BLUE;
    li[23] := C_DARK_BLUE;
    li[24] := C_VIOLET;
    li[25] := C_PURPLE;
    li[26] := C_FLESH;
    li[27] := C_PINK;
    li[28] := C_TAN;
    li[29] := C_BROWN;
    li[30] := C_MEDIUM_BROWN;
    li[31] := C_DARK_BROWN;
corp;

initColourMap()$

/*
 * ColourMatch - a routine to map a colour name to a colour number at run-time.
 */

define t_graphics proc public ColourMatch(string name)int:
    int i;

    if name == "grey" then
	C_MEDIUM_GREY
    elif name == "red" then
	C_RED
    elif name == "green" then
	C_GREEN
    elif name == "blue" then
	C_BLUE
    else
	i := MatchName(
	    "black.grey;dark.grey;medium.grey;light.white.red;brick.red."
	    "red-orange.orange.gold.yellow;cadmium.yellow;lemon.green;lime."
	    "green.green;light.green;dark.green;forest.blue-green.aqua."
	    "aqua;light.blue;sky.blue;light.blue.blue;dark.violet.purple."
	    "flesh.pink.tan.brown.brown;medium.brown;dark", name);
	if i ~= -1 then
	    GraphicsThing@p_ColourMap[i]
	else
	    -1
	fi
    fi
corp;

/*
 * ColourName - map from colour number to colour name
 */

define t_graphics proc public ColourName(int colour)string:

    case colour
    incase C_BLACK:		"black"
    incase C_DARK_GREY: 	"dark grey"
    incase C_MEDIUM_GREY:	"medium grey"
    incase C_LIGHT_GREY:	"light grey"
    incase C_WHITE:		"white"
    incase C_BRICK_RED: 	"brick red"
    incase C_RED:		"red"
    incase C_RED_ORANGE:	"red-orange"
    incase C_ORANGE:		"orange"
    incase C_GOLD:		"gold"
    incase C_CADMIUM_YELLOW:	"cadmium yellow"
    incase C_LEMON_YELLOW:	"lemon yellow"
    incase C_LIME_GREEN:	"lime green"
    incase C_GREEN:		"green"
    incase C_LIGHT_GREEN:	"light green"
    incase C_DARK_GREEN:	"dark green"
    incase C_FOREST_GREEN:	"forest green"
    incase C_BLUE_GREEN:	"blue-green"
    incase C_AQUA:		"aqua"
    incase C_LIGHT_AQUA:	"light aqua"
    incase C_SKY_BLUE:		"sky blue"
    incase C_LIGHT_BLUE:	"light blue"
    incase C_BLUE:		"blue"
    incase C_DARK_BLUE: 	"dark blue"
    incase C_VIOLET:		"violet"
    incase C_PURPLE:		"purple"
    incase C_FLESH:		"flesh"
    incase C_PINK:		"pink"
    incase C_TAN:		"tan"
    incase C_BROWN:		"brown"
    incase C_MEDIUM_BROWN:	"medium brown"
    incase C_DARK_BROWN:	"dark brown"
    default:			"<unnamed-colour>"
    esac
corp;

/*
 * ShowKnownColours - print the colour names that we understand.
 */

define t_graphics proc public ShowKnownColours()void:
    list int li;
    int i;

    li := GraphicsThing@p_ColourMap;
    Print("Known colours are: ");
    for i from 0 upto 30 do
	Print(ColourName(li[i]));
	Print(", ");
    od;
    Print(ColourName(li[31]));
    Print("\n");
corp;



/* Code to provide a generator for unique ids for effects. This is
   actually used at world sourcing time, not at client runtime. */

define tp_graphics p_NextEffectId CreateIntProp()$
GraphicsThing@p_NextEffectId := 10$	/* skip ones used in gui.m */

define t_graphics proc public NextEffectId()int:
    int id;

    id := GraphicsThing@p_NextEffectId;
    GraphicsThing@p_NextEffectId := id + 1;
    id
corp;


/* Deal with the standard clickable region that enables clicking near the
   character cursor for movement. */

define t_graphics MOVE_REGION	1000$	/* want it large, so others override */

define t_graphics proc public AddStandardRegions()void:

    AddRegionPixels(0, 0, DRAW_COLS, DRAW_ROWS, MOVE_REGION);
corp;

define t_graphics proc public EraseStandardRegions()void:

    EraseRegion(MOVE_REGION);
corp;

/*
 * RoomName - set the graphics name of the room.
 */

define tp_graphics proc public utility LocationName(thing location;
    string name1, name2)void:

    location@p_lName1 := name1;
    if name2 ~= "" then
	location@p_lName2 := name2;
    else
	location -- p_lName2;
    fi;
corp;

/*
 * DoDrawLocationName - general utility to draw the name of the given
 *	location. This can be a one or two line explicit name, or a
 *	name derived from the location's short name.
 */

define t_graphics proc public utility DoDrawLocationName(thing location)void:
    int len, pos;
    string name;
    action a;

    if location@p_lName1 ~= "" then
	DrawLocationName(location@p_lName1, location@p_lName2);
    else
	a := location@p_lNameAction;
	if a ~= nil then
	    name := call(a, string)();
	else
	    name := location@p_lName;
	fi;
	len := Length(name);
	pos := len - 1;
	while pos ~= 0 and SubString(name, pos, 1) ~= " " do
	    pos := pos - 1;
	od;
	if SubString(name, pos, 1) = " " then
	    pos := pos + 1;
	fi;
	DrawLocationName(Capitalize(SubString(name, pos, len - pos)), "");
    fi;
corp;

define t_graphics proc utility public ClearTo(int colour)void:

    GSetPen(nil, colour);
    GAMovePixels(nil, 0, 0);
    GRectanglePixels(nil, DRAW_COLS, DRAW_ROWS, true);
corp;


unuse tp_graphics
