/*
 * Amiga MUD
 *
 * Copyright (c) 1999 by Chris Gray
 */

/*
 * Common/gui.m - code for dealing with GUI frames, components, etc.
 */

public t_gui CreateTable()$
use t_gui

private tp_gui CreateTable()$
use tp_gui

/* Types of components */

define tp_gui CTYPE_PROMPTED_TEXT	1$
define tp_gui CTYPE_GENERAL_TEXT	2$
define tp_gui CTYPE_ICON_LIST		3$
define tp_gui CTYPE_ICONED_CANVAS	4$
define tp_gui CTYPE_BUTTON_PANEL	5$

/* Note: Don calls the general drawing components 'iconed canvases'. I have
   been unable to change his mind on that. No icons, in the cgMUD sense,
   are drawn on them, but tiles in the cgMUD sense can be drawn on them.
   I think he uses the same innards for icons as well as tiles, so hence
   his choice of name. However, since there isn't a component which you
   can do general drawing/picture displaying on that cannot be used for
   tiles, just calling the component a 'drawing canvas' would be less
   confusing, I think. */


/* Containers and components for our two styles of GUI */

define tp_gui CONTAINER_MAIN		100$
define tp_gui CONTAINER_TOP		101$
define tp_gui CONTAINER_MBUTTONS	102$
define tp_gui CONTAINER_DRAW		103$
define tp_gui CONTAINER_PICTURE		104$
define tp_gui CONTAINER_BUTTONS		105$
define tp_gui CONTAINER_PBUTTONS	107$

define tp_gui COMPONENT_TEXT		1$
	define t_gui TEXT_COLS			80$
	define t_gui TEXT_ROWS			18$
define tp_gui COMPONENT_INPUT		2$
	define tp_gui COLS_INPUT		TEXT_COLS + 4$
define t_gui COMPONENT_DRAW		3$
	define t_gui DRAW_COLS			320$
	define t_gui DRAW_ROWS			256$
define t_gui COMPONENT_PICTURE		4$
	define tp_gui WIDTH_PICTURE		320$
	define tp_gui HEIGHT_PICTURE		240$
define tp_gui COMPONENT_ROOM_NAME	5$
	define tp_gui HEIGHT_ROOM_NAME_OLD	50$
	define tp_gui HEIGHT_ROOM_NAME_NEW	90$
	define tp_gui WIDTH_ROOM_NAME		320$
define t_gui COMPONENT_ICONS		6$
	define tp_gui WIDTH_ICONS		WIDTH_ROOM_NAME$
	define tp_gui HEIGHT_ICONS		60$
define t_gui COMPONENT_MBUTTONS		7$
define tp_gui COMPONENT_MBUTTONS_LABEL	8$
define tp_gui COMPONENT_PBUTTONS	9$
define tp_gui COMPONENT_PBUTTONS_LABEL	10$
	define t_gui WIDTH_BUTTONS_OLD		WIDTH_ROOM_NAME$
	define t_gui WIDTH_BUTTONS_NEW		WIDTH_PICTURE / 2 - 4$
	define tp_gui HEIGHT_BUTTONS		80$
	define tp_gui HEIGHT_LABEL		18$


/* Which style of GUI is active for the character */

define tp_gui p_pGUIFrameType CreateIntProp()$
	define tp_gui GUI_FRAME_OLD		1$
	define tp_gui GUI_FRAME_PICTURE		2$


/* Minimum values, used for drawing objects and characters */

define t_gui MIN_COLS		if DRAW_COLS < WIDTH_PICTURE then
				    DRAW_COLS else WIDTH_PICTURE fi$
define t_gui MIN_ROWS		if DRAW_ROWS < HEIGHT_PICTURE then
				    DRAW_ROWS else HEIGHT_PICTURE fi$


/* A standard set of colours */

define t_gui C_BLACK		000$
define t_gui C_DARK_GREY	252$
define t_gui C_MEDIUM_GREY	253$
define t_gui C_LIGHT_GREY	254$
define t_gui C_WHITE		251$
define t_gui C_BRICK_RED	168$
define t_gui C_RED		210$
define t_gui C_RED_ORANGE	222$
define t_gui C_ORANGE		228$
define t_gui C_GOLD		234$
define t_gui C_CADMIUM_YELLOW	240$
define t_gui C_LEMON_YELLOW	246$
define t_gui C_LIME_GREEN	204$
define t_gui C_GREEN		036$
define t_gui C_LIGHT_GREEN	164$
define t_gui C_DARK_GREEN	018$
define t_gui C_FOREST_GREEN	024$
define t_gui C_BLUE_GREEN	040$
define t_gui C_AQUA		034$
define t_gui C_LIGHT_AQUA	041$
define t_gui C_SKY_BLUE		029$
define t_gui C_LIGHT_BLUE	155$
define t_gui C_BLUE		005$
define t_gui C_DARK_BLUE	003$
define t_gui C_VIOLET		215$
define t_gui C_PURPLE		129$
define t_gui C_FLESH		237$
define t_gui C_PINK		238$
define t_gui C_TAN		194$
define t_gui C_BROWN		186$
define t_gui C_MEDIUM_BROWN	180$
define t_gui C_DARK_BROWN	138$


/*
 * GUIFrameOld - create a frame of the old style (one main drawing/picture
 *	area, with icon panel).
 */

define t_gui proc public GUIFrameOld()void:

    CreateContainer(0, CONTAINER_MAIN, true, 0);
    CreateContainer(CONTAINER_MAIN, CONTAINER_TOP, false, 0);
    CreateComponent(CONTAINER_TOP,
		    COMPONENT_DRAW, DRAW_COLS, DRAW_ROWS,
		    CTYPE_ICONED_CANVAS);
    CreateContainer(CONTAINER_TOP, CONTAINER_MBUTTONS, true, 0);
    CreateComponent(CONTAINER_MBUTTONS,
		    COMPONENT_ROOM_NAME, WIDTH_ROOM_NAME, HEIGHT_ROOM_NAME_OLD,
		    CTYPE_ICONED_CANVAS);
    CreateComponent(CONTAINER_MBUTTONS,
		    COMPONENT_MBUTTONS, WIDTH_BUTTONS_OLD, HEIGHT_BUTTONS,
		    CTYPE_BUTTON_PANEL);
    CreateComponent(CONTAINER_MBUTTONS,
		    COMPONENT_ICONS, WIDTH_ICONS, HEIGHT_ICONS,
		    CTYPE_ICON_LIST);
    CreateComponent(CONTAINER_MAIN,
		    COMPONENT_TEXT, TEXT_COLS, TEXT_ROWS,
		    CTYPE_GENERAL_TEXT);
    CreateComponent(CONTAINER_MAIN,
		    COMPONENT_INPUT, COLS_INPUT, 1, CTYPE_PROMPTED_TEXT);

    MakeFrame();
    Me()@p_pGUIFrameType := GUI_FRAME_OLD;
    ignore TextComponent(nil, COMPONENT_TEXT);
    ignore DrawComponent(nil, COMPONENT_DRAW);
corp;

/*
 * GUIFramePicture - create a frame of the new style (two drawing/picture
 *	areas, no icon panel).
 */

define t_gui proc public GUIFramePicture()void:

    CreateContainer(0, CONTAINER_MAIN, true, 0);
    CreateContainer(CONTAINER_MAIN, CONTAINER_TOP, false, 0);
    CreateContainer(CONTAINER_TOP, CONTAINER_DRAW, true, 0);
    CreateComponent(CONTAINER_DRAW,
		    COMPONENT_DRAW, DRAW_COLS, DRAW_ROWS,
		    CTYPE_ICONED_CANVAS);
    CreateComponent(CONTAINER_DRAW,
		    COMPONENT_ROOM_NAME, DRAW_COLS, HEIGHT_ROOM_NAME_NEW,
		    CTYPE_ICONED_CANVAS);
    CreateContainer(CONTAINER_TOP, CONTAINER_PICTURE, true, 0);
    CreateComponent(CONTAINER_PICTURE,
		    COMPONENT_PICTURE, WIDTH_PICTURE, HEIGHT_PICTURE,
		    CTYPE_ICONED_CANVAS);
    CreateContainer(CONTAINER_PICTURE, CONTAINER_BUTTONS, false, 0);
    CreateContainer(CONTAINER_BUTTONS, CONTAINER_MBUTTONS, true, 0);
    CreateComponent(CONTAINER_MBUTTONS,
		    COMPONENT_MBUTTONS, WIDTH_BUTTONS_NEW, HEIGHT_BUTTONS,
		    CTYPE_BUTTON_PANEL);
    CreateComponent(CONTAINER_MBUTTONS,
		    COMPONENT_MBUTTONS_LABEL, WIDTH_BUTTONS_NEW, HEIGHT_LABEL,
		    CTYPE_ICONED_CANVAS);
    CreateContainer(CONTAINER_BUTTONS, CONTAINER_PBUTTONS, true, 0);
    CreateComponent(CONTAINER_PBUTTONS,
		    COMPONENT_PBUTTONS, WIDTH_BUTTONS_NEW, HEIGHT_BUTTONS,
		    CTYPE_BUTTON_PANEL);
    CreateComponent(CONTAINER_PBUTTONS,
		    COMPONENT_PBUTTONS_LABEL, WIDTH_BUTTONS_NEW, HEIGHT_LABEL,
		    CTYPE_ICONED_CANVAS);
    CreateComponent(CONTAINER_MAIN,
		    COMPONENT_TEXT, TEXT_COLS, TEXT_ROWS,
		    CTYPE_GENERAL_TEXT);
    CreateComponent(CONTAINER_MAIN,
		    COMPONENT_INPUT, COLS_INPUT, 1, CTYPE_PROMPTED_TEXT);

    MakeFrame();
    Me()@p_pGUIFrameType := GUI_FRAME_PICTURE;
    ignore TextComponent(nil, COMPONENT_TEXT);
    ignore DrawComponent(nil, COMPONENT_DRAW);
corp;


/*
 * getDrawingComponent - return the component to use for drawing.
 */

define t_gui proc public getDrawingComponent()int:
    if Me()@p_pGUIFrameType = GUI_FRAME_OLD then
	COMPONENT_DRAW
    else
	COMPONENT_PICTURE
    fi
corp;


/* Code to draw a location name box and its contents */

define tp_gui ROOM_NAME_BOX_ID_OLD	1$
define tp_gui ROOM_NAME_BOX_ID_NEW	2$
define tp_gui NAME_BOX_COLS		20$

define t_gui proc public DrawLocationNameBox()void:
    int saveComponent, id, rows, cols, boxRows, boxCols, i;

    saveComponent := DrawComponent(nil, COMPONENT_ROOM_NAME);
    if Me()@p_pGUIFrameType = GUI_FRAME_OLD then
	id := ROOM_NAME_BOX_ID_OLD;
	rows := HEIGHT_ROOM_NAME_OLD;
    else
	id := ROOM_NAME_BOX_ID_NEW;
	rows := HEIGHT_ROOM_NAME_NEW;
    fi;
    cols := WIDTH_ROOM_NAME;
    if not KnowsEffect(nil, id) then
	boxCols := NAME_BOX_COLS * GetFontWidth(nil) + 20;
	boxRows := 2 * GetFontHeight(nil) + 20;
	DefineEffect(nil, id);
	GAMovePixels(nil, (cols - boxCols) / 2, (rows - boxRows) / 2);
	for i from 0 upto 3 do
	    GSetPen(nil, C_DARK_GREY + i);
	    GRectanglePixels(nil, boxCols - 4 * i, boxRows - 4 * i, true);
	    GRMovePixels(nil, 2, 2);
	od;
	EndEffect();
    fi;
    CallEffect(nil, id);
    ignore DrawComponent(nil, saveComponent);
corp;

define t_gui proc public DrawLocationName(string s1, s2)void:
    int saveComponent, rows, cols, boxRows, boxCols, len1, len2, row, col;
    int fontAscent, fontHeight, fontWidth;

    fontAscent := GetFontAscent(nil);
    fontHeight := GetFontHeight(nil);
    fontWidth := GetFontWidth(nil);
    saveComponent := DrawComponent(nil, COMPONENT_ROOM_NAME);
    if Me()@p_pGUIFrameType = GUI_FRAME_OLD then
	rows := HEIGHT_ROOM_NAME_OLD;
    else
	rows := HEIGHT_ROOM_NAME_NEW;
    fi;
    cols := WIDTH_ROOM_NAME;
    /* These are the outer box dimensions, including the box itself. */
    boxCols := NAME_BOX_COLS * fontWidth + 20;
    boxRows := 2 * fontHeight + 20;
    row := (rows - boxRows) / 2;
    col := (cols - boxCols) / 2;
    GSetPen(nil, C_BLACK);
    GAMovePixels(nil, col + 8, row + 8);
    GRectanglePixels(nil, boxCols - 16, boxRows - 16, true);
    GSetPen(nil, C_GOLD);
    len1 := Length(s1);
    if len1 > NAME_BOX_COLS then
	len1 := NAME_BOX_COLS;
	s1 := SubString(s1, 0, NAME_BOX_COLS);
    fi;
    len2 := Length(s2);
    if len2 ~= 0 then
	if len2 > NAME_BOX_COLS then
	    len2 := NAME_BOX_COLS;
	    s2 := SubString(s2, 0, NAME_BOX_COLS);
	fi;
	GAMovePixels(nil,
		     col + 10 + (NAME_BOX_COLS - len2) * fontWidth / 2,
		     row + 10 + fontHeight + fontAscent);
	GText(nil, s2);
	GAMovePixels(nil,
		     col + 10 + (NAME_BOX_COLS - len1) * fontWidth / 2,
		     row + 10 + fontAscent);
	GText(nil, s1);
    else
	GAMovePixels(nil,
		     col + 10 + (NAME_BOX_COLS - len1) * fontWidth / 2,
		     row + 10 + fontHeight / 2 + fontAscent);
	GText(nil, s1);
    fi;
    ignore DrawComponent(nil, saveComponent);
corp;


/* Key codes sent by the client for raw key events */

define t_gui KEY_HELP		0x0020$
define t_gui KEY_KP_UL		0x0001$
define t_gui KEY_KP_U		0x0002$
define t_gui KEY_KP_UR		0x0003$
define t_gui KEY_KP_L		0x0004$
define t_gui KEY_KP_C		0x0005$
define t_gui KEY_KP_R		0x0006$
define t_gui KEY_KP_DL		0x0007$
define t_gui KEY_KP_D		0x0008$
define t_gui KEY_KP_DR		0x0009$
define t_gui KEY_KP_PLUS	0x000a$
define t_gui KEY_KP_MINUS	0x000b$


/* Code to put up a standard set of direction buttons in the MBUTTON area */

define t_gui BUTTON_D_NW	1$
define t_gui BUTTON_D_N		2$
define t_gui BUTTON_D_NE	3$
define t_gui BUTTON_D_W		4$
define t_gui BUTTON_D_L		5$
define t_gui BUTTON_D_E		6$
define t_gui BUTTON_D_SW	7$
define t_gui BUTTON_D_S		8$
define t_gui BUTTON_D_SE	9$
define t_gui BUTTON_D_D		10$
define t_gui BUTTON_D_U		11$
define t_gui BUTTON_D_I		12$
define t_gui BUTTON_D_O		13$

/*
 * DirButton - given a direction, return the direction button id.
 */

define t_gui proc DirButton(int dir)int:

    case dir
    incase D_NORTH:
	BUTTON_D_N
    incase D_NORTHEAST:
	BUTTON_D_NE
    incase D_EAST:
	BUTTON_D_E
    incase D_SOUTHEAST:
	BUTTON_D_SE
    incase D_SOUTH:
	BUTTON_D_S
    incase D_SOUTHWEST:
	BUTTON_D_SW
    incase D_WEST:
	BUTTON_D_W
    incase D_NORTHWEST:
	BUTTON_D_NW
    incase D_UP:
	BUTTON_D_U
    incase D_DOWN:
	BUTTON_D_D
    incase D_ENTER:
	BUTTON_D_I
    incase D_EXIT:
	BUTTON_D_O
    incase D_HERE:
	BUTTON_D_L
    default:
	Print("*** invalid direction number ");
	IPrint(dir);
	Print(" to 'DirButton' ***\n");
	BUTTON_D_I
    esac
corp;

/*
 * ButtonDir - given a direction button id, return the direction.
 */

define t_gui proc ButtonDir(int whichButton)int:

    case whichButton
    incase BUTTON_D_N:
	D_NORTH
    incase BUTTON_D_NE:
	D_NORTHEAST
    incase BUTTON_D_E:
	D_EAST
    incase BUTTON_D_SE:
	D_SOUTHEAST
    incase BUTTON_D_S:
	D_SOUTH
    incase BUTTON_D_SW:
	D_SOUTHWEST
    incase BUTTON_D_W:
	D_WEST
    incase BUTTON_D_NW:
	D_NORTHWEST
    incase BUTTON_D_U:
	D_UP
    incase BUTTON_D_D:
	D_DOWN
    incase BUTTON_D_I:
	D_ENTER
    incase BUTTON_D_O:
	D_EXIT
    incase BUTTON_D_L:
	D_HERE
    default:
	Print("*** invalid button number ");
	IPrint(whichButton);
	Print(" to 'ButtonDir' ***\n");
	D_NORTH
    esac
corp;

define t_gui DIR_BUTTONS_ORTHOGONAL	0x01$
define t_gui DIR_BUTTONS_NON_ORTHOGONAL	0x02$
define t_gui DIR_BUTTONS_IN_OUT		0x04$
define t_gui DIR_BUTTONS_UP_DOWN	0x08$
define t_gui DIR_BUTTONS_LOOK		0x10$
define t_gui DIR_BUTTONS_ALL		0x1f$

/*
 * DirButtonsBaseRow - return the pixel row within the button component
 *	for the top of the uppermost normal direction button.
 */

define t_gui proc public DirButtonsBaseRow()int:

    (HEIGHT_BUTTONS - (GetFontHeight(nil) * 4 + 4 * 4 + 2 * 3)) / 2
corp;

/*
 * DirButtonsBaseCol - return the pixel column within the button component
 *	for the left of the leftmost normal direction button.
 */

define t_gui proc public DirButtonsBaseCol()int:
    int cols;

    if Me()@p_pGUIFrameType = GUI_FRAME_OLD then
	cols := WIDTH_BUTTONS_OLD;
    else
	cols := WIDTH_BUTTONS_NEW;
    fi;
    (cols - (GetFontWidth(nil) * 7 + 8 * 5 + 2 * 4)) / 2
corp;

define tp_gui ADD_DIR_BUTTONS_ID_OLD	3$
define tp_gui ADD_DIR_BUTTONS_ID_NEW	4$

/*
 * AddDirectionButtons - add the normal set of direction buttons to the
 *	MBUTTONS component. 'bits' describes which of the possible buttons
 *	to draw. See the DIR_BUTTONS_XXX values above.
 */

define t_gui proc public AddDirectionButtons(int bits)void:
    int saveComponent, id, row, col, fontHeight, fontWidth;
    bool defining;

    fontHeight := GetFontHeight(nil);
    fontWidth := GetFontWidth(nil);
    if bits = DIR_BUTTONS_ALL then
	id :=
	    if Me()@p_pGUIFrameType = GUI_FRAME_OLD then
		ADD_DIR_BUTTONS_ID_OLD
	    else
		ADD_DIR_BUTTONS_ID_NEW
	    fi;
    else
	id := 0;
    fi;
    saveComponent := DrawComponent(nil, COMPONENT_MBUTTONS);
    row := DirButtonsBaseRow();
    col := DirButtonsBaseCol();

    defining := false;
    if id ~= 0 and not KnowsEffect(nil, id) then
	DefineEffect(nil, id);
	defining := true;
    fi;

    if id = 0 or defining then
	if bits & DIR_BUTTONS_ORTHOGONAL ~= 0 then
	    AddButtonPixels(col + fontWidth * 3 + 8 * 2 + 2 * 2, row,
			    BUTTON_D_N, "N");
	    AddButtonPixels(col + fontWidth * 2 + 8 + 2,
			    row + fontHeight + 4 + 2,
			    BUTTON_D_W, "W");
	    AddButtonPixels(col + fontWidth * 4 + 8 * 3 + 2 * 3,
			    row + fontHeight + 4 + 2,
			    BUTTON_D_E, "E");
	    AddButtonPixels(col + fontWidth * 3 + 8 * 2 + 2 * 2,
			    row + fontHeight * 2 + 4 * 2 + 2 * 2,
			    BUTTON_D_S, "S");
	fi;

	if bits & DIR_BUTTONS_NON_ORTHOGONAL ~= 0 then
	    AddButtonPixels(col + fontWidth + 8 + 2, row,
			    BUTTON_D_NW, "NW");
	    AddButtonPixels(col + fontWidth * 4 + 8 * 3 + 2 * 3, row,
			    BUTTON_D_NE, "NE");

	    AddButtonPixels(col + fontWidth + 8 + 2,
			    row + fontHeight * 2 + 4 * 2 + 2 * 2,
			    BUTTON_D_SW, "SW");
	    AddButtonPixels(col + fontWidth * 4 + 8 * 3 + 2 * 3,
			    row + fontHeight * 2 + 4 * 2 + 2 * 2,
			    BUTTON_D_SE, "SE");
	fi;

	if bits & DIR_BUTTONS_IN_OUT ~= 0 then
	    AddButtonPixels(col, row + (fontHeight + 6) / 2, BUTTON_D_I, "I");
	    AddButtonPixels(col, row + (fontHeight + 6) * 3 / 2,
			    BUTTON_D_O, "O");
	fi;

	if bits & DIR_BUTTONS_UP_DOWN ~= 0 then
	    AddButtonPixels(col + fontWidth * 6 + 8 * 4 + 2 * 4,
			    row + (fontHeight + 6) / 2,
			    BUTTON_D_U, "U");
	    AddButtonPixels(col + fontWidth * 6 + 8 * 4 + 2 * 4,
			    row + (fontHeight + 6) * 3 / 2,
			    BUTTON_D_D, "D");
	fi;

	if bits & DIR_BUTTONS_LOOK ~= 0 then
	    AddButtonPixels(col + fontWidth * 3 + 8 * 2 + 2 * 2,
			    row + fontHeight + 4 + 2,
			    BUTTON_D_L, "L");
	fi;
    fi;

    if id ~= 0 then
	if defining then
	    EndEffect();
	fi;
	CallEffect(nil, id);
    fi;

    ignore DrawComponent(nil, saveComponent);
corp;

define tp_gui ERASE_DIR_BUTTONS_ID	5$

/*
 * EraseDirectionButtons - erase direction buttons as added above. The
 *	'bits' mask plays the same role.
 */

define t_gui proc public EraseDirectionButtons(int bits)void:
    int saveComponent;
    bool defining;

    saveComponent := DrawComponent(nil, COMPONENT_MBUTTONS);
    defining := false;
    if bits = DIR_BUTTONS_ALL and not KnowsEffect(nil, ERASE_DIR_BUTTONS_ID)
    then
	DefineEffect(nil, ERASE_DIR_BUTTONS_ID);
	defining := true;
    fi;

    if bits ~= DIR_BUTTONS_ALL or defining then
	if bits & DIR_BUTTONS_ORTHOGONAL ~= 0 then
	    EraseButton(BUTTON_D_N);
	    EraseButton(BUTTON_D_W);
	    EraseButton(BUTTON_D_E);
	    EraseButton(BUTTON_D_S);
	fi;

	if bits & DIR_BUTTONS_NON_ORTHOGONAL ~= 0 then
	    EraseButton(BUTTON_D_NW);
	    EraseButton(BUTTON_D_NE);
	    EraseButton(BUTTON_D_SW);
	    EraseButton(BUTTON_D_SE);
	fi;

	if bits & DIR_BUTTONS_IN_OUT ~= 0 then
	    EraseButton(BUTTON_D_I);
	    EraseButton(BUTTON_D_O);
	fi;

	if bits & DIR_BUTTONS_UP_DOWN ~= 0 then
	    EraseButton(BUTTON_D_U);
	    EraseButton(BUTTON_D_D);
	fi;

	if bits & DIR_BUTTONS_LOOK ~= 0 then
	    EraseButton(BUTTON_D_L);
	fi;
    fi;

    if bits = DIR_BUTTONS_ALL then
	if defining then
	    EndEffect();
	fi;
	CallEffect(nil, ERASE_DIR_BUTTONS_ID);
    fi;

    ignore DrawComponent(nil, saveComponent);
corp;


/* Code to put up picture buttons in the PBUTTON area */

define t_gui BUTTON_P_NW	21$
define t_gui BUTTON_P_N		22$
define t_gui BUTTON_P_NE	23$
define t_gui BUTTON_P_W		24$
define t_gui BUTTON_P_H		25$
define t_gui BUTTON_P_E		26$
define t_gui BUTTON_P_SW	27$
define t_gui BUTTON_P_S		28$
define t_gui BUTTON_P_SE	29$
define t_gui BUTTON_P_D		30$
define t_gui BUTTON_P_U		31$
define t_gui BUTTON_P_I		32$
define t_gui BUTTON_P_O		33$

/*
 * DirPictureButton - given a direction, return the id of the picture
 *	button for that direction.
 */

define t_gui proc DirPictureButton(int dir)int:

    case dir
    incase D_NORTH:
	BUTTON_P_N
    incase D_NORTHEAST:
	BUTTON_P_NE
    incase D_EAST:
	BUTTON_P_E
    incase D_SOUTHEAST:
	BUTTON_P_SE
    incase D_SOUTH:
	BUTTON_P_S
    incase D_SOUTHWEST:
	BUTTON_P_SW
    incase D_WEST:
	BUTTON_P_W
    incase D_NORTHWEST:
	BUTTON_P_NW
    incase D_UP:
	BUTTON_P_U
    incase D_DOWN:
	BUTTON_P_D
    incase D_ENTER:
	BUTTON_P_I
    incase D_EXIT:
	BUTTON_P_O
    incase D_HERE:
	BUTTON_P_H
    default:
	Print("*** invalid direction number ");
	IPrint(dir);
	Print(" to 'DirPictureButton' ***\n");
	BUTTON_P_I
    esac
corp;

/*
 * AddPictureButtons - possibly add some picture buttons to the
 *	PBUTTONS component. 'th' is a thing to examine for the various
 *	'p_rXXXPicture' properties to see which buttons to draw.
 */

define t_gui proc public AddPictureButtons(thing th)void:
    int saveComponent, row, col, fontHeight, fontWidth;

    fontHeight := GetFontHeight(nil);
    fontWidth := GetFontWidth(nil);
    saveComponent := DrawComponent(nil, COMPONENT_PBUTTONS);
    row := DirButtonsBaseRow();
    col := DirButtonsBaseCol();

    if th@p_lEnterPicture ~= "" then
	AddButtonPixels(col, row + (fontHeight + 10) / 2, BUTTON_P_I, "I");
    fi;
    if th@p_lExitPicture ~= "" then
	AddButtonPixels(col, row + (fontHeight + 10) * 3 / 2, BUTTON_P_O,"O");

    fi;
    if th@p_lNorthWestPicture ~= "" then
	AddButtonPixels(col + fontWidth + 8 + 2, row,
			BUTTON_P_NW, "NW");
    fi;
    if th@p_lNorthPicture ~= "" then
	AddButtonPixels(col + fontWidth * 3 + 8 * 2 + 2 * 2, row,
			BUTTON_P_N, "N");
    fi;
    if th@p_lNorthEastPicture ~= "" then
	AddButtonPixels(col + fontWidth * 4 + 8 * 3 + 2 * 3, row,
			BUTTON_P_NE, "NE");
    fi;

    if th@p_lWestPicture ~= "" then
	AddButtonPixels(col + fontWidth * 2 + 8 + 2,
			row + fontHeight + 4 + 2,
			BUTTON_P_W, "W");
    fi;
    if th@p_lHerePicture ~= "" then
	AddButtonPixels(col + fontWidth * 3 + 8 * 2 + 2 * 2,
			row + fontHeight + 4 + 2,
			BUTTON_P_H, "H");
    fi;
    if th@p_lEastPicture ~= "" then
	AddButtonPixels(col + fontWidth * 4 + 8 * 3 + 2 * 3,
			row + fontHeight + 4 + 2,
			BUTTON_P_E, "E");
    fi;

    if th@p_lSouthWestPicture ~= "" then
	AddButtonPixels(col + fontWidth + 8 + 2,
			row + fontHeight * 2 + 4 * 2 + 2 * 2,
			BUTTON_P_SW, "SW");
    fi;
    if th@p_lSouthPicture ~= "" then
	AddButtonPixels(col + fontWidth * 3 + 8 * 2 + 2 * 2,
			row + fontHeight * 2 + 4 * 2 + 2 * 2,
			BUTTON_P_S, "S");
    fi;
    if th@p_lSouthEastPicture ~= "" then
	AddButtonPixels(col + fontWidth * 4 + 8 * 3 + 2 * 3,
			row + fontHeight * 2 + 4 * 2 + 2 * 2,
			BUTTON_P_SE, "SE");
    fi;

    if th@p_lUpPicture ~= "" then
	AddButtonPixels(col + fontWidth * 6 + 8 * 4 + 2 * 4,
			row + (fontHeight + 6) / 2,
			BUTTON_P_U, "U");
    fi;
    if th@p_lDownPicture ~= "" then
	AddButtonPixels(col + fontWidth * 6 + 8 * 4 + 2 * 4,
			row + (fontHeight + 6) * 3 / 2,
			BUTTON_P_D, "D");
    fi;

    ignore DrawComponent(nil, saveComponent);
corp;

/*
 * ErasePictureButtons - erase picture-display buttons from the PBUTTONS
 *	component, as drawn above. 'th' plays the same role as above.
 */

define t_gui proc ErasePictureButtons(thing th)void:
    int saveComponent, dir;

    saveComponent := DrawComponent(nil, COMPONENT_PBUTTONS);
    for dir from D_NORTH upto D_HERE do
	if th@(DirPictureProp(dir)) ~= "" then
	    EraseButton(DirPictureButton(dir));
	fi;
    od;
    ignore DrawComponent(nil, saveComponent);
corp;


/* Export some routines that allow the labelling of the button areas. */

define t_gui proc public LabelMButtons(string label; int colour)void:
    int saveComponent, len;

    saveComponent := DrawComponent(nil, COMPONENT_MBUTTONS_LABEL);    
    len := Length(label);
    GAMovePixels(nil, (WIDTH_BUTTONS_NEW - len * GetFontWidth(nil)) / 2,
		      (HEIGHT_LABEL - GetFontHeight(nil)) / 2 +
		      GetFontAscent(nil));
    GSetPen(nil, colour);
    GText(nil, label);
    ignore DrawComponent(nil, saveComponent);
corp;

define t_gui proc public LabelPButtons(string label; int colour)void:
    int saveComponent, len;

    saveComponent := DrawComponent(nil, COMPONENT_PBUTTONS_LABEL);    
    len := Length(label);
    GAMovePixels(nil, (WIDTH_BUTTONS_NEW - len * GetFontWidth(nil)) / 2,
		      (HEIGHT_LABEL - GetFontHeight(nil)) / 2 +
		      GetFontAscent(nil));
    GSetPen(nil, colour);
    GText(nil, label);
    ignore DrawComponent(nil, saveComponent);
corp;


/*
 * AddButtonRow - a routine to help in putting up sets of buttons.
 *	Note that this routine does not fiddle with the draw component -
 *	the caller must take care of that. This routine will draw from
 *	one to four buttons in a row, evenly spaced. The row is a number
 *	starting at 0, indicating which row of buttons, based at
 *	DirButtonsBaseRow(), to draw in.
 */

define t_gui BUTTON_BORDER	70$
define t_gui WIDTH_BUTTONS	WIDTH_BUTTONS_OLD - BUTTON_BORDER * 2$

define t_gui proc utility AddButtonRow(int row;
	string name1; int id1;
	string name2; int id2;
	string name3; int id3;
	string name4; int id4)void:
    int fontWidth, col;
    int len1, len2, len3, len4, len, count;
    int width, extra, gap;

    row := DirButtonsBaseRow() + row * (GetFontHeight(nil) + 4 + 2);
    col := BUTTON_BORDER;
    len1 := Length(name1);
    len2 := Length(name2);
    len3 := Length(name3);
    len4 := Length(name4);
    len := len1 + len2 + len3 + len4;
    AddButtonPixels(col, row, id1, name1);
    if len2 ~= 0 then
	fontWidth := GetFontWidth(nil);
	if len4 = 0 then
	    if len3 = 0 then
		count := 2;
	    else
		count := 3;
	    fi;
	else
	    count := 4;
	fi;
	width := len * fontWidth + count * 8 + (count - 1) * 2;
	extra := WIDTH_BUTTONS - width;
	gap := extra / (count - 1);
	col := col + len1 * fontWidth + 8 + gap;
	AddButtonPixels(col, row, id2, name2);
	if len3 ~= 0 then
	    col := col + len2 * fontWidth + 8 + gap;
	    if count = 3 and gap * 2 < extra or
		count = 4 and gap * 3 + 1 < extra
	    then
		col := col + 1;
	    fi;
	    AddButtonPixels(col, row, id3, name3);
	    if len4 ~= 0 then
		col := col + len3 * fontWidth + 8 + gap;
		if gap * 3 ~= extra then
		    col := col + 1;
		fi;
		AddButtonPixels(col, row, id4, name4);
	    fi;
	fi;
    fi;
corp;

/*
 * ShowCurrentImage - everything is setup - show the current image.
 */

define t_gui proc ShowCurrentImage()void:

    GShowImage(nil, "", 0.0, 0.0, 1.0, 1.0, 0.0, 0.0, 1.0, 1.0);
corp;

/*
 * ShowImage - show an image. This will be in the image area for a
 *	new-style frame, and in the graphics area for an old-style one.
 */

define t_gui proc ShowImage(string name)void:
    int saveComponent;
    bool oldStyle;

    oldStyle := Me()@p_pGUIFrameType = GUI_FRAME_OLD;
    if oldStyle then
	saveComponent := DrawComponent(nil, COMPONENT_DRAW);
    else
	saveComponent := DrawComponent(nil, COMPONENT_PICTURE);
    fi;
    GSetImage(nil, name);
    IfFound(nil);
	if oldStyle then
	    RemoveCursor();
	fi;
	ShowCurrentImage();
    Fi(nil);
    ignore DrawComponent(nil, saveComponent);
corp;

/*
 * ShowDrawing - show a drawing. This will be in the image area for a
 *	new-style frame, and in the graphics area for an old-style one.
 */

define t_gui proc ShowDrawing(action a; thing th)void:
    int saveComponent;
    bool oldStyle;

    oldStyle := Me()@p_pGUIFrameType = GUI_FRAME_OLD;
    if oldStyle then
	saveComponent := DrawComponent(nil, COMPONENT_DRAW);
    else
	saveComponent := DrawComponent(nil, COMPONENT_PICTURE);
    fi;
    if oldStyle then
	RemoveCursor();
    fi;
    GClear(nil);
    call(a, void)(th);
    ignore DrawComponent(nil, saveComponent);
corp;

unuse tp_gui
