/*
 * Amiga MUD
 *
 * Copyright (c) 1999 by Chris Gray
 */

/*
 * Common/locUtil.m - somes for some special kinds of locations.
 */

public t_locUtil CreateTable()$
use t_locUtil

private tp_locUtil CreateTable()$
use tp_locUtil

/***************************************************************************/

/* code, etc. to assist/handle buying things in stores */

define t_locUtil p_lBuyList CreateThingListProp()$	/* things for sale */
define t_locUtil p_lBuyAction CreateActionProp()$	/* how to buy here */

/*
 * AddForSale - add an item for sale at the given location.
 *	Note: we WANT this one 'utility' so that it does not execute with
 *	SysAdmin privileges.
 */

define t_locUtil proc utility public AddForSale(thing location;
	string name, desc; int price; action doBuy)thing:
    thing model;
    list thing lt;

    model := CreateThing(nil);
    /* other players need to read it when shopping or buying */
    SetThingStatus(model, ts_readonly);
    model@p_oName := name;
    if desc ~= "" then
	model@p_oDesc := desc;
    fi;
    model@p_oPrice := price;
    if doBuy ~= nil then
	model@p_oBuyChecker := doBuy;
    fi;
    model@p_oHome := location;
    lt := location@p_lBuyList;
    if lt = nil then
	lt := CreateThingList();
	location@p_lBuyList := lt;
    fi;
    if FindElement(lt, model) = -1 then
	AddTail(lt, model);
    fi;
    model
corp;

/*
 * AddObjectForSale - make an already defined object be for sale.
 */

define t_locUtil proc utility public AddObjectForSale(thing location, model;
	int price; action doBuy)void:
    list thing lt;

    model@p_oPrice := price;
    if doBuy ~= nil then
	model@p_oBuyChecker := doBuy;
    fi;
    lt := location@p_lBuyList;
    if lt = nil then
	lt := CreateThingList();
	location@p_lBuyList := lt;
    fi;
    if FindElement(lt, model) = -1 then
	AddTail(lt, model);
    fi;
corp;

/*
 * SubObjectForSale - make an object no longer for sale.
 */

define t_locUtil proc utility public SubObjectForSale(
	thing location, model)bool:
    list thing lt;

    lt := location@p_lBuyList;
    if lt ~= nil then
	if FindElement(lt, model) ~= -1 then
	    DelElement(lt, model);
	    model -- p_oPrice;
	    model -- p_oBuyChecker;
	    true
	else
	    false
	fi
    else
	false
    fi
corp;

/*
 * ShowForSale - show the things for sale at a player's current location.
 */

define t_locUtil proc public ShowForSale()void:
    list thing lt;
    int count, n, price;
    thing me, lm, model;

    me := Me();
    lm := me@p_pCurrentMode;
    lt := call(lm@p_lmGetForSaleHere, list thing)();
    if lt = nil then
	Print("There is nothing for sale here.\n");
    else
	MaybeShowMeDo("examines the merchandise");
	Print("For sale here:\n");
	count := Count(lt);
	n := 0;
	while n ~= count do
	    model := lt[n];
	    Print("  " + FormatName(model@p_oName) + " - ");
	    price := model@p_oPrice;
	    if price = 0 then
		Print("free");
	    elif price = 1 then
		Print("1 bluto");
	    else
		IPrint(price);
		Print(" blutos");
	    fi;
	    Print("\n");
	    n := n + 1;
	od;
    fi;
corp;

/*
 * StoreBuy - let the user buy something at a store. This is used as the
 *	'buy' action at the store location.
 *	Note: this is NOT a utility proc, since we want the object to
 *	      be owned by SysAdmin, and since we use OPrint.
 *	Note: intended only to be called by the player doing the buy.
 *	      Looks like it would work for machines too, however.
 */

define t_locUtil proc public StoreBuy(string what)bool:
    thing location, lm, model, me, th;
    string name;
    int price, money;
    action buyAction;
    status st;

    me := Me();
    lm := me@p_pCurrentMode;
    location := call(lm@p_lmGetLocation, thing)();
    if location = nil or location@p_lBuyList = nil then
	Print("There is nothing for sale here.\n");
	false
    else
	name := FormatName(what);
	st := FindName(location@p_lBuyList, p_oName, what);
	if st = fail then
	    Print(AAn("You cannot buy", name) + " here.\n");
	    false
	elif st = continue then
	    Print(Capitalize(name) + " is ambiguous here.\n");
	    false
	else
	    model := FindResult();
	    price := model@p_oPrice;
	    money := me@p_pMoney;
	    name := FormatName(model@p_oName);
	    if price > money and not me@p_pPrivileged then
		Print(AAn("You cannot afford", name) + ".\n");
		false
	    else
		/* Skip past one level of inheritance if that is there. This
		   allows using the same model, with different price, in
		   different stores. */
		th := Parent(model);
		if th ~= nil and th@p_oPrice ~= 0 then
		    th := CreateThing(th);
		else
		    th := CreateThing(model);
		fi;
		/* We want the thing public so that anyone can do things to
		   it. ts_readonly would work not bad, but that prevents
		   a builder from modifying it. We want it owned by SysAdmin
		   so that all the code that is setuid SysAdmin has no
		   trouble with it. We have to do the SetThingStatus BEFORE
		   we give it away, else we will not have access to do so. */
		SetThingStatus(th, ts_public);
		GiveThing(th, SysAdmin);
		if model@p_oContents ~= nil then
		    th@p_oContents := CreateThingList();
		fi;
		th@p_oCreator := me;
		buyAction := model@p_oBuyChecker;
		st := continue;
		if buyAction ~= nil then
		    SetIt(th);
		    st := call(buyAction, status)();
		fi;
		if st = continue and not CarryItem(th) then
		    st := fail;
		fi;
		if st ~= fail and not me@p_pPrivileged then
		    me@p_pMoney := money - price;
		fi;
		if st = continue then
		    Print(AAn("You have just bought", name) + ".\n");
		    MaybeShowMeDo("makes a purchase");
		else
		    /* This call will destroy the object. */
		    ClearThing(th);
		fi;
		true
	    fi
	fi
    fi
corp;

/*
 * MakeStore - make a location a store.
 */

define t_locUtil proc utility public MakeStore(thing location)void:

    location@p_lBuyAction := StoreBuy;
corp;

/*
 * IsStore - ask if location is a store.
 */

define t_locUtil proc utility public IsStore(thing location)bool:

    location@p_lBuyAction = StoreBuy
corp;

/*
 * UnmakeStore - make a location no longer a store.
 */

define t_locUtil proc utility public UnmakeStore(thing location)void:

    location -- p_lBuyList;
    location -- p_lBuyAction;
corp;


/***************************************************************************/

/* Some stuff to implement banks. Note that these properties are private,
   so no-one else can change bank accounts. Note also that the routines are
   NOT utility routines, since we want the things created to represent the
   accounts to not be owned by the player. */

define tp_locUtil p_lBankAccounts CreateThingListProp()$/* bank account list */
define tp_locUtil p_oAccountValue CreateIntProp()$	/* value in account */
define tp_locUtil p_oAccountOwner CreateThingProp()$	/* who owns account */

define tp_locUtil proc bankDeposit()void:
    list thing lt;
    int money, amount, count, i;
    thing me, lm, location, account;
    string st, name;

    st := GetWord();
    if st = "" then
	Print("You must say how many blutos you wish to deposit.\n");
    else
	amount := StringToInt(st);
	if amount < 0 then
	    Print("Invalid amount - must be a number.\n");
	else
	    me := Me();
	    lm := me@p_pCurrentMode;
	    lt := nil;
	    location := call(lm@p_lmGetLocation, thing)();
	    if location ~= nil then
		lt := location@p_lBankAccounts;
	    fi;
	    if lt = nil then
		Print("*** no account list found ***\n");
	    else
		money := me@p_pMoney;
		if amount > money then
		    Print("You do not have that much money on you.\n");
		else
		    count := Count(lt);
		    i := 0;
		    while
			if i = count then
			    false
			else
			    account := lt[i];
			    account@p_oAccountOwner ~= me
			fi
		    do
			i := i + 1;
		    od;
		    name := CharacterNameG(me);
		    if i = count then
			Print("Setting up a new account for \"" + name +
			    "\". ");
			account := CreateThing(nil);
			account@p_oAccountOwner := me;
			AddTail(lt, account);
			i := 0;
		    else
			i := account@p_oAccountValue;
		    fi;
		    me@p_pMoney := money - amount;
		    amount := amount + i;
		    account@p_oAccountValue := amount;
		    Print("Thank you for your deposit. Your account now has "
			"a balance of ");
		    if amount = 1 then
			Print("one bluto.\n");
		    else
			IPrint(amount);
			Print(" blutos.\n");
		    fi;
		    MaybeShowMeDo("makes a transaction");
		fi;
	    fi;
	fi;
    fi;
corp;

define tp_locUtil proc bankWithdraw()void:
    list thing lt;
    int amount, count, i;
    thing me, lm, location, account;
    string st;

    st := GetWord();
    if st = "" then
	Print("You must say how many blutos you wish to withdraw.\n");
    else
	amount := StringToInt(st);
	if amount < 0 then
	    Print("Invalid amount - must be a number.\n");
	else
	    me := Me();
	    lm := me@p_pCurrentMode;
	    lt := nil;
	    location := call(lm@p_lmGetLocation, thing)();
	    if location ~= nil then
		lt := location@p_lBankAccounts;
	    fi;
	    if lt = nil then
		Print("*** no account list found ***\n");
	    else
		count := Count(lt);
		i := 0;
		while
		    if i = count then
			false
		    else
			account := lt[i];
			account@p_oAccountOwner ~= me
		    fi
		do
		    i := i + 1;
		od;
		if i = count then
		    Print("I'm sorry, this bank has no account for \"" +
			me@p_pName + "\".\n");
		else
		    i := account@p_oAccountValue;
		    if amount > i then
			Print("I'm sorry, you do not have that much in "
			    "your account.\n");
		    else
			me@p_pMoney := me@p_pMoney + amount;
			amount := i - amount;
			account@p_oAccountValue := amount;
			if amount = 0 then
			    Print("Withdrawal made. Your account is now "
				  "empty and has been closed.\n");
			    DelElement(lt, account);
			else
			    Print("Withdrawal made. Your account now has a "
				"balance of ");
			    if amount = 1 then
				Print("one bluto.\n");
			    else
				IPrint(amount);
				Print(" blutos.\n");
			    fi;
			fi;
		    fi;
		    MaybeShowMeDo("makes a transaction");
		fi;
	    fi;
	fi;
    fi;
corp;

define tp_locUtil proc bankBalance()void:
    list thing lt;
    int amount, count, i;
    thing me, lm, location, account;

    me := Me();
    lm := me@p_pCurrentMode;
    lt := nil;
    location := call(lm@p_lmGetLocation, thing)();
    if location ~= nil then
	lt := location@p_lBankAccounts;
    fi;
    if lt = nil then
	Print("*** no account list found ***\n");
    else
	count := Count(lt);
	i := 0;
	while
	    if i = count then
		false
	    else
		account := lt[i];
		account@p_oAccountOwner ~= me
	    fi
	do
	    i := i + 1;
	od;
	if i = count then
	    Print("I'm sorry, this bank has no account for \"" + me@p_pName +
		"\".\n");
	else
	    amount := account@p_oAccountValue;
	    Print("Your account has a balance of ");
	    if amount = 1 then
		Print("one bluto.\n");
	    else
		IPrint(amount);
		Print(" blutos.\n");
	    fi;
	    MaybeShowMeDo("makes a transaction");
	fi;
    fi;
corp;

/*
 * make this one utility, so people can only do it to their own locations.
 */

define t_locUtil proc utility public MakeBank(thing location)void:

    location@p_lBankAccounts := CreateThingList();
    AddSpecialCommand(location, "deposit", bankDeposit);
    AddSpecialCommand(location, "withdraw", bankWithdraw);
    AddSpecialCommand(location, "balance", bankBalance);
corp;

define t_locUtil proc utility public IsBank(thing location)bool:

    location@p_lBankAccounts ~= nil
corp;

define t_locUtil proc utility public UnmakeBank(thing location)status:
    list thing accounts;

    accounts := location@p_lBankAccounts;
    if accounts = nil then
	continue
    elif Count(accounts) ~= 0 then
	fail
    else
	location -- p_lBankAccounts;
	ignore RemoveSpecialCommand(location, "deposit", bankDeposit);
	ignore RemoveSpecialCommand(location, "withdraw", bankWithdraw);
	ignore RemoveSpecialCommand(location, "balance", bankBalance);
	succeed
    fi
corp;


unuse tp_locUtil
