/*
 * Amiga MUD
 *
 * Copyright (c) 1999 by Chris Gray
 */

/*
 * Common/miscVerbs.m - common verbs that need to call out to
 *	location-specific tools.
 */

public t_miscVerbs CreateTable()$
use t_miscVerbs

private tp_miscVerbs CreateTable()$
use tp_miscVerbs

/* routines for going in specific directions (these are actual verbs) */

define tp_miscVerbs proc v_north()bool:
    UserMove(D_NORTH)
corp;
define tp_miscVerbs proc v_south()bool:
    UserMove(D_SOUTH)
corp;
define tp_miscVerbs proc v_east()bool:
    UserMove(D_EAST)
corp;
define tp_miscVerbs proc v_west()bool:
    UserMove(D_WEST)
corp;
define tp_miscVerbs proc v_northEast()bool:
    UserMove(D_NORTHEAST)
corp;
define tp_miscVerbs proc v_northWest()bool:
    UserMove(D_NORTHWEST)
corp;
define tp_miscVerbs proc v_southEast()bool:
    UserMove(D_SOUTHEAST)
corp;
define tp_miscVerbs proc v_southWest()bool:
    UserMove(D_SOUTHWEST)
corp;
define tp_miscVerbs proc v_up()bool:
    UserMove(D_UP)
corp;
define tp_miscVerbs proc v_down()bool:
    UserMove(D_DOWN)
corp;
define tp_miscVerbs proc v_enter()bool:
    UserMove(D_ENTER)
corp;
define tp_miscVerbs proc v_exit()bool:
    UserMove(D_EXIT)
corp;

/* define the actual movement verbs */

Verb0(G, "north", 0, v_north)$
Verb0(G, "south", 0, v_south)$
Verb0(G, "east", 0, v_east)$
Verb0(G, "west", 0, v_west)$
Verb0(G, "northeast", 0, v_northEast)$
Verb0(G, "northwest", 0, v_northWest)$
Verb0(G, "southeast", 0, v_southEast)$
Verb0(G, "southwest", 0, v_southWest)$
Verb0(G, "up", 0, v_up)$
Verb0(G, "down", 0, v_down)$
Verb0(G, "enter", 0, v_enter)$
Verb0(G, "exit", 0, v_exit)$	/* no synonyms - is used elsewhere */

Synonym(G, "north", "n")$
Synonym(G, "south", "s")$
Synonym(G, "east", "e")$
Synonym(G, "west", "w")$
Synonym(G, "northeast", "ne")$
Synonym(G, "northeast", "north-east")$
Synonym(G, "northwest", "nw")$
Synonym(G, "northwest", "north-west")$
Synonym(G, "southeast", "se")$
Synonym(G, "southeast", "south-east")$
Synonym(G, "southwest", "sw")$
Synonym(G, "southwest", "south-west")$
Synonym(G, "up", "u")$
Synonym(G, "down", "d")$
Verb0(G, "in", 0, v_enter)$	/* also used elsewhere */
Synonym(G, "in", "inside")$
Synonym(G, "in", "into")$
Synonym(G, "in", "inwards")$
Verb0(G, "out", 0, v_exit)$	/* also used elsewhere */
Synonym(G, "out", "outside")$
Verb0(G, "leave", 0, v_exit)$

define tp_miscVerbs proc v_go(string where)bool:

    if where = "" then
	Print("You must specify a direction to " + Verb() + ".\n");
	false
    elif DirMatch(where) ~= -1 then
	if Punctuation() ~= " " then
	    where := where + Punctuation();
	fi;
	Parse(G, where) ~= 0
    else
	Print("I don't know how to " + Verb() + " \"" +
	    FormatName(where) + "\".\n");
	Print("You must specify a direction to " + Verb() + ".\n");
	false
    fi
corp;

Verb1(G, "go", FindAnyWord(G, "to"), v_go)$
Synonym(G, "go", "walk")$
Synonym(G, "go", "climb")$
Synonym(G, "go", "jump")$
Synonym(G, "go", "move")$
Synonym(G, "go", "head")$
Synonym(G, "go", "run")$
Synonym(G, "go", "crawl")$
Synonym(G, "go", "trot")$	/* canter, gallop, slither, ...? - nah! */


/*
 * some silly verbs for people to play with
 */

define tp_miscVerbs proc doPosition(string what; int pos)bool:
    thing me, it, lm;
    list thing lt;
    status st;
    string s1, s2, whatName;
    property int countProp;
    int n;
    action a;

    me := Me();
    lm := me@p_pCurrentMode;
    case pos
    incase POS_SIT_IN:
	s1 := "sit";
	s2 := "in";
	countProp := p_oCanSitIn;
    incase POS_SIT_ON:
	s1 := "sit";
	s2 := "on";
	countProp := p_oCanSitOn;
    incase POS_LIE_IN:
	s1 := "lie";
	s2 := "in";
	countProp := p_oCanLieIn;
    incase POS_LIE_ON:
	s1 := "lie";
	s2 := "on";
	countProp := p_oCanLieOn;
    incase POS_STAND_IN:
	s1 := "stand";
	s2 := "in";
	countProp := p_oCanStandIn;
    incase POS_STAND_ON:
	s1 := "stand";
	s2 := "on";
	countProp := p_oCanStandOn;
    esac;
    if me@p_pPosition ~= POS_NONE then
	Print("You are still " + ShowPosition(me) + "\n");
	false
    elif what = "" or what == "down" then
	Print("You must say what you want to " + s1 + " " + s2 + ".\n");
	false
    else
	whatName := FormatName(what);
	lt := call(lm@p_lmGetObjectsHere, list thing)();
	st := fail;
	if lt ~= nil then
	    st := FindName(lt, p_oName, what);
	fi;
	if st = succeed then
	    whatName := " the " + whatName;
	    it := FindResult();
	    n := it@countProp;
	    if n = 0 then
		Print("You can't " + s1 + " " + s2 + whatName + ".\n");
		false
	    elif n = 1 then
		Print("There is no room " + s2 + whatName + ".\n");
		false
	    else
		st := continue;
		SetIt(it);
		a := it@p_oPositionChecker;
		if a ~= nil then
		    st := call(a, status)(pos);
		fi;
		if st ~= fail then
		    if st = continue then
			Print("You " + s1 + " " + s2 + whatName + ".\n");
			MaybeShowMeDo(s1 + "s " + s2 + whatName);
		    fi;
		    me@p_pPosition := pos;
		    me@p_pWhere := it;
		    it@countProp := n - 1;
		    true
		else
		    false
		fi
	    fi
	elif st = continue then
	    Print(Capitalize(whatName) + " is ambiguous here.\n");
	    false
	else
	    if call(lm@p_lmIsSceneryHere, bool)(what) then
		Print("You can't " + s1 + " " + s2 + " the " + whatName+".\n");
	    else
		Print(IsAre("There", "no", whatName, "here.\n"));
	    fi;
	    false
	fi
    fi
corp;

define t_miscVerbs proc PositionProp(int posCode)property int:

    case posCode
    incase POS_SIT_IN:
	p_oCanSitIn
    incase POS_SIT_ON:
	p_oCanSitOn
    incase POS_LIE_IN:
	p_oCanLieIn
    incase POS_LIE_ON:
	p_oCanLieOn
    incase POS_STAND_IN:
	p_oCanStandIn
    incase POS_STAND_ON:
	p_oCanStandOn
    default:
	Print("*** Invalid code to PositionProp ***\n");
	p_oCanSitIn
    esac
corp;

define tp_miscVerbs proc unPos()bool:
    string where;
    int pos;
    thing me, it;
    property int prop;

    me := Me();
    pos := me@p_pPosition;
    where := FormatName(me@p_pWhere@p_oName);
    if pos = POS_SIT_IN or pos = POS_STAND_IN or pos = POS_LIE_IN then
	Print("You get out of the " + where + ".\n");
	MaybeShowMeDo("gets out of the " + where);
    elif pos = POS_SIT_ON or pos = POS_LIE_ON then
	Print("You stand up.\n");
	MaybeShowMeDo("stands up");
    else
	Print("You get down from the " + where + ".\n");
	MaybeShowMeDo("gets down from the " + where);
    fi;
    me@p_pPosition := POS_NONE;
    it := me@p_pWhere;
    me@p_pWhere := me;	    /* free any reference! */
    prop := PositionProp(pos);
    it@(prop) := it@(prop) + 1;
    true
corp;

define tp_miscVerbs proc v_standUp()bool:

    if Me()@p_pPosition = POS_NONE then
	Print("You are already standing up.\n");
	false
    else
	unPos()
    fi
corp;

define tp_miscVerbs proc v_getOff()bool:

    if Me()@p_pPosition = POS_NONE then
	Print("You are not on anything.\n");
	false
    else
	unPos()
    fi
corp;

define tp_miscVerbs proc v_getOut()bool:

    if Me()@p_pPosition = POS_NONE then
	Print("You are not in anything.\n");
	false
    else
	unPos()
    fi
corp;

define tp_miscVerbs proc v_sitLieStand()bool:
    string v, w, np;

    v := Verb();
    w := GetWord();
    if w == "down" then
	w := GetWord();
    fi;
    np := GetTail();
    if v == "stand" and w == "up" then
	v_standUp()
    elif np = "" then
	if w = "" then
	    w := "on";
	fi;
	Print("You must say what you want to " + v + " " + w + ".\n");
	false
    else
	np := GetNounPhrase(G, np, 0);
	if w == "in" then
	    if v == "sit" then
		doPosition(np, POS_SIT_IN)
	    elif v == "stand" then
		doPosition(np, POS_STAND_IN)
	    else
		doPosition(np, POS_LIE_IN)
	    fi
	elif w == "on" then
	    if v == "sit" then
		doPosition(np, POS_SIT_ON)
	    elif v == "stand" then
		doPosition(np, POS_STAND_ON)
	    else
		doPosition(np, POS_LIE_ON)
	    fi
	else
	    Print("I don't know how to " + v + " " + w + " something.\n");
	    false
	fi
    fi
corp;

VerbTail(G, "sit", v_sitLieStand)$
VerbTail(G, "lie", v_sitLieStand)$
VerbTail(G, "lay", v_sitLieStand)$
VerbTail(G, "stand", v_sitLieStand)$
Verb0(G, "standup", 0, v_standUp)$
Verb0(G, "get", FindAnyWord(G, "off"), v_getOff)$
Verb0(G, "get", FindAnyWord(G, "down"), v_getOff)$
Verb0(G, "get", FindAnyWord(G, "out"), v_getOut)$


/* some standard verb routines and the corresponding verbs */

/*
 * Note that the code here is very similar to that in EnterRoom in util1.m
 * This is as desired - the same stuff should be seen.
 */

define tp_miscVerbs proc v_lookAround()bool:
    thing me, lm;

    me := Me();
    lm := me@p_pCurrentMode;
    if not call(lm@p_lmCanSeeHere, bool)(me) then
	Print("It is dark here.\n");
	false
    else
	if call(lm@p_lmShowHereToMe, bool)(true) then
	    MaybeShowMeDo("looks around");
	    true
	else
	    false
	fi
    fi
corp;

define tp_miscVerbs proc v_look(string what)bool:
    list thing lt;
    thing me, lm, object;
    string ambig, name, s;
    action a;
    status st;
    int dir;
    bool wasScenery;

    me := Me();
    lm := me@p_pCurrentMode;
    if what = "" then
	v_lookAround()
    elif not call(lm@p_lmCanSeeHere, bool)(me) then
	Print("It is dark here.\n");
	false
    elif what == "all" then
	lt := call(lm@p_lmGetObjectsHere, list thing)();
	st := fail;
	if lt ~= nil then
	    st := DoAll(lt, LookAtObject);
	fi;
	if st = fail then
	    Print("There are no obvious things here to look at.\n");
	    false
	else
	    st = continue
	fi
    else
	object := call(lm@p_lmFindAgentHere, thing)(what);
	if object ~= nil then
	    /* player is examining another player or a monster */
	    LookAtCharacter(object)
	else
	    dir := DirMatch(what);
	    if dir ~= -1 then
		/* looking in a given direction */
		s := call(lm@p_lmLookDirHere, string)(dir);
		if s = "" then
		    Print("You see nothing special in that direction.\n");
		else
		    NPrint(s);
		fi;
		s := call(lm@p_lmDirImageHere, string)(dir);
		if s ~= "" then
		    ShowImage(s);
		fi;
		true
	    else
		/* player is examining an object */
		name := FormatName(what);
		object := nil;
		wasScenery := false;
		ambig := " is ambiguous here.\n";
		st := FindName(me@p_pCarrying, p_oName, what);
		if st = fail then
		    lt := call(lm@p_lmGetObjectsHere, list thing)();
		    st := fail;
		    if lt ~= nil then
			st := FindName(lt, p_oName, what);
		    fi;
		    lt := call(lm@p_lmGetForSaleHere, list thing)();
		    if st = fail and lt ~= nil then
			st := FindName(lt, p_oName, what);
		    fi;
		    if st = fail then
			if call(lm@p_lmIsSceneryHere, bool)(what) then
			    Print("You see nothing special about the " +
				name + ".\n");
			    wasScenery := true;
			else
			    Print(IsAre("There", "no", name, "here.\n"));
			fi;
		    elif st = continue then
			Print(Capitalize(name));
			Print(ambig);
		    else
			object := FindResult();
		    fi;
		elif st = continue then
		    Print(Capitalize(name));
		    Print(ambig);
		else
		    object := FindResult();
		fi;
		if object = nil then
		    wasScenery
		else
		    LookAtObject(object)
		fi
	    fi
	fi
    fi
corp;

define tp_miscVerbs proc v_lookInwards()bool:
    v_look("in")
corp;

define tp_miscVerbs proc v_exits()bool:
    call(Me()@p_pCurrentMode@p_lmShowExitsHere, void)();
    true
corp;

Verb1(G, "look", FindAnyWord(G, "at"), v_look)$
/* Do the "around" one first, so that just "look" gets it. */
Verb0(G, "look", FindAnyWord(G, "around"), v_lookAround)$
/* Special case to enable "look inwards" */
Verb0(G, "look", FindAnyWord(G, "in"), v_lookInwards)$
Synonym(G, "look", "examine")$
Synonym(G, "look", "l")$
Verb0(G, "exits", 0, v_exits)$
Synonym(G, "exits", "x")$


define tp_miscVerbs proc v_read(string what)bool:
    list thing lt;
    thing me, lm, object;
    status st;
    action a;
    string name, ambig;
    int saveHeight;
    bool wasScenery;

    me := Me();
    lm := me@p_pCurrentMode;
    if what = "" then
	Print("You must specify what you want to read.\n");
	false
    elif not call(lm@p_lmCanSeeHere, bool)(me) then
	Print("It is dark here.\n");
	false
    else
	st := continue;
	a := me@p_pReadChecker;
	if a ~= nil then
	    st := call(a, status)();
	fi;
	if st = continue then
	    lt := call(lm@p_lmGetObjectsHere, list thing)();
	    if what == "all" then
		saveHeight := TextHeight(100000);
		st := DoAll(me@p_pCarrying, ReadObject);
		if st ~= succeed and lt ~= nil then
		    st := DoAll(lt, ReadObject);
		fi;
		ignore TextHeight(saveHeight);
		if st = fail then
		    Print("There are no obvious things here to read.\n");
		    false
		else
		    st = continue
		fi
	    else
		name := FormatName(what);
		object := FindAgent(what);
		if object ~= nil then
		    ReadObject(object)
		else
		    object := nil;
		    wasScenery := false;
		    ambig := " is ambiguous here.\n";
		    st := FindName(me@p_pCarrying, p_oName, what);
		    if st = fail then
			if lt ~= nil then
			    st := FindName(lt, p_oName, what);
			fi;
			if st = fail then
			    lt := call(lm@p_lmGetForSaleHere, list thing)();
			    if lt ~= nil and FindName(lt, p_oName,what) ~= fail
			    then
				Print("You should buy the " + name +
				    " before you try to read it.\n");
			    elif call(lm@p_lmIsSceneryHere, bool)(what) then
				Print("There is nothing to read on the " +
				    name + ".\n");
				wasScenery := true;
			    else
				Print(IsAre("There", "no", name, "here.\n"));
			    fi;
			elif st = continue then
			    Print(Capitalize(name));
			    Print(ambig);
			else
			    object := FindResult();
			fi;
		    elif st = continue then
			Print(Capitalize(name));
			Print(ambig);
		    else
			object := FindResult();
		    fi;
		    if object = nil then
			wasScenery
		    else
			ReadObject(object)
		    fi
		fi
	    fi
	else
	    st ~= fail
	fi
    fi
corp;
Verb1(G, "read", 0, v_read)$


define t_baseProp p_oNotGetString CreateStringProp()$

define t_baseUtil proc public DoGet(thing where, who, what)status:
    thing lm;
    action a;
    string whatName;
    status st;

    whatName := FormatName(what@p_oName);
    if what@p_oNotGetString ~= "" then
	Print(what@p_oNotGetString + "\n");
	fail
    elif what@p_oNotGettable then
	Print("You cannot get the " + whatName + ".\n");
	fail
    else
	lm := who@p_pCurrentMode;
	a := what@p_oGetChecker;
	st := continue;
	if a ~= nil then
	    SetIt(what);
	    st := call(a, status)(what);
	fi;
	if st = continue then
	    /* object not chained to floor or anything */
	    st := continue;
	    if where ~= nil then
		st := DoLocationGetChecks(where, what);
	    fi;
	    if st = continue then
		/* room floor isn't a pool of glue or anything */
		if CarryItem(what) then
		    call(lm@p_lmDelObjectLocation, void)(where, what);
		    what -- p_oWhere;
		    SPrint(who, whatName + ": taken.\n");
		    if call(lm@p_lmCanSeeLocation, bool)(where, who) then
			if who@p_pHidden then
			    call(lm@p_lmShowOthersLocation, void)(where, who,
				"The " + whatName +
				" rises and vanishes from view\n");
			else
			    call(lm@p_lmShowOthersLocation, void)(where, who,
				Capitalize(CharacterNameG(who)) +
				" has taken the " + whatName + ".\n");
			fi;
		    fi;
		    succeed
		else
		    fail
		fi
	    else
		st
	    fi
	else
	    st
	fi
    fi
corp;

define tp_miscVerbs proc getAllStub(thing object)bool:
    thing location;

    location := call(Me()@p_pCurrentMode@p_lmGetLocation, thing)();
    DoGet(location, Me(), object) ~= fail
corp;

define tp_miscVerbs proc v_get(string what)bool:
    list thing lt;
    string verb, whatName;
    thing me, lm, object;
    status st;

    verb := Verb();
    me := Me();
    lm := Me()@p_pCurrentMode;
    lt := call(lm@p_lmGetObjectsHere, list thing)();
    if what = "" then
	if verb == "pick" then
	    verb := "pick up";
	fi;
	Print("You must specify what you want to " + verb + ".\n");
	false
    elif what == "inventory" and verb == "take" then
	v_inventory()
    elif what == "exit" and verb == "take" then
	v_exit()
    elif what == "entrance" and verb == "take" then
	v_enter()
    elif what == "up" and verb == "get" then
	v_standUp()
    elif what == "all" then
	st := fail;
	if lt ~= nil then
	    st := DoAll(lt, getAllStub);
	fi;
	if st = fail then
	    Print("There are no obvious things here to get.\n");
	    false
	else
	    st = continue
	fi
    else
	whatName := FormatName(what);
	st := fail;
	if lt ~= nil then
	    st := FindName(lt, p_oName, what);
	fi;
	if st = succeed then
	    object := FindResult();
	    getAllStub(object)
	elif st = continue then
	    Print(Capitalize(whatName) + " is ambiguous here.\n");
	    false
	else
	    lt := call(lm@p_lmGetForSaleHere, list thing)();
	    if lt ~= nil and FindName(lt, p_oName, what) ~= fail then
		Print("You should buy the " + whatName +
		    " before you take it.\n");
	    elif call(lm@p_lmFindAgentHere, thing)(what) ~= nil then
		Print("You can't get " + whatName + ".\n");
	    else
		if FindName(me@p_pCarrying, p_oName, what) ~= fail then
		    Print("You are already carrying the " + whatName + ".\n");
		else
		    if call(lm@p_lmIsSceneryHere, bool)(what) then
			Print("You can't get the " + whatName + ".\n");
		    else
			Print(IsAre("There", "no", whatName, "here.\n"));
		    fi;
		fi;
	    fi;
	    false
	fi
    fi
corp;

Verb1(G, "pick", FindAnyWord(G, "up"), v_get)$
Verb1(G, "get", 0, v_get)$
Synonym(G, "get", "pickup")$
Synonym(G, "get", "take")$


/* this is public since it is used in the build code */

public proc public DoDrop(thing where, who, what)status:
    thing lm;
    action a;
    string whatName;
    status st;

    /* NOTE: we do any object-specific stuff first. The mixture of the
       fighting stuff and the garbage room relies on this. */
    st := continue;
    a := what@p_oUnGetChecker;
    if a ~= nil then
	SetIt(what);
	st := call(a, status)(what);
    fi;
    if st = continue then
	a := what@p_oDropChecker;
	if a ~= nil then
	    SetIt(what);
	    st := call(a, status)(what);
	fi;
    fi;
    if st = continue then
	/* it wasn't a bomb that blew up or anything */
	lm := who@p_pCurrentMode;
	st := continue;
	if where ~= nil then
	    st := DoLocationDropChecks(where, what);
	fi;
	if st = continue then
	    /* nothing funny about dropping things here */
	    call(lm@p_lmAddObjectLocation, void)(where, what);
	    DelElement(who@p_pCarrying, what);
	    what -- p_oCarryer;
	    if where = nil then
		what -- p_oWhere;
	    else
		what@p_oWhere := where;
	    fi;
	    whatName := FormatName(what@p_oName);
	    SPrint(who, whatName + ": dropped.\n");
	    if call(lm@p_lmCanSeeLocation, bool)(where, who) then
		if who@p_pHidden then
		    call(lm@p_lmShowOthersLocation, void)(where, who,
			"The " + whatName +
			" appears from nowhere and drops.\n");
		else
		    call(lm@p_lmShowOthersLocation, void)(where, who,
			Capitalize(CharacterNameG(who)) +
			" has dropped the " + whatName + ".\n");
		fi;
	    fi;
	    continue
	else
	    st
	fi
    else
	st
    fi
corp;

define tp_miscVerbs proc dropAllStub(thing object)bool:
    thing location;

    location := call(Me()@p_pCurrentMode@p_lmGetLocation, thing)();
    DoDrop(location, Me(), object) ~= fail
corp;

define tp_miscVerbs proc v_drop(string what)bool:
    thing me, object;
    string whatName;
    status st;

    me := Me();
    if what = "" then
	Print("You must specify what you want to drop.\n");
	false
    elif what == "all" then
	st := DoAll(me@p_pCarrying, dropAllStub);
	if st = fail then
	    Print("You are not carrying anything obvious to drop.\n");
	    false
	else
	    st = continue
	fi
    else
	whatName := FormatName(what);
	st := FindName(me@p_pCarrying, p_oName, what);
	if st = succeed then
	    object := FindResult();
	    dropAllStub(object)
	elif st = continue then
	    Print(Capitalize(whatName) + " is ambiguous.\n");
	    false
	else
	    Print(AAn("You are not carrying", whatName) + ".\n");
	    false
	fi
    fi
corp;

Verb1(G, "put", FindAnyWord(G, "down"), v_drop)$
Verb1(G, "drop", 0, v_drop)$


/* the fancy Verb2 ones */

define tp_miscVerbs proc v_give(string what, toWho)bool:
    string whatName, whoName, myName;
    status st;
    thing me, lm, item, who;
    action a;
    character ch;
    int len;

    whatName := FormatName(what);
    whoName := FormatName(toWho);
    me := Me();
    lm := me@p_pCurrentMode;
    myName := Capitalize(CharacterNameS(me));
    who := call(lm@p_lmFindAgentHere, thing)(toWho);
    if who = me then
	Print("There is no point in giving to yourself.\n");
	false
    elif who = nil then
	if call(lm@p_lmIsSceneryHere, bool)(toWho) then
	    Print("You can't give things to the scenery!\n");
	else
	    Print(Capitalize(whoName) + " is not here!\n");
	fi;
	false
    elif Count(who@p_pCarrying) >= MAX_CARRY then
	Print(Capitalize(whoName) + " can't carry anything else.\n");
	false
    else
	st := FindName(me@p_pCarrying, p_oName, what);
	if st = succeed then
	    item := FindResult();
	    SetIt(item);
	    st := continue;
	    a := item@p_oUnGetChecker;
	    if a ~= nil then
		st := call(a, status)(item);
	    fi;
	    if st = continue then
		a := who@p_pGivePre;
		if a ~= nil then
		    st := ForceAction(who, a);
		fi;
	    fi;
	    if st = continue then
		a := item@p_oGiveChecker;
		if a ~= nil then
		    st := call(a, status)(who);
		fi;
	    fi;
	    if st = continue then
		a := who@p_pGivePost;
		if a ~= nil then
		    st := ForceAction(who, a);
		fi;
	    fi;
	    if st = continue then
		/* cannot use CarryItem here - wrong person is running */
		whatName := FormatName(item@p_oName);
		AddTail(who@p_pCarrying, item);
		DelElement(me@p_pCarrying, item);
		item@p_oCarryer := who;
		ch := ThingCharacter(who);
		if item@p_oCreator = me and ch ~= nil then
		    item@p_oCreator := who;
		fi;
		if Owner(item) = MeCharacter() then
		    if ch = nil then
			ch := SysAdmin;
		    fi;
		    GiveThing(item, ch);
		fi;
		Print("You give the " + whatName + " to " + whoName + ".\n");
		if not me@p_pHidden and call(lm@p_lmCanSeeHere, bool)(who) then
		    SPrint(who, AAn(myName + " gives you", whatName) + ".\n");
		    call(lm@p_lmShowABHere, void)(me, who,
			AAn(myName + " gives", whatName) + " to " +
			whoName + ".\n");
		else
		    SPrint(who, AAn("Someone gives you", whatName) + ".\n");
		fi;
		true
	    else
		st = succeed
	    fi
	elif st = continue then
	    Print(Capitalize(whatName) + " is ambiguous.\n");
	    false
	else
	    if SubString(what, 0, 5) == "bluto" then
		len := Length(what) - 5;
		what := SubString(what, 5, len);
		if SubString(what, 0, 1) == "s" then
		    len := len - 1;
		    what := SubString(what, 1, len);
		fi;
		if SubString(what, 0, 1) == ";" then
		    len := len - 1;
		    what := SubString(what, 1, len);
		fi;
		if len = 0 then
		    Print("You must say how many blutos to give.\n");
		    false
		else
		    len := StringToInt(what);
		    if len < 0 then
			Print("Invalid bluto count.\n");
			false
		    else
			if len > me@p_pMoney then
			    Print("You don't have that many!\n");
			    false
			else
			    if not me@p_pHidden and
				call(lm@p_lmCanSeeHere, bool)(who)
			    then
				if len = 1 then
				    Print("You give one bluto to " +
					whoName + ".\n");
				    SPrint(who, myName +
					" gives you one bluto.\n");
				else
				    Print("You give " + IntToString(len) +
					" blutos to " + whoName + ".\n");
				    SPrint(who, myName + " gives you " +
					IntToString(len) + " blutos.\n");
				fi;
				call(lm@p_lmShowABHere, void)(me, who,
				    AAn(myName + " gives", whatName) +
				    " gives some blutos to " + whoName +
				    ".\n");
			    else
				if len = 1 then
				    SPrint(who,
					"Someone gives you one bluto.\n");
				else
				    SPrint(who, "Someone gives you " +
					IntToString(len) + " blutos.\n");
				fi;
			    fi;
			    me@p_pMoney := me@p_pMoney - len;
			    who@p_pMoney := who@p_pMoney + len;
			    true
			fi
		    fi
		fi
	    else
		Print(AAn("You are not carrying", whatName) + ".\n");
		false
	    fi
	fi
    fi
corp;

Verb2(G, "give", FindAnyWord(G, "to"), v_give)$
Synonym(G, "give", "donate")$
Synonym(G, "give", "grant")$


define tp_miscVerbs proc v_putIn(string itemRaw, containerRaw)bool:
    string itemName, containerName;
    thing me, lm, item, container;
    status st;
    list thing lt;
    action a;

    itemName := FormatName(itemRaw);
    containerName := FormatName(containerRaw);
    me := Me();
    lm := me@p_pCurrentMode;
    st := FindName(me@p_pCarrying, p_oName, itemRaw);
    if st = fail then
	Print(AAn("You aren't carrying", itemName) + ".\n");
	false
    elif st = continue then
	Print(Capitalize(itemName) + " is ambiguous.\n");
	false
    else
	item := FindResult();
	itemName := FormatName(item@p_oName);
	st := FindName(me@p_pCarrying, p_oName, containerRaw);
	lt := call(lm@p_lmGetObjectsHere, list thing)();
	if st = fail and lt ~= nil then
	    st := FindName(lt, p_oName, containerRaw);
	fi;
	if st = fail then
	    lt := call(lm@p_lmGetForSaleHere, list thing)();
	    if lt ~= nil and FindName(lt, p_oName, containerRaw) ~= fail then
		Print("You should buy the " + containerName +
		    " before you try to put things into it.\n");
	    elif call(lm@p_lmIsSceneryHere, bool)(containerRaw) then
		Print("You can't put things into the " +containerName + ".\n");
	    else
		Print("There is no " + containerName + " here.\n");
	    fi;
	    false
	elif st = continue then
	    Print(Capitalize(containerName) + " is ambiguous.\n");
	    false
	else
	    container := FindResult();
	    containerName := FormatName(container@p_oName);
	    lt := container@p_oContents;
	    if lt = nil then
		Print("You can't put things into the " +containerName + ".\n");
		false
	    elif item = container then
		Print("You can't put the " + itemName + " into itself!\n");
		false
	    else
		st := continue;
		a := item@p_oUnGetChecker;
		if a ~= nil then
		    st := call(a, status)(item);
		fi;
		if st = continue then
		    a := item@p_oPutMeInChecker;
		    if a ~= nil then
			st := call(a, status)(item, container);
		    fi;
		fi;
		if st = continue then
		    a := container@p_oPutInMeChecker;
		    if a ~= nil then
			st := call(a, status)(item, container);
		    elif Count(lt) >= container@p_oCapacity then
			Print("There is no room in the " + containerName +
			    " for the " + itemName + ".\n");
			st := fail;
		    fi;
		fi;
		if st = continue then
		    AddTail(lt, item);
		    DelElement(me@p_pCarrying, item);
		    item -- p_oCarryer;
		    item@p_oWhere := container;
		    Print("You put the " + itemName + " into the " +
			containerName + ".\n");
		    true
		else
		    st ~= fail
		fi
	    fi
	fi
    fi
corp;

define tp_miscVerbs proc v_takeFrom(string itemRaw, containerRaw)bool:
    string itemName, containerName;
    thing me, lm, item, container;
    status st;
    list thing lt;
    action a;

    itemName := FormatName(itemRaw);
    containerName := FormatName(containerRaw);
    me := Me();
    lm := me@p_pCurrentMode;
    st := FindName(me@p_pCarrying, p_oName, containerRaw);
    if st = fail then
	lt := call(lm@p_lmGetObjectsHere, list thing)();
	if lt ~= nil then
	    st := FindName(lt, p_oName, containerRaw);
	fi;
    fi;
    if st = fail then
	lt := call(lm@p_lmGetForSaleHere, list thing)();
	if lt ~= nil and FindName(lt, p_oName, containerRaw) ~= fail then
	    Print("You should buy the " + containerName +
		" before you try to take things from it.\n");
	elif call(lm@p_lmIsSceneryHere, bool)(containerRaw) then
	    Print("You can't take things from the " + containerName + ".\n");
	else
	    Print("There is no " + containerName + " here.\n");
	fi;
	false
    elif st = continue then
	Print(Capitalize(containerName) + " is ambiguous.\n");
	false
    else
	container := FindResult();
	containerName := FormatName(container@p_oName);
	lt := container@p_oContents;
	if lt = nil then
	    Print("There is nothing in the " + containerName + ".\n");
	    false
	else
	    st := FindName(container@p_oContents, p_oName, itemRaw);
	    if st = fail then
		Print("There is no " + itemName + " in the " + containerName +
		    ".\n");
		false
	    elif st = continue then
		Print(Capitalize(itemName) + " is ambiguous.\n");
		false
	    elif Count(me@p_pCarrying) >= MAX_CARRY then
		Print("You can't carry anything else.\n");
		false
	    else
		item := FindResult();
		itemName := FormatName(item@p_oName);
		st := continue;
		a := item@p_oTakeMeFromChecker;
		if a ~= nil then
		    st := call(a, status)(item, container);
		fi;
		if st = continue then
		    a := container@p_oTakeFromMeChecker;
		    if a ~= nil then
			st := call(a, status)(item, container);
		    fi;
		fi;
		if st = continue then
		    a := item@p_oGetChecker;
		    if a ~= nil then
			st := call(a, status)(item);
		    fi;
		fi;
		if st = continue then
		    AddTail(me@p_pCarrying, item);
		    DelElement(container@p_oContents, item);
		    item -- p_oWhere;
		    item@p_oCarryer := me;
		    Print("You take the " + itemName + " from the " +
			containerName + ".\n");
		    true
		else
		    st ~= fail
		fi
	    fi
	fi
    fi
corp;

define tp_miscVerbs proc v_lookIn(string containerRaw)bool:
    string containerName;
    thing me, lm, container;
    status st;
    list thing lt;
    string str;
    action a;

    if containerRaw = "" then
	Print("You must specify what you want to look in.\n");
	false
    else
	me := Me();
	lm := me@p_pCurrentMode;
	containerName := FormatName(containerRaw);
	st := FindName(me@p_pCarrying, p_oName, containerRaw);
	if st = fail then
	    lt := call(lm@p_lmGetObjectsHere, list thing)();
	    if lt ~= nil then
		st := FindName(lt, p_oName, containerRaw);
	    fi;
	fi;
	if st = fail then
	    lt := call(lm@p_lmGetForSaleHere, list thing)();
	    if lt ~= nil and FindName(lt, p_oName, containerRaw) ~= fail then
		Print("You should buy the " + containerName + " first.\n");
	    elif call(lm@p_lmIsSceneryHere, bool)(containerRaw) then
		if not call(lm@p_lmCanSeeHere, bool)(me) then
		    Print("It is dark.\n");
		else
		    Print("There is nothing to see in the " + containerName +
			".\n");
		fi;
	    else
		Print("There is no " + containerName + " here.\n");
	    fi;
	    false
	elif st = continue then
	    Print(Capitalize(containerName) + " is ambiguous.\n");
	    false
	else
	    /* do this BEFORE calling CanSee! */
	    container := FindResult();
	    if not call(lm@p_lmCanSeeHere, bool)(me) then
		Print("It is dark.\n");
		false
	    else
		containerName := FormatName(container@p_oName);
		lt := container@p_oContents;
		if lt = nil then
		    str := container@p_oLookInString;
		    if str ~= "" then
			Print(str + "\n");
			true
		    else
			a := container@p_oLookInAction;
			if a ~= nil then
			    Print(call(a, string)(container) + "\n");
			    true
			else
			    Print("There is nothing in the " +
				containerName + ".\n");
			    false
			fi
		    fi
		else
		    if ShowList(lt, "The " + containerName + " contains:\n")
		    then
			Print("The " + containerName + " is empty.\n");
		    fi;
		    true
		fi
	    fi
	fi
    fi
corp;

define tp_miscVerbs proc v_insert0(string what)bool:

    Print("You must say what you want to insert the " + FormatName(what) +
	" into.\n");
    false
corp;

Verb2(G, "put", FindAnyWord(G, "in"), v_putIn)$
Verb2(G, "put", FindAnyWord(G, "on"), v_putIn)$
/* Need to do separate verb for "insert", rather than making it a
   synonym of "put", since you cannot have something be both a synonym
   and a verb in its own right. */
Verb1(G, "insert", 0, v_insert0)$
Verb2(G, "insert", FindAnyWord(G, "in"), v_putIn)$
Verb2(G, "get", FindAnyWord(G, "from"), v_takeFrom)$
Verb2(G, "remove", FindAnyWord(G, "from"), v_takeFrom)$
/* get will use synonyms for the other 'get' */
Verb1(G, "look", FindAnyWord(G, "in"), v_lookIn)$


define tp_miscVerbs proc v_fillFrom(string containerRaw, itemRaw)bool:
    list thing lt;
    string itemName, containerName;
    thing me, lm, item, container;
    status st;
    action a;

    containerName := FormatName(containerRaw);
    itemName := FormatName(itemRaw);
    me := Me();
    st := FindName(me@p_pCarrying, p_oName, containerRaw);
    if st = fail then
	Print(AAn("You aren't carrying", containerName) + ".\n");
	false
    elif st = continue then
	Print(Capitalize(containerName) + " is ambiguous.\n");
	false
    else
	lm := me@p_pCurrentMode;
	container := FindResult();
	containerName := FormatName(container@p_oName);
	st := FindName(me@p_pCarrying, p_oName, itemRaw);
	if st = fail then
	    lt := call(lm@p_lmGetObjectsHere, list thing)();
	    if lt ~= nil then
		st := FindName(lt, p_oName, itemRaw);
	    fi;
	fi;
	if st = fail then
	    lt := call(lm@p_lmGetForSaleHere, list thing)();
	    if lt ~= nil and FindName(lt, p_oName, itemRaw) ~= fail then
		Print("You should buy the " + itemName +
		    " before you try to put it into something.\n");
	    elif call(lm@p_lmIsSceneryHere, bool)(itemRaw) then
		Print("You can't fill the " + containerName + " from the " +
		    itemName + ".\n");
	    else
		Print("There is no " + itemName + " here.\n");
	    fi;
	    false
	elif st = continue then
	    Print(Capitalize(itemName) + " is ambiguous.\n");
	    false
	else
	    item := FindResult();
	    itemName := FormatName(item@p_oName);
	    a := container@p_oFillMeWithChecker;
	    if a = nil then
		Print("You can't fill the " + containerName + ".\n");
		false
	    elif item = container then
		Print("You can't fill the " + itemName + " from itself!\n");
		false
	    else
		st := call(a, status)(container, item);
		if st = continue then
		    a := item@p_oFillWithMeChecker;
		    if a ~= nil then
			st := call(a, status)(container, item);
		    fi;
		fi;
		if st = continue then
		    Print("You fill the " + containerName + " from the " +
			itemName + ".\n");
		    true
		elif st = fail then
		    Print("You can't fill the " + containerName +
			" from the " + itemName + ".\n");
		    false
		else
		    true
		fi
	    fi
	fi
    fi
corp;

Verb2(G, "fill", FindAnyWord(G, "from"), v_fillFrom)$
Verb2(G, "fill", FindAnyWord(G, "with"), v_fillFrom)$

define tp_miscVerbs proc v_empty(string containerRaw)bool:
    string containerName;
    thing me, container;
    status st;
    action a;

    containerName := FormatName(containerRaw);
    me := Me();
    st := FindName(me@p_pCarrying, p_oName, containerRaw);
    if st = fail then
	Print(AAn("You aren't carrying", containerName) + ".\n");
	false
    elif st = continue then
	Print(Capitalize(containerName) + " is ambiguous.\n");
	false
    else
	container := FindResult();
	containerName := FormatName(container@p_oName);
	a := container@p_oEmptyChecker;
	if a = nil then
	    if container@p_oContents ~= nil then
		Print("Empty the " + containerName +
		    " by taking things out of it.\n");
	    else
		Print("You can't empty the " + containerName + ".\n");
	    fi;
	    false
	else
	    st := call(a, status)(container);
	    if st = continue then
		Print("You empty the " + containerName + ".\n");
		true
	    elif st = fail then
		Print("You can't empty the " + containerName + ".\n");
		false
	    else
		true
	    fi
	fi
    fi
corp;

Verb1(G, "empty", 0, v_empty)$


define tp_miscVerbs proc v_unlock(string lockRaw, keyRaw)bool:
    string keyName, lockName, str;
    thing me, lm, key, lock;
    status st;
    list thing lt;
    action a;

    lockName := FormatName(lockRaw);
    keyName := FormatName(keyRaw);
    me := Me();
    lm := me@p_pCurrentMode;
    lt := call(lm@p_lmGetObjectsHere, list thing)();
    st := fail;
    if lt ~= nil then
	st := FindName(lt, p_oName, lockRaw);
    fi;
    if st = fail then
	st := FindName(me@p_pCarrying, p_oName, lockRaw);
    fi;
    if st = fail then
	lt := call(lm@p_lmGetForSaleHere, list thing)();
	if lt ~= nil and FindName(lt, p_oName, lockRaw) ~= fail then
	    Print("You should buy the " + lockName +
		" before you try to unlock it.\n");
	elif call(lm@p_lmIsSceneryHere, bool)(lockRaw) then
	    Print("You can't unlock the " + lockName + ".\n");
	else
	    Print("There is no " + lockName + " here.\n");
	fi;
	false
    elif st = continue then
	Print(Capitalize(lockName) + " is ambiguous.\n");
	false
    else
	lock := FindResult();
	lockName := FormatName(lock@p_oName);
	if lock@p_oNotLockable then
	    Print("You can't unlock the " + lockName + ".\n");
	    false
	else
	    st := FindName(me@p_pCarrying, p_oName, keyRaw);
	    if st = fail then
		Print(AAn("You aren't carrying", keyName) + ".\n");
		false
	    elif st = continue then
		Print(Capitalize(keyName) + " is ambiguous.\n");
		false
	    else
		key := FindResult();
		keyName := FormatName(key@p_oName);
		a := lock@p_oUnlockMeWithChecker;
		if a = nil then
		    if lock@p_oNotUnlockString ~= "" then
			Print(lock@p_oNotUnlockString + "\n");
		    else
			Print("You can't unlock the " + lockName + ".\n");
		    fi;
		    false
		elif key = lock then
		    Print("You can't unlock the " + lockName +
			" with itself!\n");
		    false
		else
		    st := call(a, status)(lock, key);
		    if st = continue then
			a := key@p_oUnlockWithMeChecker;
			if a ~= nil then
			    st := call(a, status)(lock, key);
			fi;
		    fi;
		    if st = continue then
			if lock@p_oLocked then
			    Print("You unlock the " + lockName + " with the " +
				keyName + ".\n");
			    lock@p_oLocked := false;
			    true
			else
			    Print("The " + lockName+" is already unlocked.\n");
			    false
			fi
		    elif st = fail then
			Print("You can't unlock the " + lockName +
			    " with the " + keyName + ".\n");
			false
		    else
			true
		    fi
		fi
	    fi
	fi
    fi
corp;

Verb2(G, "unlock", FindAnyWord(G, "with"), v_unlock)$

define tp_miscVerbs proc v_lock(string lockRaw, keyRaw)bool:
    string keyName, lockName, str;
    thing me, lm, key, lock;
    status st;
    list thing lt;
    action a;

    lockName := FormatName(lockRaw);
    keyName := FormatName(keyRaw);
    me := Me();
    lm := me@p_pCurrentMode;
    lt := call(lm@p_lmGetObjectsHere, list thing)();
    st := fail;
    if lt ~= nil then
	st := FindName(lt, p_oName, lockRaw);
    fi;
    if st = fail then
	st := FindName(me@p_pCarrying, p_oName, lockRaw);
    fi;
    if st = fail then
	lt := call(lm@p_lmGetForSaleHere, list thing)();
	if lt ~= nil and FindName(lt, p_oName, lockRaw) ~= fail then
	    Print("You should buy the " + lockName +
		" before you try to lock it.\n");
	elif call(lm@p_lmIsSceneryHere, bool)(lockRaw) then
	    Print("You can't lock the " + lockName + ".\n");
	else
	    Print("There is no " + lockName + " here.\n");
	fi;
	false
    elif st = continue then
	Print(Capitalize(lockName) + " is ambiguous.\n");
	false
    else
	lock := FindResult();
	lockName := FormatName(lock@p_oName);
	if lock@p_oNotLockable then
	    Print("You can't lock the " + lockName + ".\n");
	    false
	else
	    st := FindName(me@p_pCarrying, p_oName, keyRaw);
	    if st = fail then
		Print(AAn("You aren't carrying", keyName) + ".\n");
		false
	    elif st = continue then
		Print(Capitalize(keyName) + " is ambiguous.\n");
		false
	    else
		key := FindResult();
		keyName := FormatName(key@p_oName);
		a := lock@p_oLockMeWithChecker;
		if a = nil then
		    if lock@p_oNotLockString ~= "" then
			Print(lock@p_oNotLockString + "\n");
		    else
			Print("You can't lock the " + lockName + ".\n");
		    fi;
		    false
		elif key = lock then
		    Print("You can't lock the " + lockName +
			" with itself!\n");
		    false
		else
		    st := call(a, status)(lock, key);
		    if st = continue then
			a := key@p_oLockWithMeChecker;
			if a ~= nil then
			    st := call(a, status)(lock, key);
			fi;
		    fi;
		    if st = continue then
			if lock@p_oLocked then
			    Print("The " + lockName + " is already locked.\n");
			    false
			else
			    Print("You lock the " + lockName + " with the " +
				keyName + ".\n");
			    lock@p_oLocked := true;
			    true
			fi
		    elif st = fail then
			Print("You can't lock the " + lockName +
			    " with the " + keyName + ".\n");
			false
		    else
			true
		    fi
		fi
	    fi
	fi
    fi
corp;

Verb2(G, "lock", FindAnyWord(G, "with"), v_lock)$


define tp_miscVerbs proc v_follow(string name)bool:
    thing me, lm, who;

    me := Me();
    lm := me@p_pCurrentMode;
    if name = "" then
	who := me@p_pFollowing;
	if who ~= nil then
	    name := CharacterNameG(who);
	    Print("You are currently following " + name + ".\n");
	    true
	else
	    Print("You must specify who you want to follow.\n");
	    Print("Use 'unfollow', or just move, to stop following.\n");
	    false
	fi
    else
	who := call(lm@p_lmFindAgentHere, thing)(name);
	name := FormatName(name);
	if who = me then
	    Print("You can't follow yourself.\n");
	    false
	elif who = nil then
	    if call(lm@p_lmIsSceneryHere, bool)(name) then
		Print("You can't follow the scenery!\n");
	    else
		Print(name + " is not here!\n");
	    fi;
	    false
	elif who = me@p_pFollowing then
	    Print("You are already following " + name + ".\n");
	    true
	else
	    UnFollow();
	    Follow(who)
	fi
    fi
corp;

Verb1(G, "follow", 0, v_follow)$

define tp_miscVerbs proc v_unfollow()bool:

    if Me()@p_pFollowing = nil then
	Print("You are not following anyone.\n");
	false
    else
	UnFollow();
	true
    fi
corp;

Verb0(G, "unfollow", 0, v_unfollow)$


define tp_miscVerbs proc v_hide()bool:
    thing me, lm;

    me := Me();
    lm := me@p_pCurrentMode;
    if not IsNormal() then
	if me@p_pHidden then
	    Print("You are now visible to others.\n");
	    me@p_pHidden := false;
	    MaybeShowMeDo("fades into view");
	    call(lm@p_lmShowMe, void)();
	else
	    Print("You are no longer visible to others.\n");
	    MaybeShowMeDo("fades out of view");
	    me@p_pHidden := true;
	    call(lm@p_lmUnShowMe, void)();
	    ClearFollowers(me);
	fi;
	true
    else
	Print("Hide? This isn't a game of hide and seek!\n");
	false
    fi
corp;

Verb0(G, "hide", 0, v_hide)$


/*
 * v_say - handler for speaking. Attempt to catch the likely forms.
 *	say hello => hello
 *	say to fred hello => fred hello
 *	tell joe to blah blah => joe blah blah
 */

define tp_miscVerbs proc v_say()bool:
    string word1, word2, tail;

    word1 := GetWord();
    if word1 = "" then
	Print("Put what you want to say after the 'say'.\n");
    else
	word2 := GetWord();
	tail := GetTail();
	if word2 == "to" then
	    if tail = "" then
		DoSay(word1);
	    else
		DoSay(word1 + " " + tail);
	    fi;
	else
	    if tail = "" then
		if word2 = "" then
		    DoSay(word1);
		else
		    DoSay(word1 + " " + word2);
		fi;
	    else
		DoSay(word1 + " " + word2 + " " + tail);
	    fi;
	fi;
    fi;
    true
corp;

VerbTail(G, "say", v_say)$
Synonym(G, "say", "\"")$
Synonym(G, "say", "tell")$


/* use the generic verb stuff for some typical verbs */

define t_baseProp p_oPlayString CreateStringProp()$
define t_baseProp p_oPlayChecker CreateActionProp()$
define t_baseProp p_pPlayChecker CreateActionProp()$
define t_baseProp p_oEraseString CreateStringProp()$
define t_baseProp p_oEraseChecker CreateActionProp()$
define t_baseProp p_pEraseChecker CreateActionProp()$
define t_baseProp p_oEatString CreateStringProp()$
define t_baseProp p_oEatChecker CreateActionProp()$
define t_baseProp p_pEatChecker CreateActionProp()$
define t_baseProp p_oUseString CreateStringProp()$
define t_baseProp p_oUseChecker CreateActionProp()$
define t_baseProp p_pUseChecker CreateActionProp()$
define t_baseProp p_oActivateString CreateStringProp()$
define t_baseProp p_oActivateChecker CreateActionProp()$
define t_baseProp p_pActivateChecker CreateActionProp()$
define t_baseProp p_oDeActivateString CreateStringProp()$
define t_baseProp p_oDeActivateChecker CreateActionProp()$
define t_baseProp p_pDeActivateChecker CreateActionProp()$
define t_baseProp p_oLightString CreateStringProp()$
define t_baseProp p_oLightChecker CreateActionProp()$
define t_baseProp p_pLightChecker CreateActionProp()$
define t_baseProp p_oExtinguishString CreateStringProp()$
define t_baseProp p_oExtinguishChecker CreateActionProp()$
define t_baseProp p_pExtinguishChecker CreateActionProp()$
define t_baseProp p_oWearString CreateStringProp()$
define t_baseProp p_oWearChecker CreateActionProp()$
define t_baseProp p_pWearChecker CreateActionProp()$
define t_baseProp p_oTouchString CreateStringProp()$
define t_baseProp p_oTouchChecker CreateActionProp()$
define t_baseProp p_pTouchChecker CreateActionProp()$
define t_baseProp p_oSmellString CreateStringProp()$
define t_baseProp p_oSmellChecker CreateActionProp()$
define t_baseProp p_pSmellChecker CreateActionProp()$
define t_baseProp p_oListenString CreateStringProp()$
define t_baseProp p_oListenChecker CreateActionProp()$
define t_baseProp p_pListenChecker CreateActionProp()$
define t_baseProp p_oOpenString CreateStringProp()$
define t_baseProp p_oOpenChecker CreateActionProp()$
define t_baseProp p_pOpenChecker CreateActionProp()$
define t_baseProp p_oCloseString CreateStringProp()$
define t_baseProp p_oCloseChecker CreateActionProp()$
define t_baseProp p_pCloseChecker CreateActionProp()$
define t_baseProp p_oPushString CreateStringProp()$
define t_baseProp p_oPushChecker CreateActionProp()$
define t_baseProp p_pPushChecker CreateActionProp()$
define t_baseProp p_oPullString CreateStringProp()$
define t_baseProp p_oPullChecker CreateActionProp()$
define t_baseProp p_pPullChecker CreateActionProp()$
define t_baseProp p_oTurnString CreateStringProp()$
define t_baseProp p_oTurnChecker CreateActionProp()$
define t_baseProp p_pTurnChecker CreateActionProp()$
define t_baseProp p_oLiftString CreateStringProp()$
define t_baseProp p_oLiftChecker CreateActionProp()$
define t_baseProp p_pLiftChecker CreateActionProp()$
define t_baseProp p_oLowerString CreateStringProp()$
define t_baseProp p_oLowerChecker CreateActionProp()$
define t_baseProp p_pLowerChecker CreateActionProp()$

define t_baseProp proc utility public GetVerbStringProp(string verb)
	property string:
    case MatchName(
	"play."
	"erase."
	"eat,lick,taste,drink,quaff,imbibe."
	"use,apply."
	"activate."
	"deactivate."
	"light."
	"extinguish."
	"wear."
	"touch,feel,pet."
	"smell,sniff."
	"listen."
	"open."
	"close,shut."
	"push,shove."
	"pull,yank,jerk,tug."
	"turn,twist,rotate,spin."
	"lift,raise."
	"lower."
	"get,take."
	"unlock"
	, verb)
    incase 0:
	p_oPlayString
    incase 1:
	p_oEraseString
    incase 2:
	p_oEatString
    incase 3:
	p_oUseString
    incase 4:
	p_oActivateString
    incase 5:
	p_oDeActivateString
    incase 6:
	p_oLightString
    incase 7:
	p_oExtinguishString
    incase 8:
	p_oWearString
    incase 9:
	p_oTouchString
    incase 10:
	p_oSmellString
    incase 11:
	p_oListenString
    incase 12:
	p_oOpenString
    incase 13:
	p_oCloseString
    incase 14:
	p_oPushString
    incase 15:
	p_oPullString
    incase 16:
	p_oTurnString
    incase 17:
	p_oLiftString
    incase 18:
	p_oLowerString
    incase 19:
	p_oNotGetString
    incase 20:
	p_oNotUnlockString
    default:
	nil
    esac
corp;

define t_baseProp proc utility public GetVerbCheckerProp(string verb)
	property action:
    case MatchName(
	"play."
	"erase."
	"eat,lick,taste,drink,quaff,imbibe."
	"use,apply."
	"activate."
	"deactivate."
	"light."
	"extinguish."
	"wear."
	"touch,feel,pet."
	"smell,sniff."
	"listen."
	"open."
	"close,shut."
	"push,shove."
	"pull,yank,jerk,tug."
	"turn,twist,rotate,spin."
	"lift,raise."
	"lower"
	, verb)
    incase 0:
	p_oPlayChecker
    incase 1:
	p_oEraseChecker
    incase 2:
	p_oEatChecker
    incase 3:
	p_oUseChecker
    incase 4:
	p_oActivateChecker
    incase 5:
	p_oDeActivateChecker
    incase 6:
	p_oLightChecker
    incase 7:
	p_oExtinguishChecker
    incase 8:
	p_oWearChecker
    incase 9:
	p_oTouchChecker
    incase 10:
	p_oSmellChecker
    incase 11:
	p_oListenChecker
    incase 12:
	p_oOpenChecker
    incase 13:
	p_oCloseChecker
    incase 14:
	p_oPushChecker
    incase 15:
	p_oPullChecker
    incase 16:
	p_oTurnChecker
    incase 17:
	p_oLiftChecker
    incase 18:
	p_oLowerChecker
    default:
	nil
    esac
corp;

define tp_miscVerbs proc v_play(string what)bool:
    VerbHere("play", p_oPlayString, p_oPlayChecker, p_pPlayChecker,
	     "You cannot play", what)
corp;
define tp_miscVerbs proc v_erase(string what)bool:
    VerbHere("erase", p_oEraseString, p_oEraseChecker, p_pEraseChecker,
	     "You cannot erase", what)
corp;
/* want eat/drink to be VerbHere so can drink from streams, etc. */
define tp_miscVerbs proc v_eat(string what)bool:
    VerbHere("eat", p_oEatString, p_oEatChecker, p_pEatChecker,
	     "YECH! You cannot eat", what)
corp;
define tp_miscVerbs proc v_drink(string what)bool:
    VerbHere("drink", p_oEatString, p_oEatChecker, p_pEatChecker,
	     "YUCK! You cannot drink", what)
corp;
define tp_miscVerbs proc v_use(string what)bool:
    VerbCarry("use", p_oUseString, p_oUseChecker, p_pUseChecker,
	      "You cannot use", what)
corp;
define tp_miscVerbs proc v_activate(string what)bool:
    VerbHere("activate", p_oActivateString, p_oActivateChecker,
	     p_pActivateChecker, "You cannot activate", what)
corp;
define tp_miscVerbs proc v_deactivate(string what)bool:
    VerbHere("deactivate", p_oDeActivateString, p_oDeActivateChecker,
	     p_pDeActivateChecker, "You cannot deactivate", what)
corp;
define tp_miscVerbs proc v_light(string what)bool:
    VerbHere("light", p_oLightString, p_oLightChecker, p_pLightChecker,
	     "You cannot light", what)
corp;
define tp_miscVerbs proc v_extinguish(string what)bool:
    VerbHere("extinguish", p_oExtinguishString, p_oExtinguishChecker,
	     p_pExtinguishChecker, "You cannot extinguish", what)
corp;
define tp_miscVerbs proc v_wear(string what)bool:
    VerbCarry("wear", p_oWearString, p_oWearChecker, p_pWearChecker,
	      "You cannot wear", what)
corp;
define tp_miscVerbs proc v_touch(string what)bool:
    VerbHere("touch", p_oTouchString, p_oTouchChecker, p_pTouchChecker,
	     "You feel nothing special about", what)
corp;
define tp_miscVerbs proc v_smell(string what)bool:
    VerbHere("smell", p_oSmellString, p_oSmellChecker, p_pSmellChecker,
	     "You smell nothing special about", what)
corp;
define tp_miscVerbs proc v_listen(string what)bool:
    VerbHere("listen to", p_oListenString, p_oListenChecker, p_pListenChecker,
	     "You hear nothing special from", what)
corp;
define tp_miscVerbs proc v_open(string what)bool:
    VerbHere("open", p_oOpenString, p_oOpenChecker, p_pOpenChecker,
	     "You cannot open", what)
corp;
define tp_miscVerbs proc v_close(string what)bool:
    VerbHere("close", p_oCloseString, p_oCloseChecker, p_pCloseChecker,
	     "You cannot close", what)
corp;
define tp_miscVerbs proc v_push(string what)bool:
    VerbHere("push", p_oPushString, p_oPushChecker, p_pPushChecker,
	     "You cannot push", what)
corp;
define tp_miscVerbs proc v_pull(string what)bool:
    VerbHere("pull", p_oPullString, p_oPullChecker, p_pPullChecker,
	     "You cannot pull", what)
corp;
define tp_miscVerbs proc v_turn(string what)bool:
    VerbHere("turn", p_oTurnString, p_oTurnChecker, p_pTurnChecker,
	     "You cannot turn", what)
corp;
define tp_miscVerbs proc v_lift(string what)bool:
    VerbHere("lift", p_oLiftString, p_oLiftChecker, p_pLiftChecker,
	     "You cannot lift", what)
corp;
define tp_miscVerbs proc v_lower(string what)bool:
    VerbHere("lower", p_oLowerString, p_oLowerChecker, p_pLowerChecker,
	     "You cannot lower", what)
corp;

Verb1(G, "play", 0, v_play)$
Verb1(G, "erase", 0, v_erase)$
Verb1(G, "eat", 0, v_eat)$
Synonym(G, "eat", "lick")$
Synonym(G, "eat", "taste")$
Verb1(G, "drink", 0, v_drink);
Synonym(G, "drink", "quaff")$
Synonym(G, "drink", "imbibe")$
Verb1(G, "use", 0, v_use)$
Synonym(G, "use", "apply")$
Verb1(G, "activate", 0, v_activate)$
Verb1(G, "turn", FindAnyWord(G, "on"), v_activate)$
Verb1(G, "deactivate", 0, v_deactivate)$
Synonym(G, "deactivate", "inactivate")$
Verb1(G, "turn", FindAnyWord(G, "off"), v_deactivate)$
Verb1(G, "light", 0, v_light)$
Synonym(G, "light", "ignite")$
Verb1(G, "extinguish", 0, v_extinguish)$
Synonym(G, "extinguish", "douse")$
Verb1(G, "wear", 0, v_wear)$
Verb1(G, "touch", 0, v_touch)$
Synonym(G, "touch", "feel")$
Synonym(G, "touch", "pet")$
Verb1(G, "smell", 0, v_smell)$
Synonym(G, "smell", "sniff")$
Verb1(G, "listen", FindAnyWord(G, "to"), v_listen)$
Verb1(G, "listen", FindAnyWord(G, "at"), v_listen)$
Verb1(G, "open", 0, v_open)$
Verb1(G, "close", 0, v_close)$
Synonym(G, "close", "shut")$
Verb1(G, "push", FindAnyWord(G, "on"), v_push)$
Synonym(G, "push", "shove")$
Verb1(G, "pull", FindAnyWord(G, "on"), v_pull)$
Synonym(G, "pull", "yank")$
Synonym(G, "pull", "jerk")$
Synonym(G, "pull", "tug")$
Verb1(G, "turn", 0, v_turn)$
Synonym(G, "turn", "rotate")$
Synonym(G, "turn", "twist")$
Synonym(G, "turn", "spin")$
Synonym(G, "turn", "crank")$
Synonym(G, "turn", "wind")$
Verb1(G, "lift", FindAnyWord(G, "up"), v_lift)$
Synonym(G, "lift", "raise")$
Verb1(G, "lower", 0, v_lower)$


/* properties associated with specific commands ('register', 'buy') */

define t_miscVerbs p_lRegisterAction CreateActionProp()$
define t_miscVerbs p_lHintAction CreateActionProp()$
define t_miscVerbs p_lHintString CreateStringProp()$
define t_miscVerbs p_lInfoAction CreateActionProp()$
define t_miscVerbs p_lInfoString CreateStringProp()$

/* and other verbs that use actions attached to locations */

define tp_miscVerbs proc v_register()bool:
    thing location;
    action a;

    location := call(Me()@p_pCurrentMode@p_lmGetLocation, thing)();
    a := nil;
    if location ~= nil then
	a := location@p_lRegisterAction;
    fi;
    if a = nil then
	Print("There is nothing to register at here.\n");
	false
    else
	call(a, bool)()
    fi
corp;

Verb0(G, "register", 0, v_register)$
Synonym(G, "register", "reg")$
Synonym(G, "register", "r")$

define tp_miscVerbs proc v_buy(string what)bool:
    thing location;
    action a;

    if what = "" then
	Print("You must specify what you want to buy.\n");
	false
    else
	location := call(Me()@p_pCurrentMode@p_lmGetLocation, thing)();
	a := nil;
	if location ~= nil then
	    a := location@p_lBuyAction;
	fi;
	if a = nil then
	    Print("There is nothing here to buy.\n");
	    false
	else
	    call(a, bool)(what)
	fi
    fi
corp;

Verb1(G, "buy", 0, v_buy)$
Synonym(G, "buy", "purchase")$

define tp_miscVerbs proc v_shop()bool:
    ShowForSale();
    true
corp;

Verb0(G, "shop", 0, v_shop)$
Synonym(G, "shop", "price")$
Synonym(G, "shop", "prices")$
Synonym(G, "shop", "cost")$
Synonym(G, "shop", "list")$


define tp_miscVerbs proc v_hint(string what)bool:
    thing location;
    action a;
    string st;

    location := call(Me()@p_pCurrentMode@p_lmGetLocation, thing)();
    a := nil;
    if location ~= nil then
	a := location@p_lHintAction;
    fi;
    if a = nil then
	st := "";
	if location ~= nil then
	    st := location@p_lHintString;
	fi;
	if st = "" then
	    Print("There are no hints here.\n");
	    false
	else
	    Print(st + "\n");
	    true
	fi
    else
	call(a, bool)(what)
    fi
corp;

Verb1(G, "hint", 0, v_hint)$

define tp_miscVerbs proc v_info(string what)bool:
    thing location;
    action a;
    string st;

    location := call(Me()@p_pCurrentMode@p_lmGetLocation, thing)();
    a := nil;
    if location ~= nil then
	a := location@p_lInfoAction;
    fi;
    if a = nil then
	st := "";
	if location ~= nil then
	    st := location@p_lInfoString;
	fi;
	if st = "" then
	    Print("There is no info to give here.\n");
	    false
	else
	    Print(st + "\n");
	    true
	fi
    else
	call(a, bool)(what)
    fi
corp;

Verb1(G, "info", 0, v_hint)$
Synonym(G, "info", "information")$


/* In order to use this 'cast' verb (as in casting magic spells), the
   character must have the verbs defined in his/her private table. They
   are looked up there and executed. Source file 'st:extras/spells.m'
   will give some handy spells to whatever wizard sources it. There is
   a "spells" spell there which lists the ones it creates. The build
   code also creates spells 'makebuilder' and 'unmakebuilder' for
   SysAdmin. */

define tp_miscVerbs proc v_cast()bool:
    table tb;
    string what;
    action a;

    tb := LookupTable(PrivateTable(), "t_spells");
    if tb = nil then
	Print("You have no spells to cast.\n");
	false
    else
	what := GetWord();
	if what = "" then
	    Print("You must specify the spell you want to cast.\n");
	    false
	else
	    a := LookupAction(tb, what);
	    if a = nil then
		Print("You know no spell by that name.\n");
		false
	    else
		call(a, void)();
		true
	    fi
	fi
    fi
corp;

VerbTail(G, "cast", v_cast)$


unuse tp_miscVerbs
