/*
 * Amiga MUD
 *
 * Copyright (c) 1997 by Chris Gray
 */

/*
 * quests.m - define the quest setup for the starter scenario.
 */

public t_quests CreateTable()$
use t_quests

private tp_quests CreateTable()$
use tp_quests

/* define some Quest stuff here so that we can refer to it from everywhere */

define t_quests Questor CreateThing(nil)$
define t_quests p_rQuestUser CreateThingProp()$

define tp_quests p_pQuestDoneList CreateThingListProp()$
define tp_quests p_sQuestName CreateStringProp()$
define tp_quests p_sQuestDesc CreateActionProp()$
define tp_quests p_sQuestChecker CreateActionProp()$
define tp_quests p_sQuestHint CreateActionProp()$
define tp_quests p_sQuestType CreateIntProp()$
define tp_quests QUEST_DIRECT	0$
define tp_quests QUEST_GIVE	1$
define tp_quests QUEST_TELL	2$
define tp_quests p_sQuestList CreateThingListProp()$

Questor@p_sQuestList := CreateThingList()$

define t_quests proc QuestDirect(string name; action desc, checker, hint)void:
    thing quest;

    if desc = nil or checker = nil or hint = nil then
	Print("*** Invalid action given to QuestDirect.\n");
    else
	quest := CreateThing(nil);
	quest@p_sQuestName := name;
	quest@p_sQuestDesc := desc;
	quest@p_sQuestChecker := checker;
	quest@p_sQuestHint := hint;
	quest@p_sQuestType := QUEST_DIRECT;
	AddTail(Questor@p_sQuestList, quest);
    fi;
corp;

define t_quests proc QuestGive(string name; action desc, checker, hint)void:
    thing quest;

    if desc = nil or checker = nil or hint = nil then
	Print("*** Invalid action given to QuestGive.\n");
    else
	quest := CreateThing(nil);
	quest@p_sQuestName := name;
	quest@p_sQuestDesc := desc;
	quest@p_sQuestChecker := checker;
	quest@p_sQuestHint := hint;
	quest@p_sQuestType := QUEST_GIVE;
	AddTail(Questor@p_sQuestList, quest);
    fi;
corp;

define t_quests proc public GiveToQuestor(string what)void:
    thing me;

    me := TrueMe();
    SPrint(me, "You give the " + what + " to Questor.\n");
    /* No-one should be able to see this message, but in case SysAdmin
       *poofs* into the Questor's Office to watch, or some other wizard
       similarly gets in, we'll do it "right". */
    ABPrint(AgentLocation(me), me, nil,
	    Capitalize(CharacterNameS(me)) + AAn(" gives", what) +
	    " to Questor.\n");
corp;

define t_quests proc QuestTell(string name; action desc, checker, hint)void:
    thing quest;

    if desc = nil or checker = nil or hint = nil then
	Print("*** Invalid action given to QuestTell.\n");
    else
	quest := CreateThing(nil);
	quest@p_sQuestName := name;
	quest@p_sQuestDesc := desc;
	quest@p_sQuestChecker := checker;
	quest@p_sQuestHint := hint;
	quest@p_sQuestType := QUEST_TELL;
	AddTail(Questor@p_sQuestList, quest);
    fi;
corp;

define tp_quests proc completeQuest(thing player, quest)bool:
    list thing lt;

    lt := player@p_pQuestDoneList;
    if lt = nil then
	lt := CreateThingList();
	player@p_pQuestDoneList := lt;
    fi;
    if FindElement(lt, quest) ~= -1 then
	SPrint(player, "You have already completed the " +
	    quest@p_sQuestName + " quest!\n");
	false
    else
	AddTail(lt, quest);
	APrint("\n" + Capitalize(CharacterNameS(player)) +
	    " has completed a quest!!\n\n");
	true
    fi
corp;

define t_quests proc public DoQuest(action checker)void:
    list thing quests, lt;
    thing me, quest;
    int i;

    me := Me();
    quests := Questor@p_sQuestList;
    for i from 0 upto Count(quests) - 1 do
	quest := quests[i];
	if quest@p_sQuestChecker = checker then
	    ignore completeQuest(me, quest);
	fi;
    od;
corp;

define t_quests proc public DoneQuest(action checker)bool:
    list thing lt;
    int i;
    bool found;

    found := false;
    lt := Me()@p_pQuestDoneList;
    if lt ~= nil then
	for i from 0 upto Count(lt) - 1 do
	    if lt[i]@p_sQuestChecker = checker then
		found := true;
	    fi;
	od;
    fi;
    found
corp;

define t_quests proc ShowQuests(thing who; bool wantDetail)bool:
    list thing lt;
    int count, i, oldIndent;

    lt := who@p_pQuestDoneList;
    if lt = nil then
	false
    else
	count := Count(lt);
	if count = 0 then
	    false
	else
	    Print(Capitalize(CharacterNameS(who)));
	    if wantDetail then
		oldIndent := GetIndent();
		SetIndent(oldIndent + 2);
		Print(" has completed the following quests:");
		for i from 0 upto count - 1 do
		    Print(" " + lt[i]@p_sQuestName);
		od;
		SetIndent(oldIndent);
		Print("\n");
	    else
		Print(" has completed ");
		if count = 1 then
		    Print("one quest.\n");
		else
		    IPrint(count);
		    Print(" quests.\n");
		fi;
	    fi;
	    true
	fi
    fi
corp;

/* setup the 'quests' verb */

define tp_quests proc v_quests()bool:
    string name;
    thing who;

    name := GetWord();
    if name = "" then
	if not ShowQuests(Me(), true) then
	    Print("You have not yet completed any quests.\n");
	fi;
	true
    else
	who := FindAgent(name);
	if who ~= nil then
	    if who = Me() then
		if not ShowQuests(Me(), true) then
		    Print("You have not yet completed any quests.\n");
		fi;
	    else
		if not ShowQuests(who, IsWizard()) then
		    Print(Capitalize(name) +
			" has not yet completed any quests.\n");
		fi;
	    fi;
	    true
	else
	    Print("There is no " + name + " here.\n");
	    false
	fi
    fi
corp;

VerbTail(G, "quests", v_quests)$

define tp_quests proc questorStep()void:
corp;

define tp_quests proc questorSay(string what)void:
    list thing lt;
    thing quest, player;
    int count, i;
    bool found, toPackrat, first;
    string who, word, specialWord;

    player := Questor@p_rQuestUser;
    /* so that the checker procs can find the player in question */
    SetIt(player);
    who := SetSay(what);
    found := false;
    toPackrat := false;
    first := true;
    specialWord := "";
    while
	word := GetWord();
	word ~= "" and not found
    do
	if first then
	    if SaySubject(word, "Packrat") or SaySubject(word, "pack") or
		SaySubject(word, "pac")
	    then
		toPackrat := true;
	    fi;
	    first := false;
	fi;
	lt := Questor@p_sQuestList;
	count := Count(lt);
	i := 0;
	while i ~= count and not found do
	    quest := lt[i];
	    if quest@p_sQuestType = QUEST_TELL and
		call(quest@p_sQuestChecker, bool)(word)
	    then
		found := true;
		i := count;
		if completeQuest(player, quest) then
		    SPrint(player,
			"Questor smiles warmly and says: Congratulations " +
			who + ", you have completed the " +
			quest@p_sQuestName + " quest!\n");
		fi;
	    else
		if word == "quest" or word == "quests" or
		    word == "info" or word == "information" or
		    word == "help" or word == "hint" or word == "hints"
		then
		    specialWord := word;
		fi;
		i := i + 1;
	    fi;
	od;
    od;
    if not found then
	if specialWord ~= "" then
	    if specialWord == "hint" or specialWord == "hints" then
		DoSay("Go outside and ask - don't bother me!");
	    else
		DoSay("To find out about quests, read the sign outside!");
	    fi;
	elif who ~= "Packrat" and not toPackrat then
	    DoSay("Quit gabbing and get on with your business!");
	fi;
    fi;
corp;

define tp_quests proc questorWhisperMe(string what)void:

    if SetWhisperMe(what) ~= "" then
	DoSay("Don't try to get familiar with me!");
    fi;
corp;

define tp_quests proc questorOverhear(string what)void:

    DoSay("Whispering is impolite!");
corp;

define tp_quests proc questorSaw(string what)void:

    DoSay("Stop messing around!");
corp;

define tp_quests proc questorNoNo()status:

    Print("Questor glares at you, and so does the cobra!\n");
    fail
corp;

define tp_quests proc questorPre()status:
    list thing lt;
    thing me, item, quest;
    int count, i;
    status st;
    bool found;

    me := TrueMe();
    item := It();
    lt := Questor@p_sQuestList;
    count := Count(lt);
    found := false;
    i := 0;
    while i ~= count do
	quest := lt[i];
	if quest@p_sQuestType = QUEST_GIVE then
	    st := call(quest@p_sQuestChecker, status)();
	    if st ~= continue then
		found := true;
		i := count;
		if st = succeed then
		    if completeQuest(me, quest) then
			SPrint(me,
			    "Questor smiles warmly and says: Congratulations "+
			      me@p_pName +  ", you have completed the " +
			      quest@p_sQuestName + " quest!\n");
		    fi;
		fi;
		ZapObject(item);
		DelElement(me@p_pCarrying, item);
	    else
		i := i + 1;
	    fi;
	else
	    i := i + 1;
	fi;
    od;
    if found then succeed else continue fi
corp;

define t_quests proc QuestorInit()void:

    Questor@p_pStandard := true;
    SetupMachine(Questor);
    Questor@p_pDesc :=
	"Questor is a tall, skinny man with a long, flowing white beard and "
	"an equally long and flowing mustache. His massive white eyebrows "
	"don't flow quite as much. He wears a long, flowing (naturally) cloak "
	"of jet-black satin with no decorations. His head is bare (and a "
	"little bit bald on top), but on his chest is an extremely large "
	"and ugly pendant in the shape of the letter 'Q'. It is supported on "
	"a massive chain of iron links. Wearing this may account for his "
	"perpetual look of annoyance. All of these things you notice only "
	"briefly, before you look at his eyes, which are bright grey, and "
	"seem to penetrate to your very core.";
    CreateMachine("Questor", Questor, nil, questorStep);
    ignore SetMachineActive(Questor, questorStep);
    ignore SetMachineSay(Questor, questorSay);
    ignore SetMachineWhisperMe(Questor, questorWhisperMe);
    ignore SetMachineWhisperOther(Questor, questorOverhear);
    ignore SetMachinePose(Questor, questorSaw);
    Questor@p_oTouchChecker := questorNoNo;
    Questor@p_oSmellChecker := questorNoNo;
    Questor@p_oPushChecker := questorNoNo;
    Questor@p_oPullChecker := questorNoNo;
    Questor@p_oTurnChecker := questorNoNo;
    Questor@p_oLiftChecker := questorNoNo;
    Questor@p_oLowerChecker := questorNoNo;
    Questor@p_oEatChecker := questorNoNo;
    Questor@p_Image := "Characters/Questor";
    Questor@p_pGivePre := questorPre;
corp;

define t_quests proc ResetQuestor()void:
    list thing lt;
    int i;
    thing th;

    lt := Questor@p_pCarrying;
    i := Count(lt);
    while i ~= 0 do
	i := i - 1;
	th := lt[i];
	ZapObject(th);
	DelElement(lt, th);
    od;
    Questor -- p_rQuestUser;
corp;

define t_quests proc DisplayQuests()string:
    list thing lt;
    int count, i, oldIndent;
    thing quest;
    string result;

    lt := Questor@p_sQuestList;
    count := Count(lt);
    if count = 0 then
	result := "There are no quests yet.";
    else
	result := "The current quests are:";
	oldIndent := GetIndent();
	for i from 0 upto count - 1 do
	    quest := lt[i];
	    result := result + "\n  " + quest@p_sQuestName + " quest: " +
		call(quest@p_sQuestDesc, string)();
	od;
	result := result + "\nIf you need a hint on a quest, just ask.";
    fi;
    result
corp;

define t_quests proc QuestHint(string what)bool:
    list thing lt;
    thing quest;
    int count, i;
    bool found;

    if what = "" then
	Print("Use 'hint <quest-name>' to get a hint for a given quest.\n");
	false
    else
	lt := Questor@p_sQuestList;
	count := Count(lt);
	if count = 0 then
	    Print("There are no quests yet!\n");
	    false
	else
	    if SubString(what, 0, 6) = "quest;" then
		what := SubString(what, 6, Length(what) - 6);
	    fi;
	    found := false;
	    for i from 0 upto count - 1 do
		quest := lt[i];
		if quest@p_sQuestName == what then
		    Print(call(quest@p_sQuestHint, string)() + "\n");
		    found := true;
		fi;
	    od;
	    if not found then
		Print("There is no '" + what + "' quest.\n");
		false
	    else
		true
	    fi
	fi
    fi
corp;

unuse tp_quests
