/*
 * Amiga MUD
 *
 * Copyright (c) 1997 by Chris Gray
 */

/*
 * questRoom.m - define the quest room for the starter scenario.
 */

private tp_questRoom CreateTable()$
use tp_questRoom

define tp_questRoom QUESTROOM_ID NextEffectId()$
define tp_questRoom proc drawQuestRoom()void:

    if not KnowsEffect(nil, QUESTROOM_ID) then
	DefineEffect(nil, QUESTROOM_ID);
	GSetImage(nil, "Town/questRoom");
	IfFound(nil);
	    ShowCurrentImage();
	Else(nil);
	    TextBox("Complete your", "quests here", "");
	Fi(nil);
	EndEffect();
    fi;
    CallEffect(nil, QUESTROOM_ID);
corp;

define tp_questRoom r_questRoom CreateThing(nil)$
SetupRoom(r_questRoom, "in Questor's office",
    "The office is old-fashioned, but quite opulent in its way. The walls "
    "are lined with shelves and racks containing all sorts of trophies. "
    "There are fine collections of Bushman spears, Watusi shields, "
    "Aborigine horns, Iroquois pipes, Samurai swords, Malay masks, "
    "modern skateboards, Meershaum pipes, etc. One large case contains "
    "a zoo of live-looking animals, many of which you cannot classify. "
    "A shelf contains jars, each filled with smaller, very dead-looking "
    "animals. Many appear to have faces and hands. Questor's desk is a "
    "large construction of red-swirled marble. It is quite neat and tidy, "
    "with a couple of piles of papers, a fine onyx pen-stand and a trio "
    "of Faberg eggs. Behind the desk is a matching marble chair with "
    "purple velvet cushions and a pair of huge gems as handrests. Arching "
    "over Questor's head (for he is sitting in the chair watching you as "
    "you stare around) is a cobra with flaring hood, carved from pure "
    "white marble. The cobra's eyes are deep red and it seems to be watching "
    "you.")$
r_questRoom@p_lNoMachines := true$
Scenery(r_questRoom,
    "wall,shelf,shelves,rack,trophy,trophies,collection,case;fine,large."
    "spear;Bushman."
    "shield;Watusi."
    "horn;Aborigine."
    "pipe;Iroquois."
    "sword;samurai."
    "mask;Malay."
    "skateboard,board;skate,modern."
    "pipe;Meerschaum."
    "zoo."
    "animal;live-looking,live,looking,dead-looking,dead."
    "jar."
    "face,hand."
    "desk;large,red-swirled,red,swirled,marble."
    "paper,pile;piles,of."
    "pen-stand,stand;pen,fine,onyx."
    "egg;Faberge,Faberg,trio,of."
    "chair;matching,marble."
    "cushion;purple,velvet."
    "gem,handrest,rest;hand,pair,of,huge,gem."
    "beard,mustache,eyebrow,brow;eye,long,flowing,massive,white."
    "cloak,satin;long,flowing,jet-black,jet,black,satin."
    "pendant;extremely,large,ugly."
    "q,'q';letter."
    "chain,link;chain,of,iron.")$
RoomGraphics(r_questRoom, "Questor's", "Office", NextMapGroup(), 0, 0,
	     drawQuestRoom)$
FakeObject(CreateThing(nil), r_questRoom,
    "cobra;marble.cobra,hood,snake;with,flaring,hood,pure,white,marble",
    "Other than being made of white marble and having deep red eyes that "
    "seem to be staring at you, there is nothing special about the cobra.")$
FakeObject(CreateThing(nil), r_questRoom, "eyes.eye;Questor's,Questor",
    "Questor's eyes are not something you want to stare at.")$
FakeObject(CreateThing(nil), r_questRoom,
    "eyes.eye;cobra's,snake's,cobra,snake,deep,red",
    "The stone cobra's eyes seem to be watching you.")$
FakeObject(CreateThing(nil), r_questRoom, "office",
    "Just use 'look' to look at the office.")$

/*
 * Just in case some wise-guy wizard tries to force him out.
 */

define tp_questRoom proc questorExit()status:

    if Me() = Questor then
	fail
    else
	continue
    fi
corp;
AddAnyLeaveChecker(r_questRoom, questorExit, false)$

define tp_questRoom proc questActiveAction()void:

    Print("\n* You were moved out of Questor's office on a server "
	  "restart. *\n\n");
    DelElement(Me()@p_pEnterActions, questActiveAction);
corp;

define tp_questRoom proc questRestart(thing th)void:
    thing who;

    who := Questor@p_rQuestUser;
    if who ~= nil then
	if ThingCharacter(who) ~= nil then
	    SetCharacterLocation(ThingCharacter(who), r_questRoom@p_lExit);
	    AddTail(who@p_pEnterActions, questActiveAction);
	else
	    /* Must have been a machine */
	    SetAgentLocation(who, r_questRoom@p_lExit);
	fi;
	ResetQuestor();
    fi;
corp;

RegisterActiveAction(Questor, questRestart)$

define tp_questRoom proc questIdle()void:

    DelElement(Me()@p_pExitActions, questIdle);
    ResetQuestor();
    SetLocation(r_questRoom@p_lExit);
corp;

define tp_questRoom proc questEnter()status:
    thing who, me;

    who := Questor@p_rQuestUser;
    if who = nil then
	me := Me();
	if CharacterNameS(me) ~= "Packrat" then
	    Questor@p_rQuestUser := me;
	    Print(
		"The huge door opens easily, and you enter Questor's office.\n"
		"NOTE: no-one else can enter the office while you are in it - "
		"please keep your stay as short as possible. Thank-you.\n");
	    AddHead(me@p_pExitActions, questIdle);
	fi;
	continue
    else
	Print("The huge door will not open - you cannot enter. Questor is "
	    "currently in conference with " + CharacterNameS(who) + ".\n");
	fail
    fi
corp;

define tp_questRoom proc questExit()status:

    if ThingCharacter(Me()) ~= nil then
	/* Special case - we allow Packrat to come in, but when she leaves,
	   we do not want to reset Questor. */
	DelElement(Me()@p_pExitActions, questIdle);
	ResetQuestor();
	Print("You open the huge door and go outside. "
	    "Questor can now deal with other players.\n");
    fi;
    continue
corp;

QuestorInit()$

define tp_questRoom questSign CreateThing(nil)$

/*
 * SetupQuestorOffice - set things up so that the given direction from the
 *	given location leads to Questor's office.
 */

define tp_misc proc SetupQuestorOffice(thing where; int dir)void:

    Connect(where, r_questRoom, dir);
    Connect(where, r_questRoom, D_ENTER);
    AddDirChecker(where, dir, questEnter, false);
    AddDirChecker(r_questRoom, DirBack(dir), questExit, false);
    AddEnterChecker(where, questEnter, false);
    AddExitChecker(r_questRoom, questExit, false);
    ExtendDesc(where,
	"The building here appears to be constructed from solid granite "
	"blocks and is quite featureless. The door is a large one of foot-"
	"thick oak beams strengthed with iron straps. There are no windows. "
	"In the center of the door is the letter 'Q', pounded out of thick "
	"iron. Beside the door, a discreet sign labelled \"Quests\" has a "
	"list of some kind.");
    Scenery(where,
	"block,granite;solid,granite."
	"building;featureless."
	"door,beam;foot-thick,foot,thick,oak."
	"strap;iron."
	"q,'q',letter;letter,thick,pounded,iron");
    FakeObject(questSign, where,
	"sign;discreet.sign,list,quest;discreet,quest,quests", "");
    questSign@p_oReadAction := DisplayQuests;
    where@p_lHintAction := QuestHint;
    
    /* Some final init of Questor. */
    Questor@p_pCurrentMode := Room;
    SetAgentLocation(Questor, r_questRoom);
    GNewIcon(Questor, makeQuestorIcon());
corp;

unuse tp_questRoom
