/*
 * Amiga MUD
 *
 * Copyright (c) 1997 by Chris Gray
 */

/*
 * roomGraphics.m - some stuff to do room based simple graphics.
 */

public t_roomGraphics CreateTable() $
use t_roomGraphics

private tp_roomGraphics CreateTable() $
use tp_roomGraphics

/* When Java is drawing polygons, the right hand and bottom edges are one
   short of where the should be. This is hard to kludge in the client, so
   I'm doing it here. Big sigh. This value should normally be a 1. */
define t_roomGraphics JAVA_FUDGE	0 $

define t_roomGraphics p_pRoomGraphicsDone CreateBoolProp() $

define t_roomGraphics p_rBackGroundPen CreateIntProp() $
define t_roomGraphics p_rForeGroundPen CreateIntProp() $
define t_roomGraphics p_rEdgePen CreateIntProp() $
define t_roomGraphics p_rDoorPen CreateIntProp() $
define t_roomGraphics p_rDrawBoth CreateBoolProp() $

define tp_roomGraphics p_rAutoDrawAction CreateActionProp() $

define tp_roomGraphics HORIZONTAL_DOOR_ID NextEffectId() $
define tp_roomGraphics VERTICAL_DOOR_ID NextEffectId() $
define tp_roomGraphics NORTHWEST_DOOR_ID NextEffectId() $
define tp_roomGraphics NORTHEAST_DOOR_ID NextEffectId() $

define t_roomGraphics p_rCursorX CreateIntProp() $
define t_roomGraphics p_rCursorY CreateIntProp() $
define t_roomGraphics p_pRoomButtonsNow CreateBoolProp() $

/* size of the default room-mode cursor */

define t_roomGraphics CURSOR_WIDTH	14 $
define t_roomGraphics CURSOR_HEIGHT	14 $

/* tools for providing unique map group and effect id values */

/* some standard MapGroup values */

define t_roomGraphics NO_MAP_GROUP	-1 $

define tp_roomGraphics RoomsThing CreateThing(nil) $
define tp_roomGraphics p_NextMapGroup CreateIntProp() $
RoomsThing@p_NextMapGroup := 1 $

define t_roomGraphics proc public NextMapGroup()int:
    int g;

    g := RoomsThing@p_NextMapGroup;
    RoomsThing@p_NextMapGroup := g + 1;
    g
corp;

/*
 * TextBox - draw, as the complete graphics for a room, a pretty box
 *	containing 1, 2, or 3 lines of text. Heavy assumption of using
 *	an 8 x 8 fixed font.
 */

define t_roomGraphics proc TextBox(string s1, s2, s3)void:
    int saveComponent, rows, cols, maxChars, charWidth, charHeight,
	xOffset, yOffset, len1, len2, len3, width, height, i;

    rows := DRAW_ROWS;
    cols := DRAW_COLS;
    charWidth := GetFontWidth(nil);
    charHeight := GetFontHeight(nil);
    maxChars := (cols - 10) / charWidth;
    len1 := Length(s1);
    if len1 > maxChars then
	len1 := maxChars;
    fi;
    len2 := Length(s2);
    if len2 > maxChars then
	len2 := maxChars;
    fi;
    len3 := Length(s3);
    if len3 > maxChars then
	len3 := maxChars;
    fi;
    maxChars := len1;
    if len2 > maxChars then
	maxChars := len2;
    fi;
    if len3 > maxChars then
	maxChars := len3;
    fi;
    width := 10 + maxChars * charWidth;
    xOffset := (cols - width) / 2;
    if s3 = "" then
	if s2 = "" then
	    height := 2;
	else
	    height := 1;
	fi;
    else
	height := 3;
    fi;
    height := (height + 1) * charHeight + 10;
    yOffset := (rows - height) / 2;
    GAMovePixels(nil, xOffset, yOffset);
    for i from 0 upto 3 do
	GSetPen(nil, C_DARK_GREY + i);
	GRectanglePixels(nil, width, height, false);
	GRMovePixels(nil, 1, 1);
	width := width - 2;
	height := height - 2;
    od;
    xOffset := xOffset + 5;
    GSetPen(nil, C_GOLD);
    GAMovePixels(nil, xOffset + (maxChars - len1) * charWidth / 2,
		 yOffset + charHeight + 3);
    GText(nil, s1);
    if s2 ~= "" then
	GAMovePixels(nil, xOffset + (maxChars - len2) * charWidth / 2,
		     yOffset + charHeight * 2 + 4);
	GText(nil, s2);
	if s3 ~= "" then
	    GAMovePixels(nil, xOffset + (maxChars - len3) * charWidth / 2,
			 yOffset + charHeight * 3 + 5);
	    GText(nil, s3);
	fi;
    fi;
corp;

/* a proc to yield the standard cursor for a character */

define t_roomGraphics proc public MakeCursor()list int:
    list int li;

    li := CreateIntArray(8);
/*
    li[0] := 0b01111100000000001100011000000000;
    li[1] := 0b10101010000000001000001000000000;
    li[2] := 0b10111010000000001100011000000000;
    li[3] := 0b01111100000000000000000000000000;
    li[4] := 0b00000000000000000000000000000000;
    li[5] := 0b00000000000000000000000000000000;
    li[6] := 0b00000000000000000000000000000000;
    li[7] := 0b00000000000000000000000000000000;
*/
    li[0] := 0b00000111100000000001100001100000;
    li[1] := 0b00100000000100000100110011001000;
    li[2] := 0b01001100110010001000000000000100;
    li[3] := 0b10000011000001001000001100000100;
    li[4] := 0b10000000000001000100100001001000;
    li[5] := 0b01000111100010000010000000010000;
    li[6] := 0b00011000011000000000011110000000;
    li[7] := 0b00000000000000000000000000000000;
    li
corp;

/* proc that can be attached to things, and which zaps all buttons. */
/* We need this since we cannot attach builtins to things. */

define t_roomGraphics proc EraseAllButtons()void:
    int saveComponent;

    saveComponent := DrawComponent(nil, COMPONENT_MBUTTONS);
    ClearButtons();
    ignore DrawComponent(nil, saveComponent);
corp;

define t_roomGraphics proc public AddRoomButtons()void:
    
    AddDirectionButtons(DIR_BUTTONS_ALL);
    Me()@p_pRoomButtonsNow := true;
corp;

define t_roomGraphics proc public EraseRoomButtons()void:
    
    EraseDirectionButtons(DIR_BUTTONS_ALL);
    Me()@p_pRoomButtonsNow := false;
corp;

/* a standard button handler for the above buttons */

define t_roomGraphics proc public RoomButtonHandler(int whichButton)void:

    case whichButton
    incase BUTTON_D_NW:
	InsertCommand("northwest");
    incase BUTTON_D_N:
	InsertCommand("north");
    incase BUTTON_D_NE:
	InsertCommand("northeast");
    incase BUTTON_D_W:
	InsertCommand("west");
    incase BUTTON_D_L:
	InsertCommand("look");
    incase BUTTON_D_E:
	InsertCommand("east");
    incase BUTTON_D_SW:
	InsertCommand("southwest");
    incase BUTTON_D_S:
	InsertCommand("south");
    incase BUTTON_D_SE:
	InsertCommand("southeast");
    incase BUTTON_D_U:
	InsertCommand("up");
    incase BUTTON_D_D:
	InsertCommand("down");
    incase BUTTON_D_I:
	InsertCommand("in");
    incase BUTTON_D_O:
	InsertCommand("out");
    default:
	Print("Unknown button ");
	IPrint(whichButton);
	Print(" hit!\n");
    esac;
corp;

/* a standard region hit handler for the standard region */

define t_roomGraphics proc public RoomMouseDownHandler(int region, x, y)void:
    thing here;
    int cursorX, cursorY, deltaX, deltaY;

    if region = MOVE_REGION then
	here := Here();
	cursorX := here@p_rCursorX;
	cursorY := here@p_rCursorY;
	if cursorX = 0 and cursorY = 0 then
	    /* pick the middle of the mapping region */
	    cursorX := DRAW_COLS / 2;
	    cursorY := DRAW_ROWS / 2;
	fi;
	if cursorX < x then
	    deltaX := x - cursorX;
	else
	    deltaX := cursorX - x;
	fi;
	if cursorY < y then
	    deltaY := y - cursorY;
	else
	    deltaY := cursorY - y;
	fi;
	if deltaY * 10 <= deltaX * 4 then
	    if x < cursorX then
		InsertCommand("west");
	    else
		InsertCommand("east");
	    fi;
	elif deltaX * 10 <= deltaY * 4 then
	    if y < cursorY then
		InsertCommand("north");
	    else
		InsertCommand("south");
	    fi;
	else
	    if x < cursorX then
		if y < cursorY then
		    InsertCommand("northwest");
		else
		    InsertCommand("southwest");
		fi;
	    else
		if y < cursorY then
		    InsertCommand("northeast");
		else
		    InsertCommand("southeast");
		fi;
	    fi;
	fi;
    else
	Print("Unknown region ");
	IPrint(region);
	Print(" hit!\n");
    fi;
corp;

/* Show the named image, with cursor but no icons. */

define t_roomGraphics proc public ShowImageAndRefresh(string name)void:
    GSetImage(nil, name);
    IfFound(nil);
	RemoveCursor();
	GResetIcons(nil);
	ShowCurrentImage();
	PlaceCursorPixels(DRAW_COLS / 2 - CURSOR_WIDTH / 2,
			  DRAW_ROWS / 2 - CURSOR_HEIGHT / 2);
    Fi(nil);
    Me()@p_MapGroup := NO_MAP_GROUP;
corp;

/* Do the setup needed to startup room mode in the display. */

define t_roomGraphics proc public InitRoomGraphics()void:
    thing me;

    me := Me();
    GUIFrameOld();
    GClear(nil);
    DrawLocationNameBox();
    AddRoomButtons();
    AddStandardRegions();
    SetCursorPen(me@p_pCursorColour);
    SetCursorPattern(me@p_pCursor);
    GSetIconPen(nil, me@p_pIconColour);
    me@p_pRoomGraphicsDone := true;
corp;

/*
 * callDraws - call any drawing actions there are here.
 */

define tp_roomGraphics proc public utility callDraws(thing me, here)void:
    int meGroup, hereGroup;
    action customDraw, autoDraw;

    meGroup := me@p_MapGroup;
    hereGroup := here@p_MapGroup;
    if meGroup ~= hereGroup or meGroup = NO_MAP_GROUP then
	autoDraw := here@p_rAutoDrawAction;
	customDraw := here@p_rDrawAction;
	GClear(nil);
	if autoDraw ~= nil and
	    (customDraw = nil or here@p_rDrawBoth)
	then
	    call(autoDraw, void)();
	    if customDraw = nil then
		me@p_MapGroup := NO_MAP_GROUP;
	    fi;
	fi;
	if customDraw ~= nil then
	    if hereGroup ~= 0 then
		me@p_MapGroup := hereGroup;
	    fi;
	    call(customDraw, void)();
	fi;
    fi;
corp;

/*
 * AutoRoomRedrawOnly - external interface to callDraws.
 */

define t_roomGraphics proc public utility AutoRoomRedrawOnly()void:

    callDraws(Me(), Here());
corp;

/*
 * drawExtras - draw the extra stuff we need for the display.
 */

define tp_roomGraphics proc public utility drawExtras(thing here)void:
    int xAdj, yAdj, cursorX, cursorY;

    DoDrawLocationName(here);
    xAdj := CURSOR_WIDTH / 2;
    yAdj := CURSOR_HEIGHT / 2;
    cursorX := here@p_rCursorX;
    cursorY := here@p_rCursorY;
    if cursorX = 0 and cursorY = 0 then
	cursorX := DRAW_COLS / 2;
	cursorY := DRAW_ROWS / 2;
    fi;
    /* Location specifies center of cursor. Adjust for the standard
       7x7 cursor pattern. */
    PlaceCursorPixels(cursorX - xAdj, cursorY - yAdj);
corp;

/* these are used when entering and exiting a room using this scheme. */

define t_roomGraphics proc public EnterRoomDraw()void:
    thing me, here;
    int saveComponent;

    saveComponent := DrawComponent(nil, COMPONENT_DRAW);
    me := Me();
    here := Here();
    if not me@p_pRoomGraphicsDone then
	InitRoomGraphics();
    fi;
    if here@p_Image ~= "" then
	/* Use key 0 - this is a temporary effect only. */
	DefineEffect(nil, 0);
	GSetImage(nil, here@p_Image);
	IfFound(nil);
	    ShowCurrentImage();
	Else(nil);
	    callDraws(me, here);
	Fi(nil);
	EndEffect();
	CallEffect(nil, 0);
    else
	callDraws(me, here);
    fi;
    drawExtras(here);
    ignore DrawComponent(nil, saveComponent);
corp;

define t_roomGraphics proc public LeaveRoomDraw(thing dest)void:
    int saveComponent;

    saveComponent := DrawComponent(nil, COMPONENT_DRAW);
    RemoveCursor();
    if dest = nil or dest@p_rEnterRoomDraw = nil then
	/* No room graphics in destination - clear the display area, etc. */
	GClear(nil);
	DrawLocationName("", "");
	Me()@p_MapGroup := NO_MAP_GROUP;
    fi;
    ignore DrawComponent(nil, saveComponent);
corp;

/*
 * RoomGraphics - set up a room for standard form custom graphics.
 */

define t_roomGraphics proc public RoomGraphics(thing room; string name1, name2;
	int group; int x, y; action drawAction)void:

    if name1 ~= "" then
	room@p_lName1 := name1;
    fi;
    if name2 ~= "" then
	room@p_lName2 := name2;
    fi;
    room@p_MapGroup := group;
    if x ~= 0 then
	room@p_rCursorX := x;
    fi;
    if y ~= 0 then
	room@p_rCursorY := y;
    fi;
    if drawAction ~= nil then
	room@p_rDrawAction := drawAction;
    fi;
    room@p_rEnterRoomDraw := EnterRoomDraw;
    room@p_rLeaveRoomDraw := LeaveRoomDraw;
corp;

/* the following few are 'utility' so they can be used in build code */

/*
 * AutoRoomGraphics - set up a room to use auto graphics.
 */

define t_roomGraphics proc public utility AutoRoomGraphics(thing room;
    action drawAction)void:

    room@p_rEnterRoomDraw := EnterRoomDraw;
    room@p_rLeaveRoomDraw := LeaveRoomDraw;
    room@p_rAutoDrawAction := drawAction;
corp;

/*
 * AutoRoomRedraw - redraw the room for the current user.
 */

define t_roomGraphics proc public utility AutoRoomRedraw()void:

    callDraws(Me(), Here());
    drawExtras(Here());
corp;

/*
 * AutoRoomPens - set the pens for auto graphics.
 */

define t_roomGraphics proc public utility AutoRoomPens(thing room;
	int back, fore, edge, door)void:

    if back ~= 0 then
	room@p_rBackGroundPen := back;
    else
	room -- p_rBackGroundPen;
    fi;
    if fore ~= 0 then
	room@p_rForeGroundPen := fore;
    else
	room -- p_rForeGroundPen;
    fi;
    if edge ~= 0 then
	room@p_rEdgePen := edge;
    else
	room -- p_rEdgePen;
    fi;
    if door ~= 0 then
	room@p_rDoorPen := door;
    else
	room -- p_rDoorPen;
    fi;
corp;


define tp_roomGraphics GOLD_UP_ARROW_ID NextEffectId() $
define tp_roomGraphics GOLD_DOWN_ARROW_ID NextEffectId() $

define tp_roomGraphics ARROW_SIZE	DRAW_COLS / 32 $

/*
 * DrawUpArrow - draw the up arrow.
 */

define t_roomGraphics proc public DrawUpArrow(int pen)void:
    int id;

    if pen = C_GOLD then
	id := GOLD_UP_ARROW_ID;
    else
	id := 0;
    fi;
    if id = 0 or not KnowsEffect(nil, id) then
	if id ~= 0 then
	    DefineEffect(nil, id);
	fi;
	GSetPen(nil, C_GOLD);
	GAMovePixels(nil, DRAW_COLS / 2 - ARROW_SIZE,
			  DRAW_ROWS / 2 - ARROW_SIZE);
	GRDrawPixels(nil, ARROW_SIZE, - ARROW_SIZE);
	GRDrawPixels(nil, ARROW_SIZE, ARROW_SIZE);
	GRMovePixels(nil, - ARROW_SIZE, - ARROW_SIZE);
	GRDrawPixels(nil, 0, ARROW_SIZE * 2 + ARROW_SIZE / 2);
	if id ~= 0 then
	    EndEffect();
	fi;
    fi;
    if id ~= 0 then
	CallEffect(nil, GOLD_UP_ARROW_ID);
    fi;
corp;

/*
 * DrawDownArrow - draw the down arrow.
 */

define t_roomGraphics proc public DrawDownArrow(int pen)void:
    int id;

    if pen = C_GOLD then
	id := GOLD_DOWN_ARROW_ID;
    else
	id := 0;
    fi;
    if id = 0 or not KnowsEffect(nil, id) then
	if id ~= 0 then
	    DefineEffect(nil, id);
	fi;
	GSetPen(nil, C_GOLD);
	GAMovePixels(nil, DRAW_COLS / 2 - ARROW_SIZE,
			  DRAW_ROWS / 2 + ARROW_SIZE);
	GRDrawPixels(nil, ARROW_SIZE, ARROW_SIZE);
	GRDrawPixels(nil, ARROW_SIZE, - ARROW_SIZE);
	GRMovePixels(nil, - ARROW_SIZE, ARROW_SIZE);
	GRDrawPixels(nil, 0, - ARROW_SIZE * 2 - ARROW_SIZE / 2);
	if id ~= 0 then
	    EndEffect();
	fi;
    fi;
    if id ~= 0 then
	CallEffect(nil, GOLD_DOWN_ARROW_ID);
    fi;
corp;

/*
 * DrawUpDown - draw up/down arrows for any of the auto graphics.
 */

define t_roomGraphics proc public DrawUpDown(thing here; list int exits)void:
    int pen;

    pen := here@p_rEdgePen;
    if pen = 0 or pen = here@p_rBackGroundPen or pen = here@p_rForeGroundPen
    then
	pen := C_GOLD;
    fi;
    if FindElement(exits, D_UP) ~= -1 then
	DrawUpArrow(pen);
    fi;
    if FindElement(exits, D_DOWN) ~= -1 then
	DrawDownArrow(pen);
    fi;
corp;

/*
 * Routines for drawing the standard "doors".
 */

define t_roomGraphics DOOR_HALF_WIDTH	DRAW_COLS / 64 $
define t_roomGraphics DOOR_HALF_HEIGHT	DOOR_HALF_WIDTH - 2 $
define t_roomGraphics DOOR_HEIGHT	DOOR_HALF_HEIGHT * 2 $
define t_roomGraphics DOOR_WIDTH	DOOR_HALF_WIDTH * 2 $

define t_roomGraphics proc public HorizontalDoor()void:

    if not KnowsEffect(nil, HORIZONTAL_DOOR_ID) then
	DefineEffect(nil, HORIZONTAL_DOOR_ID);
	GRMovePixels(nil, - DOOR_HALF_WIDTH, - DOOR_HALF_HEIGHT);
	GRDrawPixels(nil, 0, DOOR_HEIGHT);
	GRMovePixels(nil, 0, - DOOR_HALF_HEIGHT);
	GRDrawPixels(nil, DOOR_WIDTH, 0);
	GRMovePixels(nil, 0, - DOOR_HALF_HEIGHT);
	GRDrawPixels(nil, 0, DOOR_HEIGHT);
	GRMovePixels(nil, - DOOR_HALF_WIDTH, - DOOR_HALF_HEIGHT);
	EndEffect();
    fi;
    CallEffect(nil, HORIZONTAL_DOOR_ID);
corp;

define t_roomGraphics proc public VerticalDoor()void:

    if not KnowsEffect(nil, VERTICAL_DOOR_ID) then
	DefineEffect(nil, VERTICAL_DOOR_ID);
	GRMovePixels(nil, - DOOR_HALF_HEIGHT, - DOOR_HALF_WIDTH);
	GRDrawPixels(nil, DOOR_HEIGHT, 0);
	GRMovePixels(nil, -DOOR_HALF_HEIGHT, 0);
	GRDrawPixels(nil, 0, DOOR_WIDTH);
	GRMovePixels(nil, -DOOR_HALF_HEIGHT, 0);
	GRDrawPixels(nil, DOOR_HEIGHT, 0);
	GRMovePixels(nil, -DOOR_HALF_HEIGHT, - DOOR_HALF_WIDTH);
	EndEffect();
    fi;
    CallEffect(nil, VERTICAL_DOOR_ID);
corp;

define t_roomGraphics proc public NorthWestDoor()void:

    if not KnowsEffect(nil, NORTHWEST_DOOR_ID) then
	DefineEffect(nil, NORTHWEST_DOOR_ID);
	GRMovePixels(nil, DOOR_HALF_WIDTH - DOOR_HALF_HEIGHT,
			  - (DOOR_HALF_WIDTH + DOOR_HALF_HEIGHT));
	GRDrawPixels(nil, DOOR_HEIGHT, DOOR_HEIGHT);
	GRMovePixels(nil, - DOOR_HALF_HEIGHT, - DOOR_HALF_HEIGHT);
	GRDrawPixels(nil, - DOOR_WIDTH, DOOR_WIDTH);
	GRMovePixels(nil, - DOOR_HALF_HEIGHT, - DOOR_HALF_HEIGHT);
	GRDrawPixels(nil, DOOR_HEIGHT, DOOR_HEIGHT);
	GRMovePixels(nil, DOOR_HALF_WIDTH - DOOR_HALF_HEIGHT,
			  - (DOOR_HALF_WIDTH + DOOR_HALF_HEIGHT));
	EndEffect();
    fi;
    CallEffect(nil, NORTHWEST_DOOR_ID);
corp;

define t_roomGraphics proc public NorthEastDoor()void:

    if not KnowsEffect(nil, NORTHEAST_DOOR_ID) then
	DefineEffect(nil, NORTHEAST_DOOR_ID);
	GRMovePixels(nil, DOOR_HALF_HEIGHT - DOOR_HALF_WIDTH,
			  - (DOOR_HALF_WIDTH + DOOR_HALF_HEIGHT));
	GRDrawPixels(nil, - DOOR_HEIGHT, DOOR_HEIGHT);
	GRMovePixels(nil, DOOR_HALF_HEIGHT, - DOOR_HALF_HEIGHT);
	GRDrawPixels(nil, DOOR_WIDTH, DOOR_WIDTH);
	GRMovePixels(nil, DOOR_HALF_HEIGHT, - DOOR_HALF_HEIGHT);
	GRDrawPixels(nil, - DOOR_HEIGHT, DOOR_HEIGHT);
	GRMovePixels(nil, DOOR_HALF_HEIGHT - DOOR_HALF_WIDTH,
			  - (DOOR_HALF_WIDTH + DOOR_HALF_HEIGHT));
	EndEffect();
    fi;
    CallEffect(nil, NORTHEAST_DOOR_ID);
corp;

/*
 * AutoSimple - this is a generic form of the autographics that uses a
 *	bunch of polygons. It is used for roads, paths, tunnels, etc.
 *	SIMPLE_FACTOR and the various scaling factors used are made
 *	public so that other, more-specific, drawing routines can
 *	draw things the same size as this code does.
 */

define t_roomGraphics SIMPLE_FACTOR	IntToFixed(DRAW_COLS / 16) $

define t_roomGraphics proc public AutoSimpleParts(int bits; fixed scale;
	bool doBasic)void:
    int orthHWidth, orthHHeight, diagHWidth, diagHHeight,
	orthWidth, orthHeight, halfWidth, halfHeight;

    orthHWidth := FixedToInt(scale * SIMPLE_FACTOR);
    orthHHeight := FixedToInt(scale * SIMPLE_FACTOR);
    diagHWidth := FixedToInt(scale * SIMPLE_FACTOR * 1.414);
    diagHHeight := FixedToInt(scale * SIMPLE_FACTOR * 1.414);
    orthWidth := orthHWidth * 2;
    orthHeight := orthHHeight * 2;
    halfWidth := DRAW_COLS / 2 - orthHWidth;
    halfHeight := DRAW_ROWS / 2 - orthHHeight;
    if doBasic then
	GAMovePixels(nil, DRAW_COLS / 2, DRAW_ROWS / 2);
	GCirclePixels(nil, orthWidth, true);
    fi;
    if bits ~= 0 then
	if bits & B_NORTH ~= 0 then
	    GAMovePixels(nil, DRAW_COLS / 2 - orthHWidth, 0);
	    GRectanglePixels(nil, orthWidth, DRAW_ROWS / 2, true);
	fi;
	if bits & B_SOUTH ~= 0 then
	    GAMovePixels(nil, DRAW_COLS / 2 - orthHWidth, DRAW_ROWS / 2);
	    GRectanglePixels(nil, orthWidth, DRAW_ROWS / 2, true);
	fi;
	if bits & B_WEST ~= 0 then
	    GAMovePixels(nil, 0, DRAW_ROWS / 2 - orthHHeight);
	    GRectanglePixels(nil, DRAW_COLS / 2, orthHeight, true);
	fi;
	if bits & B_EAST ~= 0 then
	    GAMovePixels(nil, DRAW_COLS / 2, DRAW_ROWS / 2 - orthHHeight);
	    GRectanglePixels(nil, DRAW_COLS / 2, orthHeight, true);
	fi;
	if bits & B_NORTHEAST ~= 0 then
	    GPolygonStart(nil);
	    GAMovePixels(nil, DRAW_COLS - diagHWidth, 0);
	    GRDrawPixels(nil, diagHWidth, 0);
	    GRDrawPixels(nil, 0, diagHHeight);
	    GRDrawPixels(nil, - halfWidth - 3, halfHeight + 3);
	    GRDrawPixels(nil, - diagHWidth, - diagHHeight);
	    GPolygonEnd(nil);
	fi;
	if bits & B_NORTHWEST ~= 0 then
	    GPolygonStart(nil);
	    GAMovePixels(nil, diagHWidth, 0);
	    GRDrawPixels(nil, - diagHWidth, 0);
	    GRDrawPixels(nil, 0, diagHHeight);
	    GRDrawPixels(nil, halfWidth + 3, halfHeight + 3);
	    GRDrawPixels(nil, diagHWidth, - diagHHeight);
	    GPolygonEnd(nil);
	fi;
	if bits & B_SOUTHEAST ~= 0 then
	    GPolygonStart(nil);
	    GAMovePixels(nil, DRAW_COLS - diagHWidth, DRAW_ROWS);
	    GRDrawPixels(nil, diagHWidth, 0);
	    GRDrawPixels(nil, 0, - diagHHeight);
	    GRDrawPixels(nil, - halfWidth - 3, - halfHeight - 3);
	    GRDrawPixels(nil, - diagHWidth, diagHHeight);
	    GPolygonEnd(nil);
	fi;
	if bits & B_SOUTHWEST ~= 0 then
	    GPolygonStart(nil);
	    GAMovePixels(nil, diagHWidth, DRAW_ROWS);
	    GRDrawPixels(nil, - diagHWidth, 0);
	    GRDrawPixels(nil, 0, - diagHHeight);
	    GRDrawPixels(nil, halfWidth + 3, - halfHeight - 3);
	    GRDrawPixels(nil, diagHWidth, diagHHeight);
	    GPolygonEnd(nil);
	fi;
    fi;
corp;

define t_roomGraphics proc public AutoSimple(fixed scale;
	int defBackground, defForeground; bool doBasic)void:
    list int exits;
    thing here;
    int background, foreground, bits, i;

    here := Here();
    background := here@p_rBackGroundPen;
    foreground := here@p_rForeGroundPen;
    if background = 0 and foreground = 0 then
	background := defBackground;
	foreground := defForeground;
    fi;
    if doBasic then
	ClearTo(background);
    fi;
    GSetPen(nil, foreground);
    exits := here@p_rExits;
    bits := 0;
    if exits ~= nil then
	for i from D_NORTH upto D_NORTHWEST do
	    if FindElement(exits, i) ~= -1 then
		bits := bits | (1 << i);
	    fi;
	od;
    fi;
    AutoSimpleParts(bits, scale, doBasic);
    DrawUpDown(here, exits);
corp;

/*
 * AutoRoads - autodraw a roads view.
 */

define t_roomGraphics ROAD_SCALE	1.0 $

define t_roomGraphics proc public AutoRoads()void:

    AutoSimple(ROAD_SCALE, C_DARK_GREEN, C_TAN, true);
corp;

/*
 * AutoPaths - autodraw a paths view.
 */

define t_roomGraphics PATH_SCALE	0.5 $

define t_roomGraphics proc public AutoPaths()void:

    AutoSimple(PATH_SCALE, C_FOREST_GREEN, C_TAN, true);
corp;

/*
 * AutoTunnels - autodraw a tunnels view.
 */

define t_roomGraphics TUNNEL_SCALE	0.8 $

define t_roomGraphics proc public AutoTunnels()void:

    AutoSimple(TUNNEL_SCALE, C_DARK_GREY, C_LIGHT_GREY, true);
corp;


define tp_roomGraphics CHAMBER_HORIZ_GAP	DRAW_COLS * 10 / 58 $
define tp_roomGraphics CHAMBER_VERT_GAP		DRAW_ROWS * 10 / 56 $
define tp_roomGraphics CHAMBER_HORIZ_SLOPE	DRAW_COLS * 10 / 58 $
define tp_roomGraphics CHAMBER_VERT_SLOPE	DRAW_ROWS * 10 / 56 $

define tp_roomGraphics CHAMBER_HORIZ_BOTH	CHAMBER_HORIZ_GAP +
						    CHAMBER_HORIZ_SLOPE $
define tp_roomGraphics CHAMBER_VERT_BOTH	CHAMBER_VERT_GAP +
						    CHAMBER_VERT_SLOPE $

define tp_roomGraphics GREYS_TUNNEL_CHAMBER_ID NextEffectId() $

define tp_roomGraphics proc chamberStrokes(int background, foreground)void:

    ClearTo(background);
    GSetPen(nil, foreground);
    GPolygonStart(nil);
    GAMovePixels(nil, CHAMBER_HORIZ_BOTH, CHAMBER_VERT_GAP);
    GADrawPixels(nil, DRAW_COLS - CHAMBER_HORIZ_BOTH, CHAMBER_VERT_GAP);
    GADrawPixels(nil, DRAW_COLS - CHAMBER_HORIZ_GAP, CHAMBER_VERT_BOTH);
    GADrawPixels(nil, DRAW_COLS - CHAMBER_HORIZ_GAP,
		      DRAW_ROWS - CHAMBER_VERT_BOTH);
    GADrawPixels(nil, DRAW_COLS - CHAMBER_HORIZ_BOTH,
		      DRAW_ROWS - CHAMBER_VERT_GAP);
    GADrawPixels(nil, CHAMBER_HORIZ_BOTH, DRAW_ROWS - CHAMBER_VERT_GAP);
    GADrawPixels(nil, CHAMBER_HORIZ_GAP, DRAW_ROWS - CHAMBER_VERT_BOTH);
    GADrawPixels(nil, CHAMBER_HORIZ_GAP, CHAMBER_VERT_BOTH);
    GPolygonEnd(nil);
corp;

/*
 * DrawTunnelChamber - draw the non-exit part of a tunnel chamber.
 */

define t_roomGraphics proc public DrawTunnelChamber(
	int background, foreground)void:

    if background = C_DARK_GREY and foreground = C_LIGHT_GREY then
	if not KnowsEffect(nil, GREYS_TUNNEL_CHAMBER_ID) then
	    DefineEffect(nil, GREYS_TUNNEL_CHAMBER_ID);
	    chamberStrokes(C_DARK_GREY, C_LIGHT_GREY);
	    EndEffect();
	fi;
	CallEffect(nil, GREYS_TUNNEL_CHAMBER_ID);
    else
	chamberStrokes(background, foreground);
    fi;
corp;

/*
 * AutoTunnelChamber - autodraw a large tunnel chamber.
 */

define t_roomGraphics proc public AutoTunnelChamber()void:
    thing here;
    int background, foreground;

    here := Here();
    background := here@p_rBackGroundPen;
    foreground := here@p_rForeGroundPen;
    if background = 0 and foreground = 0 then
	background := C_DARK_GREY;
	foreground := C_LIGHT_GREY;
    fi;
    DrawTunnelChamber(background, foreground);
    AutoSimple(0.8, background, foreground, false);
corp;

/*
 * AutoOpenSpace - autodraw an open space with edges.
 */

define t_roomGraphics OPEN_HORIZ_GAP	DRAW_COLS / 8 $
define t_roomGraphics OPEN_VERT_GAP	OPEN_HORIZ_GAP $
define t_roomGraphics OPEN_HORIZ_EXTRA	DRAW_COLS * 10 / 105 $
define t_roomGraphics OPEN_VERT_EXTRA	OPEN_HORIZ_EXTRA $

define tp_roomGraphics OPEN_HORIZ_BOTH	OPEN_HORIZ_GAP + OPEN_HORIZ_EXTRA $
define tp_roomGraphics OPEN_VERT_BOTH	OPEN_VERT_GAP + OPEN_VERT_EXTRA $

define t_roomGraphics proc public AutoOpenSpaceParts(int bits)void:

    /* Note that the tests here are the reverse of in the other auto-draw
       code. This is because we draw the blocking part if there is no
       exit in a given direction, rather than drawing an exit. */
    if bits & B_NORTH = 0 then
	GAMovePixels(nil, OPEN_HORIZ_BOTH, 0);
	GRectanglePixels(nil, DRAW_COLS - 2 * OPEN_HORIZ_BOTH,
			      OPEN_VERT_GAP, true);
    fi;
    if bits & B_NORTHEAST = 0 then
	GPolygonStart(nil);
	GAMovePixels(nil, DRAW_COLS - OPEN_HORIZ_BOTH, 0);
	GADrawPixels(nil, DRAW_COLS - JAVA_FUDGE, 0);
	GADrawPixels(nil, DRAW_COLS - JAVA_FUDGE, OPEN_VERT_BOTH);
	GADrawPixels(nil, DRAW_COLS - OPEN_HORIZ_GAP, OPEN_VERT_BOTH);
	GADrawPixels(nil, DRAW_COLS - OPEN_HORIZ_GAP, OPEN_VERT_GAP);
	GADrawPixels(nil, DRAW_COLS - OPEN_HORIZ_BOTH, OPEN_VERT_GAP);
	GPolygonEnd(nil);
    fi;
    if bits & B_EAST = 0 then
	GAMovePixels(nil, DRAW_COLS - OPEN_HORIZ_GAP, OPEN_VERT_BOTH);
	GRectanglePixels(nil, OPEN_HORIZ_GAP,
			      DRAW_ROWS - 2 * OPEN_VERT_BOTH, true);
    fi;
    if bits & B_SOUTHEAST = 0 then
	GPolygonStart(nil);
	GAMovePixels(nil, DRAW_COLS - JAVA_FUDGE,
			  DRAW_ROWS - OPEN_VERT_BOTH);
	GADrawPixels(nil, DRAW_COLS - JAVA_FUDGE, DRAW_ROWS - JAVA_FUDGE);
	GADrawPixels(nil, DRAW_COLS - OPEN_HORIZ_BOTH,
			  DRAW_ROWS - JAVA_FUDGE);
	GADrawPixels(nil, DRAW_COLS - OPEN_HORIZ_BOTH,
			  DRAW_ROWS - OPEN_VERT_GAP);
	GADrawPixels(nil, DRAW_COLS - OPEN_HORIZ_GAP,
			  DRAW_ROWS - OPEN_VERT_GAP);
	GADrawPixels(nil, DRAW_COLS - OPEN_HORIZ_GAP,
			  DRAW_ROWS - OPEN_HORIZ_BOTH);
	GPolygonEnd(nil);
    fi;
    if bits & B_SOUTH = 0 then
	GAMovePixels(nil, OPEN_HORIZ_BOTH, DRAW_ROWS - OPEN_VERT_GAP);
	GRectanglePixels(nil, DRAW_COLS - 2 * OPEN_HORIZ_BOTH,
			      OPEN_VERT_GAP, true);
    fi;
    if bits & B_SOUTHWEST = 0 then
	GPolygonStart(nil);
	GAMovePixels(nil, 0, DRAW_ROWS - OPEN_VERT_BOTH);
	GADrawPixels(nil, 0, DRAW_ROWS - JAVA_FUDGE);
	GADrawPixels(nil, OPEN_HORIZ_BOTH, DRAW_ROWS - JAVA_FUDGE);
	GADrawPixels(nil, OPEN_HORIZ_BOTH, DRAW_ROWS - OPEN_VERT_GAP);
	GADrawPixels(nil, OPEN_HORIZ_GAP, DRAW_ROWS - OPEN_VERT_GAP);
	GADrawPixels(nil, OPEN_HORIZ_GAP, DRAW_ROWS - OPEN_VERT_BOTH);
	GPolygonEnd(nil);
    fi;
    if bits & B_WEST = 0 then
	GAMovePixels(nil, 0, OPEN_VERT_BOTH);
	GRectanglePixels(nil, OPEN_HORIZ_GAP,
			      DRAW_ROWS - 2 * OPEN_VERT_BOTH, true);
    fi;
    if bits & B_NORTHWEST = 0 then
	GPolygonStart(nil);
	GAMovePixels(nil, 0, OPEN_VERT_BOTH);
	GADrawPixels(nil, 0, 0);
	GADrawPixels(nil, OPEN_HORIZ_BOTH, 0);
	GADrawPixels(nil, OPEN_HORIZ_BOTH, OPEN_VERT_GAP);
	GADrawPixels(nil, OPEN_HORIZ_GAP, OPEN_VERT_GAP);
	GADrawPixels(nil, OPEN_HORIZ_GAP, OPEN_VERT_BOTH);
	GPolygonEnd(nil);
    fi;
corp;

define t_roomGraphics proc public AutoOpenSpace()void:
    thing here;
    int background, foreground, bits, i;
    list int exits;

    here := Here();
    background := here@p_rBackGroundPen;
    foreground := here@p_rForeGroundPen;
    if background = 0 and foreground = 0 then
	foreground := C_DARK_GREEN;
	background := C_DARK_BROWN;
    fi;
    ClearTo(foreground);
    exits := here@p_rExits;
    if exits ~= nil then
	GSetPen(nil, background);
	bits := 0;
	for i from D_NORTH upto D_NORTHWEST do
	    if FindElement(exits, i) ~= -1 then
		bits := bits | (1 << i);
	    fi;
	od;
	AutoOpenSpaceParts(bits);
	DrawUpDown(here, exits);
    fi;
corp;

/*
 * AutoHalls - a simple horizontal/vertical hallway set.
 */

define t_roomGraphics AH_HORIZ_HEIGHT	DRAW_COLS * 10 / 63 $
define t_roomGraphics AH_VERT_WIDTH	DRAW_ROWS * 10 / 56 $

define tp_roomGraphics AH_VERT_LEFT	(DRAW_COLS - AH_VERT_WIDTH) / 2 $
define tp_roomGraphics AH_VERT_RIGHT	AH_VERT_LEFT + AH_VERT_WIDTH - 1 $
define tp_roomGraphics AH_HORIZ_TOP	(DRAW_ROWS - AH_HORIZ_HEIGHT) / 2 $
define tp_roomGraphics AH_HORIZ_BOTTOM	AH_HORIZ_TOP + AH_HORIZ_HEIGHT - 1 $

define t_roomGraphics proc public AutoHallsParts(int bits; bool doCenter;
	int foreground, edge)void:

    if doCenter then
	GSetPen(nil, foreground);
	GAMovePixels(nil, AH_VERT_LEFT, AH_HORIZ_TOP);
	GRectanglePixels(nil, AH_VERT_WIDTH, AH_HORIZ_HEIGHT, true);
	GSetPen(nil, edge);
	GAMovePixels(nil, AH_VERT_LEFT - 1, AH_HORIZ_TOP - 1);
	GRectanglePixels(nil, AH_VERT_WIDTH + 2, AH_HORIZ_HEIGHT + 2, false);
    fi;
    if bits ~= 0 then
	if bits & B_NORTH ~= 0 then
	    GSetPen(nil, foreground);
	    GAMovePixels(nil, AH_VERT_LEFT, 0);
	    GRectanglePixels(nil, AH_VERT_WIDTH, AH_HORIZ_TOP, true);
	    GSetPen(nil, edge);
	    GAMovePixels(nil, AH_VERT_LEFT - 1, 0);
	    GADrawPixels(nil, AH_VERT_LEFT - 1, AH_HORIZ_TOP - 1);
	    GAMovePixels(nil, AH_VERT_RIGHT + 1, 0);
	    GADrawPixels(nil, AH_VERT_RIGHT + 1, AH_HORIZ_TOP - 1);
	fi;
	if bits & B_SOUTH ~= 0 then
	    GSetPen(nil, foreground);
	    GAMovePixels(nil, AH_VERT_LEFT, AH_HORIZ_BOTTOM + 1);
	    GRectanglePixels(nil, AH_VERT_WIDTH, AH_HORIZ_TOP, true);
	    GSetPen(nil, edge);
	    GAMovePixels(nil, AH_VERT_LEFT - 1, AH_HORIZ_BOTTOM + 1);
	    GADrawPixels(nil, AH_VERT_LEFT - 1, DRAW_ROWS - 1);
	    GAMovePixels(nil, AH_VERT_RIGHT + 1, AH_HORIZ_BOTTOM - 1);
	    GADrawPixels(nil, AH_VERT_RIGHT + 1, DRAW_ROWS - 1);
	fi;
	if bits & B_EAST ~= 0 then
	    GSetPen(nil, foreground);
	    GAMovePixels(nil, AH_VERT_RIGHT + 1, AH_HORIZ_TOP);
	    GRectanglePixels(nil, AH_VERT_LEFT + 1, AH_HORIZ_HEIGHT, true);
	    GSetPen(nil, edge);
	    GAMovePixels(nil, AH_VERT_RIGHT + 1, AH_HORIZ_TOP - 1);
	    GADrawPixels(nil, DRAW_COLS - 1, AH_HORIZ_TOP - 1);
	    GAMovePixels(nil, AH_VERT_RIGHT + 1, AH_HORIZ_BOTTOM + 1);
	    GADrawPixels(nil, DRAW_COLS - 1, AH_HORIZ_BOTTOM + 1);
	fi;
	if bits & B_WEST ~= 0 then
	    GSetPen(nil, foreground);
	    GAMovePixels(nil, 0, AH_HORIZ_TOP);
	    GRectanglePixels(nil, AH_VERT_LEFT, AH_HORIZ_HEIGHT, true);
	    GSetPen(nil, edge);
	    GAMovePixels(nil, 0, AH_HORIZ_TOP - 1);
	    GADrawPixels(nil, AH_VERT_LEFT - 1, AH_HORIZ_TOP - 1);
	    GAMovePixels(nil, 0, AH_HORIZ_BOTTOM + 1);
	    GADrawPixels(nil, AH_VERT_LEFT - 1, AH_HORIZ_BOTTOM + 1);
	fi;
    fi;
corp;

define t_roomGraphics proc public AutoHalls()void:
    thing here;
    list int exits;
    int background, foreground, edge, bits, i;

    here := Here();
    background := here@p_rBackGroundPen;
    foreground := here@p_rForeGroundPen;
    edge := here@p_rEdgePen;
    if background = 0 and foreground = 0 and edge = 0 then
	foreground := C_BLACK;
	background := C_MEDIUM_GREY;
	edge := C_TAN;
    fi;
    exits := here@p_rExits;
    bits := 0;
    if exits ~= nil then
	for i from 0 upto 3 do
	    if FindElement(exits, i * 2) ~= -1 then
		bits := bits | (1 << (i * 2));
	    fi;
	od;
    fi;
    ClearTo(background);
    AutoHallsParts(bits, true, foreground, edge);
    DrawUpDown(here, exits);
corp;

/*
 * AutoOpenRoom - autodraw a room with corridors.
 */

define tp_roomGraphics OR_HORIZ_GAP	DRAW_COLS * 10 / 63 $
define tp_roomGraphics OR_VERT_GAP	OR_HORIZ_GAP $

define tp_roomGraphics OR_HORIZ_HEIGHT	DRAW_COLS * 10 / 63 $
define tp_roomGraphics OR_VERT_WIDTH	DRAW_ROWS * 10 / 56 $

define tp_roomGraphics OR_VERT_LEFT	(DRAW_COLS - OR_VERT_WIDTH) / 2 $
define tp_roomGraphics OR_VERT_RIGHT	OR_VERT_LEFT + OR_VERT_WIDTH - 1 $
define tp_roomGraphics OR_HORIZ_TOP	(DRAW_ROWS - OR_HORIZ_HEIGHT) / 2 $
define tp_roomGraphics OR_HORIZ_BOTTOM	OR_HORIZ_TOP + OR_HORIZ_HEIGHT - 1 $

define t_roomGraphics proc public AutoOpenRoomParts(int bits; bool doCenter;
	int foreground, edge)void:

    if doCenter then
	GSetPen(nil, foreground);
	GAMovePixels(nil, OR_HORIZ_GAP, OR_VERT_GAP);
	GRectanglePixels(nil, DRAW_COLS - 2 * OR_HORIZ_GAP,
			      DRAW_ROWS - 2 * OR_VERT_GAP, true);
	GSetPen(nil, edge);
	GAMovePixels(nil, OR_HORIZ_GAP - 1, OR_VERT_GAP - 1);
	GRectanglePixels(nil, DRAW_COLS - 2 * OR_HORIZ_GAP + 2,
			      DRAW_ROWS - 2 * OR_VERT_GAP + 2, false);
    fi;
    if bits ~= 0 then
	if bits & B_NORTH ~= 0 then
	    GSetPen(nil, foreground);
	    GAMovePixels(nil, OR_VERT_LEFT, 0);
	    GRectanglePixels(nil, OR_VERT_WIDTH, OR_VERT_GAP, true);
	    GSetPen(nil, edge);
	    GAMovePixels(nil, OR_VERT_LEFT - 1, 0);
	    GADrawPixels(nil, OR_VERT_LEFT - 1, OR_VERT_GAP - 1);
	    GAMovePixels(nil, OR_VERT_RIGHT + 1, 0);
	    GADrawPixels(nil, OR_VERT_RIGHT + 1, OR_VERT_GAP - 1);
	fi;
	if bits & B_SOUTH ~= 0 then
	    GSetPen(nil, foreground);
	    GAMovePixels(nil, OR_VERT_LEFT, DRAW_ROWS - OR_VERT_GAP);
	    GRectanglePixels(nil, OR_VERT_WIDTH, OR_VERT_GAP, true);
	    GSetPen(nil, edge);
	    GAMovePixels(nil, OR_VERT_LEFT - 1, DRAW_ROWS - OR_VERT_GAP + 1);
	    GADrawPixels(nil, OR_VERT_LEFT - 1, DRAW_ROWS - 1);
	    GAMovePixels(nil, OR_VERT_RIGHT + 1, DRAW_ROWS - OR_VERT_GAP + 1);
	    GADrawPixels(nil, OR_VERT_RIGHT + 1, DRAW_ROWS - 1);
	fi;
	if bits & B_EAST ~= 0 then
	    GSetPen(nil, foreground);
	    GAMovePixels(nil, DRAW_COLS - OR_HORIZ_GAP, OR_HORIZ_TOP);
	    GRectanglePixels(nil, OR_HORIZ_GAP, OR_HORIZ_HEIGHT, true);
	    GSetPen(nil, edge);
	    GAMovePixels(nil, DRAW_COLS - OR_HORIZ_GAP + 1, OR_HORIZ_TOP - 1);
	    GADrawPixels(nil, DRAW_COLS - 1, OR_HORIZ_TOP - 1);
	    GAMovePixels(nil, DRAW_COLS - OR_HORIZ_GAP + 1,
			      OR_HORIZ_BOTTOM + 1);
	    GADrawPixels(nil, DRAW_COLS, OR_HORIZ_BOTTOM + 1);
	fi;
	if bits & B_WEST ~= 0 then
	    GSetPen(nil, foreground);
	    GAMovePixels(nil, 0, OR_HORIZ_TOP);
	    GRectanglePixels(nil, OR_HORIZ_GAP, OR_HORIZ_HEIGHT, true);
	    GSetPen(nil, edge);
	    GAMovePixels(nil, 0, OR_HORIZ_TOP - 1);
	    GADrawPixels(nil, OR_HORIZ_GAP - 1, OR_HORIZ_TOP - 1);
	    GAMovePixels(nil, 0, OR_HORIZ_BOTTOM + 1);
	    GADrawPixels(nil, OR_HORIZ_GAP - 1, OR_HORIZ_BOTTOM + 1);
	fi;
    fi;
corp;

define t_roomGraphics proc public AutoOpenRoom()void:
    thing here;
    int background, foreground, edge, bits, i;
    list int exits;

    here := Here();
    background := here@p_rBackGroundPen;
    foreground := here@p_rForeGroundPen;
    edge := here@p_rEdgePen;
    if background = 0 and foreground = 0 and edge = 0 then
	foreground := C_BLACK;
	background := C_MEDIUM_GREY;
	edge := C_TAN;
    fi;
    exits := here@p_rExits;
    bits := 0;
    if exits ~= nil then
	for i from 0 upto 3 do
	    if FindElement(exits, i * 2) ~= -1 then
		bits := bits | (1 << (i * 2));
	    fi;
	od;
    fi;
    ClearTo(background);
    AutoOpenRoomParts(bits, true, foreground, edge);
    DrawUpDown(here, exits);
corp;

/*
 * AutoClosedRoom - autodraw a room with doors.
 */

define tp_roomGraphics OC_HORIZ_GAP	OR_HORIZ_GAP $
define tp_roomGraphics OC_VERT_GAP	OC_HORIZ_GAP $

define t_roomGraphics proc public AutoClosedRoomParts(int bits; bool doCenter;
	int background, foreground, edge, door)void:

    if doCenter then
	GSetPen(nil, foreground);
	GAMovePixels(nil, OC_HORIZ_GAP, OC_VERT_GAP);
	GRectanglePixels(nil, DRAW_COLS - 2 * OC_HORIZ_GAP,
			      DRAW_ROWS - 2 * OC_VERT_GAP, true);
	GSetPen(nil, edge);
	GAMovePixels(nil, OC_HORIZ_GAP - 1, OC_VERT_GAP - 1);
	GRectanglePixels(nil, DRAW_COLS - 2 * OC_HORIZ_GAP + 2,
			      DRAW_ROWS - 2 * OC_VERT_GAP + 2, false);
    fi;
    if bits ~= 0 then
	GSetPen(nil, door);
	if bits & B_NORTH ~= 0 then
	    GAMovePixels(nil, DRAW_COLS / 2, OC_VERT_GAP - 1);
	    HorizontalDoor();
	fi;
	if bits & B_SOUTH ~= 0 then
	    GAMovePixels(nil, DRAW_COLS / 2, DRAW_ROWS - OC_VERT_GAP);
	    HorizontalDoor();
	fi;
	if bits & B_EAST ~= 0 then
	    GAMovePixels(nil, DRAW_COLS - OC_HORIZ_GAP, DRAW_ROWS /2);
	    VerticalDoor();
	fi;
	if bits & B_WEST ~= 0 then
	    GAMovePixels(nil, OC_HORIZ_GAP - 1, DRAW_ROWS / 2);
	    VerticalDoor();
	fi;
	if bits & B_NORTHEAST ~= 0 then
	    GSetPen(nil, background);
	    GPolygonStart(nil);
	    GAMovePixels(nil, DRAW_COLS - OC_HORIZ_GAP - DOOR_WIDTH - 1,
			      OC_VERT_GAP - 1);
	    GADrawPixels(nil, DRAW_COLS - OC_HORIZ_GAP + 1, OC_VERT_GAP - 1);
	    GADrawPixels(nil, DRAW_COLS - OC_HORIZ_GAP + 1,
			      OC_VERT_GAP + DOOR_WIDTH + 1);
	    GPolygonEnd(nil);
	    GSetPen(nil, door);
	    GAMovePixels(nil, DRAW_COLS - OC_HORIZ_GAP - DOOR_HALF_WIDTH,
			      OC_VERT_GAP + DOOR_HALF_WIDTH);
	    NorthEastDoor();
	fi;
	if bits & B_NORTHWEST ~= 0 then
	    GSetPen(nil, background);
	    GPolygonStart(nil);
	    GAMovePixels(nil, OC_HORIZ_GAP + DOOR_WIDTH + 1, OC_VERT_GAP - 1);
	    GADrawPixels(nil, OC_HORIZ_GAP - 1, OC_VERT_GAP - 1);
	    GADrawPixels(nil, OC_HORIZ_GAP - 1, OC_VERT_GAP + DOOR_WIDTH + 1);
	    GPolygonEnd(nil);
	    GSetPen(nil, door);
	    GAMovePixels(nil, OC_HORIZ_GAP + DOOR_HALF_WIDTH,
			      OC_VERT_GAP + DOOR_HALF_WIDTH);
	    NorthWestDoor();
	fi;
	if bits & B_SOUTHEAST ~= 0 then
	    GSetPen(nil, background);
	    GPolygonStart(nil);
	    GAMovePixels(nil, DRAW_COLS - OC_HORIZ_GAP - DOOR_WIDTH - 1,
			      DRAW_ROWS - OC_VERT_GAP + 1);
	    GADrawPixels(nil, DRAW_COLS - OC_HORIZ_GAP + 1,
			      DRAW_ROWS - OC_VERT_GAP + 1);
	    GADrawPixels(nil, DRAW_COLS - OC_HORIZ_GAP + 1,
			      DRAW_ROWS - OC_VERT_GAP - DOOR_WIDTH - 1);
	    GPolygonEnd(nil);
	    GSetPen(nil, door);
	    GAMovePixels(nil, DRAW_COLS - OC_HORIZ_GAP - DOOR_HALF_WIDTH,
			      DRAW_ROWS - OC_VERT_GAP - DOOR_HALF_WIDTH);
	    NorthWestDoor();
	fi;
	if bits & B_SOUTHWEST ~= 0 then
	    GSetPen(nil, background);
	    GPolygonStart(nil);
	    GAMovePixels(nil, OC_HORIZ_GAP + DOOR_WIDTH + 1,
			      DRAW_ROWS - OC_VERT_GAP + 1);
	    GADrawPixels(nil, OC_HORIZ_GAP - 1, DRAW_ROWS - OC_VERT_GAP + 1);
	    GADrawPixels(nil, OC_HORIZ_GAP - 1,
			      DRAW_ROWS - OC_VERT_GAP - DOOR_WIDTH - 1);
	    GPolygonEnd(nil);
	    GSetPen(nil, door);
	    GAMovePixels(nil, OC_HORIZ_GAP + DOOR_HALF_WIDTH,
			      DRAW_ROWS - OC_VERT_GAP - DOOR_HALF_WIDTH);
	    NorthEastDoor();
	fi;
    fi;
corp;

define t_roomGraphics proc public AutoClosedRoom()void:
    thing here;
    int background, foreground, edge, door, bits, i;
    list int exits;

    here := Here();
    background := here@p_rBackGroundPen;
    foreground := here@p_rForeGroundPen;
    edge := here@p_rEdgePen;
    door := here@p_rDoorPen;
    if background = 0 and foreground = 0 and edge = 0 and door = 0 then
	background := C_MEDIUM_GREY;
	foreground := C_BLACK;
	edge := C_TAN;
	door := C_BROWN;
    fi;
    exits := here@p_rExits;
    bits := 0;
    if exits ~= nil then
	for i from D_NORTH upto D_NORTHWEST do
	    if FindElement(exits, i) ~= -1 then
		bits := bits | (1 << i);
	    fi;
	od;
    fi;
    ClearTo(background);
    AutoClosedRoomParts(bits, true, background, foreground, edge, door);
    DrawUpDown(here, exits);
corp;

unuse tp_roomGraphics
