/*
 * Amiga MUD
 *
 * Copyright (c) 1999 by Chris Gray
 */

/*
 * Room/roomUtil.m - basic utility routines related to rooms.
 */

public t_roomUtil CreateTable()$
use t_roomUtil

public t_roomTypes CreateTable()$
use t_roomTypes

private tp_roomUtil CreateTable()$
use tp_roomUtil


/*
 * DirChecks - map a direction onto a property that will yield a set of
 *	direction-specific checks on a room.
 */

define t_roomUtil proc utility DirChecks(int dir)property list action:
    case dir
    incase D_NORTH:
	p_rNorthChecks
    incase D_NORTHEAST:
	p_rNorthEastChecks
    incase D_EAST:
	p_rEastChecks
    incase D_SOUTHEAST:
	p_rSouthEastChecks
    incase D_SOUTH:
	p_rSouthChecks
    incase D_SOUTHWEST:
	p_rSouthWestChecks
    incase D_WEST:
	p_rWestChecks
    incase D_NORTHWEST:
	p_rNorthWestChecks
    incase D_UP:
	p_rUpChecks
    incase D_DOWN:
	p_rDownChecks
    incase D_ENTER:
	p_rEnterChecks
    incase D_EXIT:
	p_rExitChecks
    default:
	Print("*** invalid direction number ");
	IPrint(dir);
	Print(" to 'DirChecks' ***\n");
	p_rNorthChecks
    esac
corp;

/*
 * DirDesc - map a direction onto the room property that would be a special
 *	description for that direction.
 */

define t_roomUtil proc utility public DirDesc(int dir)property string:
    case dir
    incase D_NORTH:
	p_rNorthDesc
    incase D_NORTHEAST:
	p_rNorthEastDesc
    incase D_EAST:
	p_rEastDesc
    incase D_SOUTHEAST:
	p_rSouthEastDesc
    incase D_SOUTH:
	p_rSouthDesc
    incase D_SOUTHWEST:
	p_rSouthWestDesc
    incase D_WEST:
	p_rWestDesc
    incase D_NORTHWEST:
	p_rNorthWestDesc
    incase D_UP:
	p_rUpDesc
    incase D_DOWN:
	p_rDownDesc
    incase D_ENTER:
	p_rEnterDesc
    incase D_EXIT:
	p_rExitDesc
    default:
	Print("*** invalid direction number ");
	IPrint(dir);
	Print(" to 'DirDesc' ***\n");
	p_rNorthDesc
    esac
corp;

/*
 * DirMessage - map a direction onto the property that would give a
 *	direction-specific message for going in that direction.
 */

define t_roomUtil proc utility public DirMessage(int dir)property string:
    case dir
    incase D_NORTH:
	p_rNorthMessage
    incase D_NORTHEAST:
	p_rNorthEastMessage
    incase D_EAST:
	p_rEastMessage
    incase D_SOUTHEAST:
	p_rSouthEastMessage
    incase D_SOUTH:
	p_rSouthMessage
    incase D_SOUTHWEST:
	p_rSouthWestMessage
    incase D_WEST:
	p_rWestMessage
    incase D_NORTHWEST:
	p_rNorthWestMessage
    incase D_UP:
	p_rUpMessage
    incase D_DOWN:
	p_rDownMessage
    incase D_ENTER:
	p_rEnterMessage
    incase D_EXIT:
	p_rExitMessage
    default:
	Print("*** invalid direction number ");
	IPrint(dir);
	Print(" to 'DirMessage' ***\n");
	p_rNorthMessage
    esac
corp;

/*
 * DirOMessage - map a direction onto a direction-specific room property
 *	that yields a message shown to observers when a character leaves
 *	in the given direction.
 */

define t_roomUtil proc utility public DirOMessage(int dir)property string:
    case dir
    incase D_NORTH:
	p_rNorthOMessage
    incase D_NORTHEAST:
	p_rNorthEastOMessage
    incase D_EAST:
	p_rEastOMessage
    incase D_SOUTHEAST:
	p_rSouthEastOMessage
    incase D_SOUTH:
	p_rSouthOMessage
    incase D_SOUTHWEST:
	p_rSouthWestOMessage
    incase D_WEST:
	p_rWestOMessage
    incase D_NORTHWEST:
	p_rNorthWestOMessage
    incase D_UP:
	p_rUpOMessage
    incase D_DOWN:
	p_rDownOMessage
    incase D_ENTER:
	p_rEnterOMessage
    incase D_EXIT:
	p_rExitOMessage
    default:
	Print("*** invalid direction number ");
	IPrint(dir);
	Print(" to 'DirOMessage' ***\n");
	p_rNorthOMessage
    esac
corp;

/*
 * DirEMessage - map a direction onto the direction-specific room property
 *	that yields a message to show to observers when someone arrives
 *	from that direction.
 */

define t_roomUtil proc utility public DirEMessage(int dir)property string:
    case dir
    incase D_NORTH:
	p_rNorthEMessage
    incase D_NORTHEAST:
	p_rNorthEastEMessage
    incase D_EAST:
	p_rEastEMessage
    incase D_SOUTHEAST:
	p_rSouthEastEMessage
    incase D_SOUTH:
	p_rSouthEMessage
    incase D_SOUTHWEST:
	p_rSouthWestEMessage
    incase D_WEST:
	p_rWestEMessage
    incase D_NORTHWEST:
	p_rNorthWestEMessage
    incase D_UP:
	p_rUpEMessage
    incase D_DOWN:
	p_rDownEMessage
    incase D_ENTER:
	p_rEnterEMessage
    incase D_EXIT:
	p_rExitEMessage
    default:
	Print("*** invalid direction number ");
	IPrint(dir);
	Print(" to 'DirEMessage' ***\n");
	p_rNorthEMessage
    esac
corp;


/* Routines for adding direction checkers. Note that there could easily be
   routines to delete the checkers, but I don't believe there should be any
   checkers on rooms that need to be deleted. The checker can check some
   state somewhere to make its effects conditional. It is NOT a good idea
   to have to modify and write out rooms as people wander through them.
   Later: people want them via the build facilites, so I've added them. */

define t_roomUtil proc utility AddNorthChecker(thing room; action a;
	bool front)void:
    addChecker(room, p_rNorthChecks, a, front);
corp;

define t_roomUtil proc utility DelNorthChecker(thing room; action a)void:
    delChecker(room, p_rNorthChecks, a);
corp;

define t_roomUtil proc utility AddSouthChecker(thing room; action a;
	bool front)void:
    addChecker(room, p_rSouthChecks, a, front);
corp;

define t_roomUtil proc utility DelSouthChecker(thing room; action a)void:
    delChecker(room, p_rSouthChecks, a);
corp;

define t_roomUtil proc utility AddEastChecker(thing room; action a;
	bool front)void:
    addChecker(room, p_rEastChecks, a, front);
corp;

define t_roomUtil proc utility DelEastChecker(thing room; action a)void:
    delChecker(room, p_rEastChecks, a);
corp;

define t_roomUtil proc utility AddWestChecker(thing room; action a;
	bool front)void:
    addChecker(room, p_rWestChecks, a, front);
corp;

define t_roomUtil proc utility DelWestChecker(thing room; action a)void:
    delChecker(room, p_rWestChecks, a);
corp;

define t_roomUtil proc utility AddNorthEastChecker(thing room; action a;
	bool front)void:
    addChecker(room, p_rNorthEastChecks, a, front);
corp;

define t_roomUtil proc utility DelNorthEastChecker(thing room; action a)void:
    delChecker(room, p_rNorthEastChecks, a);
corp;

define t_roomUtil proc utility AddNorthWestChecker(thing room; action a;
	bool front)void:
    addChecker(room, p_rNorthWestChecks, a, front);
corp;

define t_roomUtil proc utility DelNorthWestChecker(thing room; action a)void:
    delChecker(room, p_rNorthWestChecks, a);
corp;

define t_roomUtil proc utility AddSouthEastChecker(thing room; action a;
	bool front)void:
    addChecker(room, p_rSouthEastChecks, a, front);
corp;

define t_roomUtil proc utility DelSouthEastChecker(thing room; action a)void:
    delChecker(room, p_rSouthEastChecks, a);
corp;

define t_roomUtil proc utility AddSouthWestChecker(thing room; action a;
	bool front)void:
    addChecker(room, p_rSouthWestChecks, a, front);
corp;

define t_roomUtil proc utility DelSouthWestChecker(thing room; action a)void:
    delChecker(room, p_rSouthWestChecks, a);
corp;

define t_roomUtil proc utility AddUpChecker(thing room; action a;
	bool front)void:
    addChecker(room, p_rUpChecks, a, front);
corp;

define t_roomUtil proc utility DelUpChecker(thing room; action a)void:
    delChecker(room, p_rUpChecks, a);
corp;

define t_roomUtil proc utility AddDownChecker(thing room; action a;
	bool front)void:
    addChecker(room, p_rDownChecks, a, front);
corp;

define t_roomUtil proc utility DelDownChecker(thing room; action a)void:
    delChecker(room, p_rDownChecks, a);
corp;

define t_roomUtil proc utility AddEnterChecker(thing room; action a;
	bool front)void:
    addChecker(room, p_rEnterChecks, a, front);
corp;

define t_roomUtil proc utility DelEnterChecker(thing room; action a)void:
    delChecker(room, p_rEnterChecks, a);
corp;

define t_roomUtil proc utility AddExitChecker(thing room; action a;
	bool front)void:
    addChecker(room, p_rExitChecks, a, front);
corp;

define t_roomUtil proc utility DelExitChecker(thing room; action a)void:
    delChecker(room, p_rExitChecks, a);
corp;

define t_roomUtil proc utility AddDirChecker(thing room; int dir; action a;
	bool front)void:
    addChecker(room, DirChecks(dir), a, false);
corp;

define t_roomUtil proc utility DelDirChecker(thing room; int dir;action a)void:
    delChecker(room, DirChecks(dir), a);
corp;


/* routines for inserting checkers in rooms for specific verbs */

define t_roomUtil proc utility AddAnyEnterChecker(thing room; action a;
	bool front)void:
    addChecker(room, p_rAnyEnterChecks, a, front);
corp;

define t_roomUtil proc utility DelAnyEnterChecker(thing room; action a)void:
    delChecker(room, p_rAnyEnterChecks, a);
corp;

define t_roomUtil proc utility AddAnyEnterAction(thing room; action a;
	bool front)void:
    addChecker(room, p_rAnyEnterActions, a, front);
corp;

define t_roomUtil proc utility DelAnyEnterAction(thing room; action a)void:
    delChecker(room, p_rAnyEnterActions, a);
corp;

define t_roomUtil proc utility AddAnyLeaveChecker(thing room; action a;
	bool front)void:
    addChecker(room, p_rAnyLeaveChecks, a, front);
corp;

define t_roomUtil proc utility DelAnyLeaveChecker(thing room; action a)void:
    delChecker(room, p_rAnyLeaveChecks, a);
corp;

define t_roomUtil proc utility AddAnyLeaveAction(thing room; action a;
	bool front)void:
    addChecker(room, p_rAnyLeaveActions, a, front);
corp;

define t_roomUtil proc utility DelAnyLeaveAction(thing room; action a)void:
    delChecker(room, p_rAnyLeaveActions, a);
corp;


/* Some routines to actually do some special room checks. These are here
   so that the properties holding the checkers can be private */

define t_roomUtil proc utility DoDirChecks(thing room; int dir)status:
    DoChecks(room@DirChecks(dir))
corp;

define t_roomUtil proc utility DoRoomAnyEnterChecks(thing room)status:
    DoChecks(room@p_rAnyEnterChecks)
corp;

define t_roomUtil proc utility DoRoomAnyEnterActions(thing dest)void:
    DoThingActions(dest@p_rAnyEnterActions, dest);
corp;

define t_roomUtil proc utility DoRoomAnyLeaveChecks(thing room)status:
    DoChecks(room@p_rAnyLeaveChecks)
corp;

define t_roomUtil proc utility DoRoomAnyLeaveActions(thing here, dest)void:
    DoThingActions(here@p_rAnyLeaveActions, dest);
corp;

/***************************************************************************/

/* codes for use with LeaveRoomStuff and EnterRoomStuff */

define t_roomUtil MOVE_NORMAL	0$
define t_roomUtil MOVE_POOF 	1$
define t_roomUtil MOVE_SPECIAL	2$


/***************************************************************************/

/* some standard types of rooms */

define t_roomTypes r_indoors CreateThing(nil)$
r_indoors@p_lScenery :=
    "floor,ceiling,air.wall;north,south,east,west,n,s,e,w"$
AutoRoomGraphics(r_indoors, AutoClosedRoom)$
SetThingStatus(r_indoors, ts_readonly)$
r_indoors@p_oSmellString :=
    "The air here is not bad, but not as good as outdoors air."$
r_indoors@p_oListenString := "You hear nothing special here."$

define t_roomTypes r_outdoors CreateThing(nil)$
r_outdoors@p_lScenery := "ground,sky,air,sun,sunlight"$
AutoRoomGraphics(r_outdoors, AutoPaths)$
SetThingStatus(r_outdoors, ts_readonly)$
r_outdoors@p_oSmellString := "The air here is fairly fresh and pleasant."$
r_outdoors@p_oListenString := "You hear nothing special here."$

define t_roomTypes r_path CreateThing(r_outdoors)$
r_path@p_lScenery :=
    "ground,sky,air,sun,sunlight,path,pathway,trail,bush,bushes,grass,dust"$
AutoRoomGraphics(r_path, AutoPaths)$
SetThingStatus(r_path, ts_readonly)$
r_path@p_oSmellString := "There is a very faint dusty smell here."$
r_path@p_oListenString :=
    "You hear the gentle sound of a breeze in the greenery."$

define t_roomTypes r_road CreateThing(r_outdoors)$
r_road@p_lScenery := "ground,sky,air,sun,sunlight,road,dirt,alley,dust"$
AutoRoomGraphics(r_road, AutoRoads)$
SetThingStatus(r_road, ts_readonly)$
r_road@p_oSmellString := "There is a faint dusty smell here."$

define t_roomTypes r_forest CreateThing(r_outdoors)$
r_forest@p_lScenery :=
    "ground,sky,air,sun,sunlight,grass,bush,bushes,tree,leaf,leaves,foliage"$
AutoRoomGraphics(r_forest, AutoPaths)$
AutoRoomPens(r_forest, C_DARK_GREEN, C_BROWN, 0, 0)$
SetThingStatus(r_forest, ts_readonly)$
r_forest@p_oSmellString := "The air here smells of growing plants."$
r_forest@p_oListenString :=
    "You hear the gentle rustle of a breeze in the trees."$

define t_roomTypes r_field CreateThing(r_outdoors)$
r_field@p_lScenery :=
    "ground,sky,air,sun,sunlight,grass,field,flower,insect,pasture"$
AutoRoomGraphics(r_field, AutoOpenSpace)$
SetThingStatus(r_field, ts_readonly)$
r_field@p_oSmellString := "The air here smells of grass and flowers."$
r_field@p_oListenString :=
    "You hear the murmur of a breeze and occasional insect sounds."$

define t_roomTypes r_sidewalk CreateThing(r_outdoors)$
r_sidewalk@p_lScenery :=
    "ground,sky,air,sun,sunlight,sidewalk,pavement,road,dust"$
AutoRoomGraphics(r_sidewalk, AutoPaths)$
SetThingStatus(r_sidewalk, ts_readonly)$
r_sidewalk@p_oSmellString := "The air here smells a bit from the nearby road."$

define t_roomTypes r_park CreateThing(r_outdoors)$
r_park@p_lScenery :=
    "ground,sky,air,sun,sunlight,sidewalk,fountain,tree,bush,"
    "bushes,grass.park.flower"$
AutoRoomGraphics(r_park, AutoOpenSpace)$
SetThingStatus(r_park, ts_readonly)$
r_park@p_oSmellString := "The park air is fresher than typical outdoors air."$
r_park@p_oListenString :=
    "You can hear the sound of a breeze in the greenery."$

define t_roomTypes r_tunnel CreateThing(r_indoors)$
r_tunnel@p_lScenery :=
    "floor,ground,air,roof,ceiling,rock,stone."
    "side,wall;north,south,east,west,n,s,e,w"$
AutoRoomGraphics(r_tunnel, AutoTunnels)$
SetThingStatus(r_tunnel, ts_readonly)$
r_tunnel@p_oSmellString := "The tunnel air is somewhat stale."$
r_tunnel@p_oListenString :=
    "You hear nothing here but the slight murmur of air moving, and an "
    "occasional drop of water falling."$


/***************************************************************************/

/* some routines to help us build a world */

/*
 * ExtendDesc - add some stuff to a room description - this is used when
 *	adding a room to an already existing setup.
 */

define t_roomUtil proc utility public ExtendDesc(thing room; string desc)void:
    string s;

    s := room@p_rDesc;
    if s = "" then
	s := desc;
    else
	s := s + " " + desc;
    fi;
    room@p_rDesc := s;
corp;

/*
 * Sign - create a dummy sign.
 */

define t_roomUtil proc utility public Sign(thing room;
	string name, desc, text)void:
    thing sign;

    sign := CreateThing(nil);
    sign@p_oName := name;
    if desc ~= "" then
	sign@p_oDesc := desc;
    fi;
    sign@p_oReadString := text;
    sign@p_oNotGettable := true;
    sign@p_oInvisible := true;
    SetThingStatus(sign, ts_readonly);
    AddTail(room@p_lContents, sign);
corp;

/*
 * Several variants for setting up rooms. Note: the 'P' variants are the
 *	same as the non-P forms, except that they make the room public,
 *	i.e. add-to-able by others.
 *	The base variant takes the room, the name string, and an optional
 *	description string.
 */

define t_roomUtil proc utility public SetupRoom(thing room;
	string name, desc)void:

    room@p_lName := name;
    if desc ~= "" then
	room@p_rDesc := desc;
    fi;
    room@p_lContents := CreateThingList();
    room@p_rExits := CreateIntList();
    SetThingStatus(room, ts_readonly);
corp;

define t_roomUtil proc utility public SetupRoomP(thing room;
	string name, desc)void:

    room@p_lName := name;
    if desc ~= "" then
	room@p_rDesc := desc;
    fi;
    room@p_lContents := CreateThingList();
    room@p_rExits := CreateIntList();
    SetThingStatus(room, ts_wizard);
corp;

define t_roomUtil proc utility public SetupRoomD(thing room;
	string name, desc)void:

    room@p_lName := name;
    if desc ~= "" then
	room@p_rDesc := desc;
    fi;
    room@p_lContents := CreateThingList();
    room@p_rExits := CreateIntList();
    SetThingStatus(room, ts_readonly);
    room@p_lDark := true;
corp;

define t_roomUtil proc utility public SetupRoomDP(thing room;
	string name, desc)void:

    room@p_lName := name;
    if desc ~= "" then
	room@p_rDesc := desc;
    fi;
    room@p_lContents := CreateThingList();
    room@p_rExits := CreateIntList();
    SetThingStatus(room, ts_wizard);
    room@p_lDark := true;
corp;

/* This variant has no description, and a proc to provide the name.
   This is useful with many similar rooms, to avoid duplicating the
   name string. */

define t_roomUtil proc utility public SetupRoom2(thing room;
	action nameAction)void:

    room@p_lNameAction := nameAction;
    room@p_lContents := CreateThingList();
    room@p_rExits := CreateIntList();
    SetThingStatus(room, ts_readonly);
corp;

define t_roomUtil proc utility public SetupRoom2P(thing room;
	action nameAction)void:

    room@p_lNameAction := nameAction;
    room@p_lContents := CreateThingList();
    room@p_rExits := CreateIntList();
    SetThingStatus(room, ts_wizard);
corp;

/* Variant with name action, but a required description string */

define t_roomUtil proc utility public SetupRoom3(thing room; action nameAction;
	string desc)void:

    room@p_lNameAction := nameAction;
    room@p_rDesc := desc;
    room@p_lContents := CreateThingList();
    room@p_rExits := CreateIntList();
    SetThingStatus(room, ts_readonly);
corp;

define t_roomUtil proc utility public SetupRoom3P(thing room;
	action nameAction; string desc)void:

    room@p_lNameAction := nameAction;
    room@p_rDesc := desc;
    room@p_lContents := CreateThingList();
    room@p_rExits := CreateIntList();
    SetThingStatus(room, ts_wizard);
corp;

/* Similar utility to set up an object. Takes the object, an optional room
   to put it in and to make its home, the name string, and an optional
   description string. */

define t_roomUtil proc utility public SetupObject(thing object, where;
	string name, desc)void:

    object@p_oName := name;
    if desc ~= "" then
	object@p_oDesc := desc;
    fi;
    if where ~= nil then
	AddTail(where@p_lContents, object);
	object@p_oHome := where;
	object@p_oWhere := where;
    fi;
    SetThingStatus(object, ts_wizard);
corp;

/* this one makes a fake, non-gettable object */

define t_roomUtil proc utility public FakeObject(thing object, where;
	string name, desc)void:

    object@p_oName := name;
    if desc ~= "" then
	object@p_oDesc := desc;
    fi;
    object@p_oNotGettable := true;
    object@p_oInvisible := true;
    if where ~= nil then
	AddTail(where@p_lContents, object);
    fi;
    SetThingStatus(object, ts_wizard);
corp;

/*
 * DepositObject - add a clone of an object to the indicated room.
 */

define t_roomUtil proc utility public DepositClone(thing room, model)void:
    thing new;

    new := CreateThing(model);
    AddTail(room@p_lContents, new);
    new@p_oCreator := Me();
    new@p_oWhere := room;
    SetThingStatus(new, ts_public);
    GiveThing(new, SysAdmin);
corp;

/* Some utilities for making connections between rooms. 'Connect' is the
   most useful one. */

define t_roomUtil proc utility public UniConnect(thing r1, r2; int dir)void:
    list int exits;

    r1@DirProp(dir) := r2;
    exits := r1@p_rExits;
    if exits = nil then
	exits := CreateIntList();
	r1@p_rExits := exits;
    fi;
    AddTail(exits, dir);
corp;

define t_roomUtil proc utility public BiConnect(thing r1, r2;
	int dir1, dir2)void:
    list int exits;

    r1@DirProp(dir1) := r2;
    exits := r1@p_rExits;
    if exits = nil then
	exits := CreateIntList();
	r1@p_rExits := exits;
    fi;
    AddTail(exits, dir1);
    r2@DirProp(dir2) := r1;
    exits := r2@p_rExits;
    if exits = nil then
	exits := CreateIntList();
	r2@p_rExits := exits;
    fi;
    AddTail(exits, dir2);
corp;

define t_roomUtil proc utility public Connect(thing r1, r2; int dir)void:
    list int exits;

    r1@DirProp(dir) := r2;
    exits := r1@p_rExits;
    if exits = nil then
	exits := CreateIntList();
	r1@p_rExits := exits;
    fi;
    AddTail(exits, dir);
    dir := DirBack(dir);
    r2@DirProp(dir) := r1;
    exits := r2@p_rExits;
    if exits = nil then
	exits := CreateIntList();
	r2@p_rExits := exits;
    fi;
    AddTail(exits, dir);
corp;

/*
 * HConnect is the same as Connect, except that the connection is not
 *	added to the list of obvious exits.
 */

define t_roomUtil proc utility public HConnect(thing r1, r2; int dir)void:

    r1@DirProp(dir) := r2;
    r2@DirProp(DirBack(dir)) := r1;
corp;

/*
 * HUniConnect - a hidden one-way connection.
 */

define t_roomUtil proc utility public HUniConnect(thing r1, r2; int dir)void:

    r1@DirProp(dir) := r2;
corp;


/***************************************************************************/

/* some routines useful for verbs */

/*
 * ShowExits - show the obvious exits from the current room.
 */

define t_roomUtil proc public ShowExits(thing room)void:
    list int li;
    int count, i, oldIndent;

    li := room@p_rExits;
    if li ~= nil then
	count := Count(li);
	if count = 0 then
	    Print("There are no obvious exits.\n");
	else
	    Print("Obvious exits: ");
	    oldIndent := GetIndent();
	    SetIndent(oldIndent + 2);
	    for i from 0 upto count - 1 do
		Print(ExitName(li[i]));
		if i ~= count - 1 then
		    Print(" ");
		fi;
	    od;
	    SetIndent(oldIndent);
	    Print("\n");
	fi;
    else
	Print("There are no obvious exits.\n");
    fi;
corp;

/* NOTE: for the following sets of routines, the code usually just calls the
   the inner (one/once) routine directly via ForEachAgent, just to save one
   interpreted subroutine call. */

/*
 * ShowAgents - show who/what is in the current room.
 */

define t_roomUtil proc public ShowOneAgent(thing th)void:

    if th@p_pName ~= "" and not th@p_pHidden then
	Print(Capitalize(CharacterNameG(th)) + " is " + 
		ShowPosition(th) + "\n");
	GShowIcon(nil, th, not th@p_pStandard, Parent(th) ~= nil);
    fi;
corp;

define t_roomUtil proc public ShowAgents()void:
    int saveComponent;

    saveComponent := DrawComponent(nil, COMPONENT_ICONS);
    ForEachAgent(Here(), ShowOneAgent);
    ignore DrawComponent(nil, saveComponent);
corp;

/*
 * ShowRoomTo - show the room to someone who is in it.
 */

define t_roomUtil proc ShowRoomToMe(bool full)bool:
    thing me, room;
    action a;
    string s;
    status st;
    int saveComponent;

    me := Me();
    room := Here();
    a := room@p_lNameAction;
    if a = nil then
	s := room@p_lName;
    else
	s := call(a, string)();
    fi;
    if s ~= "" then
	Print("You are " + s + ".\n");
    fi;
    /* looking around the room can "fail" or "succeed" */
    st := DoLocationLookChecks(room);
    if st ~= fail then
	if st = continue then
	    if full or me@p_pVerbose then
		/* it may seem redundant to allow for both 'rLookChecks'
		   and 'rDescAction', but what the heck! */
		a := room@p_rDescAction;
		if a = nil then
		    s := room@p_rDesc;
		    if s = "" then
			Print("You see nothing special here.\n");
		    else
			Print(s + "\n");
		    fi;
		    ShowExits(room);
		else
		    Print(call(a, string)() + "\n");
		fi;
	    elif not me@p_pSuperBrief then
		ShowExits(room);
	    fi;
	fi;
	ignore ShowList(room@p_lContents, "Nearby:\n");
	if st = continue and GOn(nil) then
	    saveComponent := DrawComponent(nil, COMPONENT_ICONS);
	    GResetIcons(nil);
	    ForEachAgent(room, ShowOneAgent);
	    ignore DrawComponent(nil, COMPONENT_DRAW);
	    a := room@p_rEnterRoomDraw;
	    if a ~= nil then
		RemoveCursor();
		call(a, void)();
	    elif room@p_Image ~= "" then
		ShowImage(room@p_Image);
	    fi;
	    ignore DrawComponent(nil, saveComponent);
	else
	    ForEachAgent(room, ShowOneAgent);
	fi;
	a := room@p_rFurtherDesc;
	if a ~= nil then
	    call(a, void)();
	fi;
	true
    else
	false
    fi
corp;

define tp_roomUtil proc doShowRoom()status:

    ignore ShowRoomToMe(false);
    continue
corp;

define t_roomUtil proc ShowRoomToAgent(thing who)void:

    /* no need to do anything for machines */
    if who ~= Me() and ThingCharacter(who) ~= nil then
	ignore ForceAction(who, doShowRoom);
    fi;
corp;

define t_roomUtil proc UnShowRoomFromMe()void:
    action a;
    int saveComponent;

    if GOn(nil) then
	saveComponent := DrawComponent(nil, COMPONENT_ICONS);
	GResetIcons(nil);
	ignore DrawComponent(nil, saveComponent);
	a := Here()@p_rLeaveRoomDraw;
	if a ~= nil then
	    call(a, void)(nil);
	fi;
    fi;
corp;

define tp_roomUtil proc doUnShowRoom()status:

    UnShowRoomFromMe();
    continue
corp;

define t_roomUtil proc UnShowRoomFromAgent(thing who)void:

    if who ~= Me() and ThingCharacter(who) ~= nil then
	/* only do it for players */
	ignore ForceAction(who, doUnShowRoom);
    fi;
corp;

/* general code dealing with entering and exiting rooms, moving, etc. */

define t_roomUtil proc EnterRoomStuff(thing dest; int fromDir, moveKind)void:
    thing me;
    bool lightThere;
    string name, s;

    DoRoomAnyEnterActions(dest);
    me := Me();
    lightThere := LightAt(dest);
    SetLocation(dest);
    if lightThere or me@p_oLight or FindFlagOnList(me@p_pCarrying, p_oLight)
    then
	ignore ShowRoomToMe(false);
	/* if I can't see, neither can anyone else */
	if not me@p_pHidden then
	    ForEachAgent(dest, ShowIconOnce);
	    name := Capitalize(CharacterNameG(me));
	    case moveKind
	    incase MOVE_NORMAL:
		if fromDir >= 0 then
		    s := dest@(DirEMessage(fromDir));
		    if s = "" then
			OPrint(name + " has arrived from " +
				DirName(fromDir) + ".\n");
		    elif s ~= "." then
			OPrint(name + " " + s + "\n");
		    fi;
		fi;
	    incase MOVE_POOF:
		OPrint(name + " *POOFS* in.\n");
	    esac;
	else
	    if moveKind = MOVE_POOF then
		OPrint("*POOF*\n");
	    fi;
	fi;
	if not lightThere then
	    ForEachAgent(dest, ShowRoomToAgent);
	fi;
    else
	Print("It is dark here.\n");
	if dest@p_rEnterRoomDraw ~= nil and GOn(nil) then
	    UnShowRoomFromMe();
	fi;
    fi;
corp;

/*
 * similar stuff for the going out of a room half.
 */

define t_roomUtil proc LeaveRoomStuff(thing dest; int dir, moveKind)void:
    thing me, here;
    action a;
    string name, s;
    bool lightHere;
    int saveComponent;

    me := Me();
    here := Here();
    SetLocation(nil);
    lightHere := LightAt(here);
    if lightHere or me@p_oLight or FindFlagOnList(me@p_pCarrying, p_oLight)
    then
	name := Capitalize(CharacterNameG(me));
	/* have to use ABPrint, since we have done SetLocation(nil) */
	case moveKind
	incase MOVE_NORMAL:
	    if not me@p_pHidden then
		s := here@(DirOMessage(dir));
		if s = "" then
		    ABPrint(here, me, me,
			name + " has left to " + DirName(dir) + ".\n");
		elif s ~= "." then
		    ABPrint(here, me, me, name + " " + s + "\n");
		fi;
		CheckFollowers(dir);
	    fi;
	    /* we keep the rest in case he/she went hidden while here */
	    s := here@(DirMessage(dir));
	    if s ~= "" then
		NPrint(s);
	    fi;
	incase MOVE_POOF:
	    if me@p_pHidden then
		ABPrint(here, me, me, "*POOF*\n");
	    else
		ABPrint(here, me, me, name + " *POOFS* out.\n");
	    fi;
	    Print("*POOF*\n");
	    ClearFollowers(me);
	    UnFollow();
	esac;
	ForEachAgent(here, UnShowIconOnce);
	if GOn(nil) then
	    saveComponent := DrawComponent(nil, COMPONENT_ICONS);
	    GResetIcons(nil);
	    ignore DrawComponent(nil, saveComponent);
	    a := here@p_rLeaveRoomDraw;
	    if a ~= nil then
		call(a, void)(dest);
	    fi;
	fi;
	if not lightHere then
	    ForEachAgent(here, UnShowRoomFromAgent);
	fi;
    fi;
    /* put the location back to the old room for DoRoomAnyEnterActions */
    SetLocation(here);
corp;

/*
 * EnterRoom - called whenever the player finally enters a given room.
 *	Returns 'true' if all is well and we should continue with commands.
 */

define t_roomUtil proc public EnterRoom(thing dest; int dir, moveType)bool:

    LeaveRoomStuff(dest, dir, moveType);
    EnterRoomStuff(dest, DirBack(dir), moveType);
    ignore DoPlayerEnterChecks(Me());
    /* allow for funny things like instant teleports out, etc. */
    DoRoomAnyEnterChecks(dest) ~= fail
corp;

/*
 * RoomDoMove - top level routine to attempt to move in the given direction.
 *	Note: not suitable for other than called by the player.
 *	Other Note: it should work indirectly for machines, e.g. when
 *	    someone does 'say Packrat go north'.
 */

define t_roomUtil proc RoomDoMove(int dir)bool:
    thing me, here, dest;
    string s;
    action a;
    property list action checks;

    me := Me();
    if me@p_pPosition ~= POS_NONE then
	Print("You are still " + ShowPosition(me) + "\n");
	false
    else
	here := Here();
	dest := here@(DirProp(dir));
	checks := DirChecks(dir);
	if dest = nil then
	    a := here@p_rNoGoAction;
	    if a ~= nil then
		call(a, void)(dir);
	    else
		s := here@p_rNoGoString;
		if s ~= "" then
		    NPrint(s);
		else
		    Print("You can't go in that direction.\n");
		fi;
	    fi;
	    false
	elif dest@p_lLocked and CharacterThing(Owner(dest)) ~= me and
	    me ~= CharacterThing(SysAdmin)
	then
	    Print("The owner of that location has locked it.\n");
	    false
	elif (dest@p_lName = "" and dest@p_lNameAction = nil or
	    dest@p_rExits = nil or dest@p_lContents = nil) and
	    (here@checks = nil or Count(here@checks) = 0)
	then
	    Print("The location in that direction is not usable.\n");
	    false
	elif DoPlayerLeaveChecks(me, dir) ~= fail and
	    DoChecks(here@checks) ~= fail and
	    DoRoomAnyLeaveChecks(here) ~= fail
	then
	    /* nothing was blocking the exit, nothing abnormal happened
	       when we tried to go through, and nothing funny like player
	       being chained to the floor */
	    EnterRoom(dest, dir, MOVE_NORMAL)
	else
	    false
	fi
    fi
corp;

/*
 * RoomTryToMove - similar to 'RoomDoMove', but intended for machines.
 *	Returns 'true' if it can do the move.
 *	Note: intended only for calling by the machine doing the moving.
 */

define t_roomUtil proc public RoomTryToMove(int dir)bool:
    thing here, there, me;

    here := Here();
    me := Me();
    there := here@(DirProp(dir));
    if there = nil or
	there@p_lName = "" and there@p_lNameAction = nil or
	there@p_rExits = nil or there@p_lContents = nil
    then
	false
    else
	not there@p_lNoMachines and not there@p_lLocked and
	    me@p_pPosition = POS_NONE and
	    DoPlayerLeaveChecks(me, dir) ~= fail and
	    DoChecks(here@(DirChecks(dir))) ~= fail and
	    DoRoomAnyLeaveChecks(here) ~= fail
    fi
corp;

/*
 * RoomMachineMove - do the other half of 'RoomDoMove' for machines. This does
 *	NOT use any of the exit checkers. It should be used with
 *	'RoomTryToMove', and not by itself.
 *	Note: intended only to be called by the machine doing the move.
 */

define t_roomUtil proc public RoomMachineMove(int dir)void:
    thing here, me, dest;
    string myName, s;
    bool otherLight, meLight;

    here := Here();
    me := Me();
    myName := Capitalize(CharacterNameG(me));
    dest := here@(DirProp(dir));
    if dest ~= nil then
	meLight := me@p_oLight or FindFlagOnList(me@p_pCarrying, p_oLight);
	otherLight := LightAt(here);
	if otherLight or meLight then
	    if not me@p_pHidden then
		s := here@(DirOMessage(dir));
		if s = "" then
		    OPrint(myName + " has left to " + DirName(dir) + ".\n");
		elif s ~= "." then
		    OPrint(myName + " " + s + "\n");
		fi;
		CheckFollowers(dir);
	    fi;
	    if not otherLight then
		ForEachAgent(here, UnShowRoomFromAgent);
	    else
		/* we keep this in case it went hidden while here */
		ForEachAgent(here, UnShowIconOnce);
	    fi;
	fi;
	/* The following is a subset of EnterRoom. It is cheaper, since the
	   machine doesn't have to be told about the room and what and who
	   is in it. */
	SetLocation(nil);
	otherLight := LightAt(dest);
	SetLocation(dest);
	if otherLight or meLight then
	    dir := DirBack(dir);
	    s := dest@(DirEMessage(dir));
	    if s = "" then
		OPrint(myName + " has arrived from " + DirName(dir) + ".\n");
	    elif s ~= "." then
		OPrint(myName + " " + s + "\n");
	    fi;
	    if not otherLight then
		ForEachAgent(dest, ShowRoomToAgent);
	    else
		ForEachAgent(dest, ShowIconOnce);
	    fi;
	fi;
	/* allow monster gen, etc. */
	ignore DoPlayerEnterChecks(me);
	/* allow for funny things like instant teleports out, etc. */
	ignore DoRoomAnyEnterChecks(dest);
    else
	OPrint("BUGGY MACHINE " + myName + " IS STUCK HERE.\n");
    fi;
corp;

/*
 * GetAgents - return a string which is a comma separated list of the
 *	names of the agents in the given room.
 */

define tp_roomUtil p_pAgentList CreateStringProp()$

define tp_roomUtil proc addAnAgent(thing agent)void:
    thing me;
    string name, s;

    me := Me();
    name := agent@p_pName;
    if name ~= "" and not agent@p_pHidden then
	name := FormatName(name);
	s := me@p_pAgentList;
	if s ~= "" then
	    s := s + ", ";
	fi;
	s := s + name;
	me@p_pAgentList := s;
    fi;
corp;

define t_roomUtil proc GetAgents(thing room)string:
    string s;

    ForEachAgent(room, addAnAgent);
    s := Me()@p_pAgentList;
    Me() -- p_pAgentList;
    s
corp;

/*
 * ResetObjects - go through a list of objects and put them back where they
 *	belong. This is useful for single-user-at-a-time quests.
 */

define t_roomUtil proc utility public ResetObjects(list thing lt)void:
    int count;
    thing object, home, now;

    count := Count(lt);
    while count ~= 0 do
	count := count - 1;
	object := lt[count];
	home := object@p_oHome;
	if home ~= nil then
	    now := object@p_oWhere;
	    if now ~= nil then
		if now ~= home then
		    AddTail(home@p_lContents, object);
		    if now@p_lContents ~= nil then
			DelElement(now@p_lContents, object);
		    else
			DelElement(now@p_oContents, object);
		    fi;
		    object@p_oWhere := home;
		fi;
	    else
		now := object@p_oCarryer;
		if now = nil then
		    Print("An object being reset isn't anywhere!\n"
			"Please inform the owner of this quest.\n");
		else
		    AddTail(home@p_lContents, object);
		    DelElement(now@p_pCarrying, object);
		    object -- p_oCarryer;
		    object@p_oWhere := home;
		    /* Specifically SPrint, so that other players won't see
		       the objects, perhaps secret to the quest, go away.
		       Not just 'Print' in case the quest allows the object
		       to be given to someone else. */
		    SPrint(now, FormatName(object@p_oName) + " vanishes.\n");
		fi;
	    fi;
	fi;
    od;
corp;


/***************************************************************************/

/*
 * AddLight - introduce a source of light to the current room.
 */

define t_roomUtil proc public AddLight()void:
    if not CanSee(Here(), Me()) then
	if DoLocationLightChecks(Here()) ~= fail then
	    ignore ShowRoomToMe(false);
	    ForEachAgent(Here(), ShowRoomToAgent);
	fi;
    fi;
corp;

/*
 * ActiveLightObject - an object is made to now emit light.
 */

define t_roomUtil proc public ActiveLightObject()status:
    thing it;
    string name;

    it := It();
    name := FormatName(it@p_oName);
    if it@p_oLight then
	Print("The " + name + " is already lit.\n");
	fail
    else
	Print("You light the " + name + ".\n");
	OPrint(Capitalize(CharacterNameG(Me())) +
	       AAn(" lights", name) + ".\n");
	AddLight();
	it@p_oLight := true;
	/* want these to be succeed to allow proper use with VerbHere, etc. */
	succeed
    fi
corp;

/*
 * RemoveLight - remove a source of light from the current room.
 */

define t_roomUtil proc public RemoveLight()void:
    if not CanSee(Here(), Me()) then
	UnShowRoomFromMe();
	ForEachAgent(Here(), UnShowRoomFromAgent);
    fi;
corp;

/*
 * ActiveUnLightObject - an object is made to no longer emit light.
 */

define t_roomUtil proc public ActiveUnLightObject()status:
    thing it;
    string name;

    it := It();
    name := FormatName(it@p_oName);
    if not it@p_oLight then
	Print("The " + name + " is not lit.\n");
	fail
    else
	Print("You extinguish the " + name + ".\n");
	it@p_oLight := false;
	RemoveLight();
	succeed
    fi
corp;

/*
 * PassiveUnLightObject - an object is going out, independent of any player.
 */

define t_roomUtil proc public PassiveUnLightObject(thing object)void:
    thing who, where;
    character ch;

    object@p_oLight := false;
    who := object@p_oCarryer;
    where := object@p_oWhere;
    if who ~= nil then
	SPrint(who, "Your " + FormatName(object@p_oName) + " has gone out.\n");
	ch := Character(who@p_pName);
	if ch ~= nil and CharacterThing(ch) = who then
	    where := CharacterLocation(ch);
	    if not LightAt(where) then
		ForEachAgent(where, UnShowRoomFromAgent);
	    fi;
	fi;
    elif where ~= nil and not LightAt(where) then
	ForEachAgent(where, UnShowRoomFromAgent);
    fi;
corp;

define t_roomUtil proc JumpToRoom(thing theRoom; int fromDir)void:

    call(Room@p_lmResumeMode, void)();
    EnterRoomStuff(theRoom, fromDir, MOVE_NORMAL);
corp;

/* create the arrivals room */

public r_arrivals CreateThing(r_indoors)$
SetupRoom(r_arrivals, "in the arrivals room",
    "This room is where new players enter the game.")$

/* create the lost and found room */
public r_lostAndFound CreateThing(r_indoors)$
SetupRoom(r_lostAndFound, "in the lost and found room",
    "Things lost often end up here.")$

unuse tp_roomUtil
