/*
 * tileBase.m - basic properties and code for use with tile graphics worlds.
 *
 * Copyright 1999 by Chris Gray
 */

private t_tileGraphics CreateTable()$
use t_tileGraphics

private tp_tileGraphics CreateTable()$
use tp_tileGraphics

/* Manifest constants. */

define t_tileGraphics TILE_WIDTH	32$
define t_tileGraphics TILE_HEIGHT	32$
define t_tileGraphics TILE_BITS		13$
define t_tileGraphics TILE_RANGE	1 << TILE_BITS$
define t_tileGraphics TILE_MASK		TILE_RANGE - 1$
define t_tileGraphics NO_GO_BIT		0x2000$
define t_tileGraphics OPAQUE_BIT	0x4000$
define t_tileGraphics ROOF_BIT		0x8000$
define t_tileGraphics TOTAL_BITS	TILE_BITS + 1 + 1 + 1$
define t_tileGraphics TOTAL_MASK	(1 << TOTAL_BITS) - 1$
define t_tileGraphics TILES_PER_INT	32 / TOTAL_BITS$

define t_tileGraphics TILE_BASE_CHARACTER		10000$

/* Properties that are used in each "field": */

define t_tileGraphics p_fRows CreateIntProp()$
define t_tileGraphics p_fCols CreateIntProp()$
define t_tileGraphics p_fRowArray CreateThingListProp()$
define t_tileGraphics p_fColArray CreateIntListProp()$
define t_tileGraphics p_fHashArray CreateIntListProp()$
define t_tileGraphics p_fCellArray CreateThingListProp()$
define t_tileGraphics p_fCharacters CreateThingListProp()$
define t_tileGraphics p_fTileArray CreateThingListProp()$
define t_tileGraphics p_fIndexArray CreateIntListProp()$
define t_tileGraphics p_fName1 CreateStringProp()$
define t_tileGraphics p_fName2 CreateStringProp()$

/* Properties used on tiles in the field. */

define t_tileGraphics p_tPixels CreateIntListProp()$
define t_tileGraphics p_tTransColour CreateIntProp()$
define t_tileGraphics p_tBGId CreateIntProp()$

/* Properties that are used on individual special "cells": */

define t_tileGraphics p_cContents CreateThingListProp()$
define t_tileGraphics p_cOccupants CreateThingListProp()$
define t_tileGraphics p_cRoom CreateThingProp()$
define t_tileGraphics p_cSpecialAction CreateActionProp()$

/* Properties that can be used on the multi-cell "rooms": */

define t_tileGraphics p_rRow CreateIntProp()$
define t_tileGraphics p_rCol CreateIntProp()$
define t_tileGraphics p_rRows CreateIntProp()$
define t_tileGraphics p_rCols CreateIntProp()$

/* Properties attached to the player: */

define t_tileGraphics p_pCharacterTile CreateIntProp()$
define t_tileGraphics p_pCurrentField CreateThingProp()$
define t_tileGraphics p_pCurrentRow CreateIntProp()$
define t_tileGraphics p_pCurrentCol CreateIntProp()$
define t_tileGraphics p_pWindowRow CreateIntProp()$
define t_tileGraphics p_pWindowCol CreateIntProp()$
define t_tileGraphics p_pPosVal CreateIntProp()$
define t_tileGraphics p_pKnownTiles CreateIntListProp()$
define t_tileGraphics p_pCurrentRoom CreateThingProp()$

/* Properties attached to objects: */

define t_tileGraphics p_oObjectTile CreateIntProp()$

/* Properties used on saved locations: */

define tp_tileGraphics p_lSavedField CreateThingProp()$
define tp_tileGraphics p_lSavedRow CreateIntProp()$
define tp_tileGraphics p_lSavedCol CreateIntProp()$

define t_tileGraphics proc TileCreateField(int rows, cols;
	string name1, name2)thing:
    thing theField, theRow;
    int ints;

    ints := (cols + TILES_PER_INT - 1) / TILES_PER_INT;
    theField := CreateThing(nil);
    theField@p_fRows := rows;
    theField@p_fCols := cols;
    theField@p_fRowArray := CreateThingList();
    while rows ~= 0 do
	theRow := CreateThing(nil);
	AddTail(theField@p_fRowArray, theRow);
	theRow@p_fColArray := CreateIntArray(ints);
	rows := rows - 1;
    od;
    theField@p_fHashArray := CreateIntList();
    theField@p_fCellArray := CreateThingList();
    theField@p_fCharacters := CreateThingList();
    theField@p_fTileArray := CreateThingList();
    theField@p_fIndexArray := CreateIntList();
    theField@p_fName1 := name1;
    if name2 ~= "" then
	theField@p_fName2 := name2;
    fi;
    SetThingStatus(theField, ts_readonly);
    theField
corp;

define t_tileGraphics proc doDecode(action decoder; string ch; int row,col)int:

    call(decoder, int)(ch, row, col)
corp;

define t_tileGraphics proc setupData(thing theField;
	int startRow, startCol, rows, cols;
	action decodeTile; string tiles)void:
    list thing rowArray;
    list int colArray;
    int row, col, pos, oldBits, newBits, lastDecode;
    string ch, lastCh;

    rowArray := theField@p_fRowArray;    
    lastCh := "";
    pos := 0;
    for row from startRow upto startRow + rows - 1 do
	colArray := rowArray[row]@p_fColArray;
	oldBits := colArray[startCol / TILES_PER_INT];
	newBits := 0;
	for col from startCol upto startCol + cols - 1 do
	    if col % TILES_PER_INT = 0 then
		oldBits := colArray[col / TILES_PER_INT];
	    fi;
	    newBits := newBits << TOTAL_BITS;

	    ch := SubString(tiles, pos, 1);
	    if ch ~= lastCh then
		lastDecode := doDecode(decodeTile, ch, row, col);
		lastCh := ch;
	    fi;
	    newBits := newBits | lastDecode;

	    pos := pos + 1;
	    if col % TILES_PER_INT = TILES_PER_INT - 1 then
		colArray[col / TILES_PER_INT] := oldBits | newBits;
		newBits := 0;
	    fi;
	od;
	col := startCol + cols;
	if col % TILES_PER_INT ~= 0 then
	    colArray[col / TILES_PER_INT] := oldBits |
	       newBits << (TOTAL_BITS * (TILES_PER_INT - col % TILES_PER_INT));
	fi;
    od;
corp;

define t_tileGraphics proc TileSetRectangle(thing theField;
	int startRow, startCol, rows, cols;
	action decodeTile; string tiles)void:

    if Length(tiles) ~= rows * cols then
	Print("tiles string length (" + IntToString(Length(tiles)) +
	    ") not equal to rows (" + IntToString(rows) +
	    ") * cols (" + IntToString(cols) + ").\n");
    elif startRow < 0 then
	Print("startRow (" + IntToString(startRow) + ") negative!\n");
    elif startCol < 0 then
	Print("startCol (" + IntToString(startCol) + ") negative!\n");
    elif startRow + rows > theField@p_fRows then
	Print("startRow (" + IntToString(startRow) + ") + rows (" +
	    IntToString(rows) + ") > field rows (" +
	    IntToString(theField@p_fRows) + ").\n");
    elif startCol + cols > theField@p_fCols then
	Print("startCol (" + IntToString(startCol) + ") + cols (" +
	    IntToString(cols) + ") > field cols (" +
	    IntToString(theField@p_fCols) + ").\n");
    else
	setupData(theField, startRow, startCol, rows, cols, decodeTile, tiles);
    fi;
corp;

define t_tileGraphics proc TileSetTile(thing theField; int row, col, tile)void:
    list int colArray;
    int bits, shift;

    tile := tile & TILE_MASK;    
    colArray := theField@p_fRowArray[row]@p_fColArray;
    bits := colArray[col / TILES_PER_INT];
    shift := TOTAL_BITS * (TILES_PER_INT - 1 - col % TILES_PER_INT);
    bits := (bits & ~ (TILE_MASK << shift)) | (tile << shift);
    colArray[col / TILES_PER_INT] := bits;
corp;

define t_tileGraphics proc TileSetAll(thing theField; int row, col, tile)void:
    list int colArray;
    int bits, shift;
    
    colArray := theField@p_fRowArray[row]@p_fColArray;
    bits := colArray[col / TILES_PER_INT];
    shift := TOTAL_BITS * (TILES_PER_INT - 1 - col % TILES_PER_INT);
    bits := (bits & ~ (TOTAL_MASK << shift)) | (tile << shift);
    colArray[col / TILES_PER_INT] := bits;
corp;

define t_tileGraphics proc TileAddTile(thing theField;
	int index, transColour, bgId; list int definition)void:
    thing th;

    th := CreateThing(nil);
    th@p_tPixels := definition;
    if transColour ~= 0 then
	th@p_tTransColour := transColour;
    fi;
    if bgId ~= 0 then
	th@p_tBGId := bgId;
    fi;
    SetThingStatus(th, ts_readonly);
    AddTail(theField@p_fTileArray, th);
    AddTail(theField@p_fIndexArray, index);
corp;

define t_tileGraphics proc TileRoom(string name1, name2;
	int row, col, rows, cols)thing:
    thing room;

    room := CreateThing(nil);
    if name1 ~= "" then
	room@p_lName1 := name1;
	if name2 ~= "" then
	    room@p_lName2 := name2;
	fi;
    fi;
    room@p_rRow := row;
    room@p_rCol := col;
    room@p_rRows := rows;
    room@p_rCols := cols;
    SetThingStatus(room, ts_readonly);
    room
corp;

define t_tileGraphics proc utility public TileConnect(thing r1, r2;
	int dir)void:

    r1@DirProp(dir) := r2;
    dir := DirBack(dir);
    r2@DirProp(dir) := r1;
corp;

define t_tileGraphics proc TileAddPicture(thing room; int dir;string name)void:

    if dir >= D_NORTH and dir <= D_HERE then
	room@(DirPictureProp(dir)) := name;
    else
	Print("Invalid direction ");
	IPrint(dir);
	Print(" to TileAddPicture.\n");
    fi;
corp;

define t_tileGraphics proc tileHash(thing theField; int row, col)int:

    row * (theField@p_fCols + 1) + col
corp;

define t_tileGraphics proc tileFindCell(thing theField; int row, col)thing:
    thing theThing;
    int hash, index;

    hash := tileHash(theField, row, col);
    index := FindElement(theField@p_fHashArray, hash);
    if index = -1 then
	nil
    else
	theField@p_fCellArray[index]
    fi
corp;

define t_tileGraphics proc maybeFreeCell(thing theField, theCell;
	int row, col)void:

    if theCell@p_cContents = nil and theCell@p_cOccupants = nil and
	theCell@p_cRoom = nil and theCell@p_cSpecialAction = nil
    then
	DelElement(theField@p_fHashArray, tileHash(theField, row, col));
	DelElement(theField@p_fCellArray, theCell);
    fi;
corp;

define t_tileGraphics proc tileGetCell(thing theField; int row, col)thing:
    thing theThing;
    int hash, index;

    hash := tileHash(theField, row, col);
    index := FindElement(theField@p_fHashArray, hash);
    if index = -1 then
	theThing := CreateThing(nil);
	AddTail(theField@p_fHashArray, hash);
	AddTail(theField@p_fCellArray, theThing);
	SetThingStatus(theThing, ts_readonly);
	theThing
    else
	theField@p_fCellArray[index]
    fi
corp;

define t_tileGraphics proc TileSetRoom(thing theField; int row, col;
	thing theRoom)void:

    tileGetCell(theField, row, col)@p_cRoom := theRoom;
corp;

define t_tileGraphics proc TileSetSpecialAction(thing theField; int row, col;
	action theAction)void:

    tileGetCell(theField, row, col)@p_cSpecialAction := theAction;
corp;

define t_tileGraphics proc simpleTile(int pen)list int:
    list int li;
    int i;
    
    li := CreateIntArray(TILE_HEIGHT * TILE_WIDTH / 4);
    for i from 0 upto TILE_HEIGHT * TILE_WIDTH / 4 - 1 do
	li[i] := pen << 24 | pen << 16 | pen << 8 | pen;
    od;
    li
corp;

define t_tileGraphics proc TileGetTile(thing theField; int index)list int:
    int ind;

    ind := FindElement(theField@p_fIndexArray, index);
    if ind = -1 then
	nil
    else
	theField@p_fTileArray[ind]@p_tPixels
    fi
corp;

define t_tileGraphics proc getBGId(thing theField; int index)int:
    int ind;

    ind := FindElement(theField@p_fIndexArray, index);
    if ind = -1 then
	0
    else
	theField@p_fTileArray[ind]@p_tBGId
    fi
corp;

define t_tileGraphics proc doDisplayTile(thing who; int index)void:
    thing theField, th;
    int ind;

    if FindElement(who@p_pKnownTiles, index) = -1 then
	theField := who@p_pCurrentField;
	ind := FindElement(theField@p_fIndexArray, index);
	if ind ~= -1 then
	    th := theField@p_fTileArray[ind];
	    if th@p_tTransColour = -1 then
		GDefineTile(who, index, TILE_WIDTH, TILE_HEIGHT,th@p_tPixels);
	    else
		GDefineOverlayTile(who, index, th@p_tTransColour,
		    TILE_WIDTH, TILE_HEIGHT, th@p_tPixels);
	    fi;
	else
	    Print("Tile " + IntToString(index) + " not found in field\n");
	    GDefineTile(who, index, TILE_WIDTH, TILE_HEIGHT,simpleTile(index));
	fi;
	AddTail(who@p_pKnownTiles, index);
    fi;
    GDisplayTile(who, index);
corp;

define t_tileGraphics proc TileAddObject(thing theField, theObject;
	int row, col)void:
    thing theCell;
    list thing li;

    theCell := tileGetCell(theField, row, col);
    li := theCell@p_cContents;
    if li = nil then
	li := CreateThingList();
	theCell@p_cContents := li;
    fi;
    AddTail(li, theObject);
corp;

define t_tileGraphics proc TileDelObject(thing theField, theObject;
	int row, col)void:
    list thing lt;
    thing theCell;

    theCell := tileGetCell(theField, row, col);
    lt := theCell@p_cContents;
    DelElement(lt, theObject);
    if Count(lt) = 0 then
	theCell -- p_cContents;
    fi;
    maybeFreeCell(theField, theCell, row, col);
corp;

define t_tileGraphics proc tileGetBits(thing theField; int row, col)int:
    list thing rowArray;
    int bits;

    rowArray := theField@p_fRowArray;
    bits := rowArray[row]@p_fColArray[col / TILES_PER_INT];
    bits := bits >> (TOTAL_BITS * (TILES_PER_INT - 1 - (col % TILES_PER_INT)));
    bits & TOTAL_MASK
corp;

define t_tileGraphics proc tileShowCell(thing who, theField;
	int cursorRow, cursorCol, fieldRow, fieldCol)void:
    thing theCell, object, occupant;
    int bits, count, bgId;
    list thing lt;

    if GOn(who) then
	bits := tileGetBits(theField, fieldRow, fieldCol);    
	GAMovePixels(who, cursorCol * TILE_WIDTH, cursorRow * TILE_HEIGHT);
	bgId := getBGId(theField, bits & TILE_MASK);
	if bgId ~= 0 then
	    doDisplayTile(who, bgId);
	else
	    doDisplayTile(who, bits & TILE_MASK);
	fi;
	if bits & ROOF_BIT = 0 then
	    theCell := tileFindCell(theField, fieldRow, fieldCol);
	    if theCell ~= nil then
		lt := theCell@p_cContents;
		if lt ~= nil then
		    count := Count(lt);
		    if count ~= 0 then
			doDisplayTile(who, lt[count - 1]@p_oObjectTile);
		    fi;
		fi;
		lt := theCell@p_cOccupants;
		if lt ~= nil then
		    occupant := lt[Count(lt) - 1];
		    doDisplayTile(who, occupant@p_pCharacterTile);
		fi;
	    fi;
	fi;
	if bgId ~= 0 then
	    doDisplayTile(who, bits & TILE_MASK);
	fi;
    fi;
corp;

define t_tileGraphics proc TileGetContents()list thing:
    thing me, theCell;

    me := Me();
    theCell :=
	tileFindCell(me@p_pCurrentField, me@p_pCurrentRow, me@p_pCurrentCol);
    if theCell ~= nil then
	theCell@p_cContents
    else
	nil
    fi
corp;

define t_tileGraphics proc TileShowContents(thing who, theField;
	int row, col)bool:
    thing theCell;
    list thing lt;
    int count, i;

    theCell := tileFindCell(theField, row, col);
    if theCell ~= nil then
	lt := theCell@p_cContents;
	if lt ~= nil then
	    count := Count(lt);
	    if count ~= 0 then
		SPrint(who, "Nearby:\n");
		for i from 0 upto count - 1 do
		    SPrint(who, "  " + FormatName(lt[i]@p_oName) + "\n");
		od;
		true
	    else
		false
	    fi
	else
	    false
	fi
    else
	false
    fi
corp;

define t_tileGraphics proc interleave(int row, col)int:
    int i, res, mask;

    res := 0;
    for i from 0 upto 15 do
	res := (res << 1) | (row >> (15 - i)) & 1;
	res := (res << 1) | (col >> (15 - i)) & 1;
    od;
    res
corp;

define tp_tileGraphics proc tileDoForEachAt(thing theField;
	int row, col, dist; action a)bool:
    thing me, otherChar;
    list thing lt;
    int rows, cols, count, i, minPosVal, maxPosVal;
    int minRow, minCol, maxRow, maxCol;
    bool foundOne;

    me := Me();
    theField := me@p_pCurrentField;
    row := me@p_pCurrentRow;
    col := me@p_pCurrentCol;
    minRow := row - dist;
    if minRow < 0 then
	minRow := 0;
    fi;
    minCol := col - dist;
    if minCol < 0 then
	minCol := 0;
    fi;
    minPosVal := interleave(minRow, minCol);
    rows := theField@p_fRows;
    cols := theField@p_fCols;
    maxRow := row + dist;
    if maxRow >= rows then
	maxRow := rows - 1;
    fi;
    maxCol := col + dist;
    if maxCol >= cols then
	maxCol := cols - 1;
    fi;
    maxPosVal := interleave(maxRow, maxCol);
    foundOne := false;
    lt := theField@p_fCharacters;
    count := Count(lt);
    i := 0;
    while i < count and lt[i]@p_pPosVal < minPosVal do
	i := i + 1;
    od;
    while i < count and lt[i]@p_pPosVal <= maxPosVal do
	otherChar := lt[i];
	if otherChar ~= me then
	    row := otherChar@p_pCurrentRow;
	    col := otherChar@p_pCurrentCol;
	    if row >= minRow and row <= maxRow and
		col >= minCol and col <= maxCol
	    then
		call(a, void)(lt[i]);
		foundOne := true;
	    fi;
	fi;
	i := i + 1;
    od;
    foundOne
corp;

define t_tileGraphics proc TileSaveLocation(thing location, agent)void:

    location@p_lSavedField := agent@p_pCurrentField;
    location@p_lSavedRow := agent@p_pCurrentRow;
    location@p_lSavedCol := agent@p_pCurrentCol;
corp;

define t_tileGraphics proc TileDoForEachLocation(thing location; int dist;
	action a)bool:

    tileDoForEachAt(location@p_lSavedField, location@p_lSavedRow,
	location@p_lSavedCol, dist, a)
corp;

define t_tileGraphics proc TileDoForEachHere(int dist; action a)bool:
    thing me;

    me := Me();
    tileDoForEachAt(me@p_pCurrentField, me@p_pCurrentRow, me@p_pCurrentCol,
	dist, a)
corp;

define t_tileGraphics proc TileAddCharacter()void:
    thing me, theField, theCell, otherChar;
    list thing oldList, newList;
    int row, col, posVal, count, i, rows, cols, minPosVal, maxPosVal;
    int minRow, minCol, windowRow, windowCol;

    me := Me();
    theField := me@p_pCurrentField;
    row := me@p_pCurrentRow;
    col := me@p_pCurrentCol;
    posVal := interleave(row, col);
    me@p_pPosVal := posVal;
    oldList := theField@p_fCharacters;
    count := Count(oldList);
    newList := CreateThingList();
    i := 0;
    while i < count and oldList[i]@p_pPosVal < posVal do
	AddTail(newList, oldList[i]);
	i := i + 1;
    od;
    AddTail(newList, me);
    while i < count do
	AddTail(newList, oldList[i]);
	i := i + 1;
    od;
    theField@p_fCharacters := newList;
    count := count + 1;
    theCell := tileGetCell(theField, row, col);
    oldList := theCell@p_cOccupants;
    if oldList = nil then
	oldList := CreateThingList();
	theCell@p_cOccupants := oldList;
    fi;
    AddTail(oldList, me);

    /* This will redraw for the active character, too. */
    if tileGetBits(theField, row, col) & ROOF_BIT = 0 then
	rows := DRAW_ROWS / TILE_HEIGHT;
	cols := DRAW_COLS / TILE_WIDTH;
	minRow := row - rows;
	if minRow < 0 then
	    minRow := 0;
	fi;
	minCol := col - cols;
	if minCol < 0 then
	    minCol := 0;
	fi;
	minPosVal := interleave(minRow, minCol);
	maxPosVal := interleave(row + rows, col + cols);
	i := 0;
	while i < count and newList[i]@p_pPosVal < minPosVal do
	    i := i + 1;
	od;
	while i < count and newList[i]@p_pPosVal < maxPosVal do
	    otherChar := newList[i];
	    windowRow := otherChar@p_pWindowRow;
	    windowCol := otherChar@p_pWindowCol;
	    if row >= windowRow and row < windowRow + rows and
		col >= windowCol and col < windowCol + cols
	    then
		tileShowCell(otherChar, theField, row - windowRow,
			     col - windowCol, row, col);
	    fi;
	    i := i + 1;
	od;
    fi;
corp;

define t_tileGraphics proc TileDelCharacter()void:
    thing me, theField, theCell, otherChar;
    list thing oldList;
    int row, col, count, i, rows, cols, minPosVal, maxPosVal;
    int minRow, minCol, windowRow, windowCol;

    me := Me();
    theField := me@p_pCurrentField;
    row := me@p_pCurrentRow;
    col := me@p_pCurrentCol;
    theCell := tileFindCell(theField, row, col);
    DelElement(theCell@p_cOccupants, me);
    if Count(theCell@p_cOccupants) = 0 then
	theCell -- p_cOccupants;
	maybeFreeCell(theField, theCell, row, col);
    fi;
    oldList := theField@p_fCharacters;
    /* This will undraw from the display of the active character as well. */
    if tileGetBits(theField, row, col) & ROOF_BIT = 0 then
	rows := DRAW_ROWS / TILE_HEIGHT;
	cols := DRAW_COLS / TILE_WIDTH;
	minRow := row - rows;
	if minRow < 0 then
	    minRow := 0;
	fi;
	minCol := col - cols;
	if minCol < 0 then
	    minCol := 0;
	fi;
	minPosVal := interleave(minRow, minCol);
	maxPosVal := interleave(row + rows, col + cols);
	i := 0;
	count := Count(oldList);
	while i < count and oldList[i]@p_pPosVal < minPosVal do
	    i := i + 1;
	od;
	while i < count and oldList[i]@p_pPosVal < maxPosVal do
	    otherChar := oldList[i];
	    windowRow := otherChar@p_pWindowRow;
	    windowCol := otherChar@p_pWindowCol;
	    if row >= windowRow and row < windowRow + rows and
		col >= windowCol and col < windowCol + cols
	    then
		tileShowCell(otherChar, theField, row - windowRow,
			     col - windowCol, row, col);
	    fi;
	    i := i + 1;
	od;
    fi;
    DelElement(oldList, me);
corp;

define t_tileGraphics proc DoTileDelCharacter()status:
    TileDelCharacter();
    continue
corp;

define t_tileGraphics proc TileMaybeDelCharacter()void:
    thing me, theField, theCell;
    
    me := Me();
    theField := me@p_pCurrentField;
    if theField ~= nil then
	theCell := tileFindCell(theField, me@p_pCurrentRow, me@p_pCurrentCol);
	if theCell ~= nil and FindElement(theCell@p_cOccupants, me) ~= -1 then
	    TileDelCharacter();
	fi;
    fi;
corp;

define t_tileGraphics proc TileRedrawCharacter()void:
    thing me, otherChar;
    list thing oldList;
    int row, col, posVal, count, i, rows, cols, minPosVal, maxPosVal;
    int minRow, minCol, windowRow, windowCol;

    /* This redraws for the active character, too. */
    me := Me();
    row := me@p_pCurrentRow;
    col := me@p_pCurrentCol;
    if tileGetBits(me@p_pCurrentField, row, col) & ROOF_BIT = 0 then
	posVal := interleave(row, col);
	oldList := me@p_pCurrentField@p_fCharacters;
	rows := DRAW_ROWS / TILE_HEIGHT;
	cols := DRAW_COLS / TILE_WIDTH;
	minRow := row - rows;
	if minRow < 0 then
	    minRow := 0;
	fi;
	minCol := col - cols;
	if minCol < 0 then
	    minCol := 0;
	fi;
	minPosVal := interleave(minRow, minCol);
	maxPosVal := interleave(row + rows, col + cols);
	i := 0;
	count := Count(oldList);
	while i < count and oldList[i]@p_pPosVal < minPosVal do
	    i := i + 1;
	od;
	while i < count and oldList[i]@p_pPosVal < maxPosVal do
	    otherChar := oldList[i];
	    windowRow := otherChar@p_pWindowRow;
	    windowCol := otherChar@p_pWindowCol;
	    if row >= windowRow and row < windowRow + rows and
		col >= windowCol and col < windowCol + cols
	    then
		tileShowCell(otherChar, me@p_pCurrentField,
		    row - windowRow, col - windowCol, row, col);
	    fi;
	    i := i + 1;
	od;
    fi;
corp;

define t_tileGraphics proc TileShowCurrentRoom()void:
    thing me, theField;
    int windowBottom, windowRight;
    int windowRows, windowCols, windowRow, windowCol;
    int fieldRows, fieldCols, currentRow, currentCol, r, c;

    me := Me();
    theField := me@p_pCurrentField;
    fieldRows := theField@p_fRows;
    fieldCols := theField@p_fCols;
    windowRows := DRAW_ROWS / TILE_HEIGHT;
    windowCols := DRAW_COLS / TILE_WIDTH;
    GSetPen(nil, 0);
    GAMovePixels(nil, 0, 0);
    GRectanglePixels(nil,
	windowCols * TILE_WIDTH, windowRows * TILE_HEIGHT, true);
    windowRow := me@p_pWindowRow;
    windowCol := me@p_pWindowCol;
    currentRow := me@p_pCurrentRow;
    currentCol := me@p_pCurrentCol;
    if windowRow + windowRows > fieldRows then
	windowBottom := fieldRows - windowRow;
    else
	windowBottom := windowRow + windowRows;
    fi;
    if windowCol + windowCols > fieldCols then
	windowRight := fieldCols - windowCol;
    else
	windowRight := windowCol + windowCols;
    fi;
    for r from windowRow upto windowBottom - 1 do
	for c from windowCol upto windowRight - 1 do
	    tileShowCell(me, theField, r - windowRow, c - windowCol, r, c);
	od;
    od;
corp;

define t_tileGraphics proc enterRoomStuff()void:
    thing room, field;

    room := Me()@p_pCurrentRoom;
    if room ~= nil then
	AddPictureButtons(room);
	if room@p_lName1 ~= "" then
	    DrawLocationName(room@p_lName1, room@p_lName2);
	fi;
	if room@p_lHerePicture ~= "" then
	    ShowImage(room@p_lHerePicture);
	fi;
    else
	field := Me()@p_pCurrentField;
	DrawLocationName(field@p_fName1, field@p_fName2);
    fi;
corp;

define t_tileGraphics proc TileInitialDraw()void:
    thing me, theField;
    int windowRows, windowCols, fieldRows, fieldCols;
    int currentRow, currentCol, windowRow, windowCol;

    me := Me();
    theField := me@p_pCurrentField;
    fieldRows := theField@p_fRows;
    fieldCols := theField@p_fCols;
    windowRows := DRAW_ROWS / TILE_HEIGHT;
    windowCols := DRAW_COLS / TILE_WIDTH;
    currentRow := me@p_pCurrentRow;
    currentCol := me@p_pCurrentCol;
    if fieldRows >= currentRow + (windowRows + 1) / 2 then
	windowRow := currentRow - windowRows / 2;
	if windowRow < 0 then
	    windowRow := 0;
	fi;
    elif fieldRows >= windowRows then
	windowRow := fieldRows - windowRows;
    else
	windowRow := 0;
    fi;
    if fieldCols >= currentCol + (windowCols + 1) / 2 then
	windowCol := currentCol - windowCols / 2;
	if windowCol < 0 then
	    windowCol := 0;
	fi;
    elif fieldCols >= windowCols then
	windowCol := fieldCols - windowCols;
    else
	windowCol := 0;
    fi;
    me@p_pWindowRow := windowRow;
    me@p_pWindowCol := windowCol;
    TileShowCurrentRoom();
    enterRoomStuff();
corp;

define t_tileGraphics proc TileSetupGui()void:

    Me()@p_pKnownTiles := CreateIntList();
    if GOn(nil) then
	GUIFramePicture();
	LabelPButtons("Pictures", C_BLUE);
	LabelMButtons("Move", C_DARK_GREEN);
	GClear(nil);
	DrawLocationNameBox();
	AddDirectionButtons(DIR_BUTTONS_ORTHOGONAL | DIR_BUTTONS_LOOK);
	AddStandardRegions();
	TileInitialDraw();
    fi;
    TileAddCharacter();
corp;

define t_tileGraphics proc TileIdleHandler()void:

    TileDelCharacter();
corp;

define t_tileGraphics proc TileEnter(thing theField, theRoom; int row,col)void:
    thing me;

    me := Me();
    me@p_pCurrentField := theField;
    me@p_pCurrentRow := row;
    me@p_pCurrentCol := col;
    me@p_pKnownTiles := CreateIntList();
    me -- p_pCurrentRoom;
    if theRoom ~= nil then
	me@p_pCurrentRoom := theRoom;
    fi;
    if me@p_pCharacterTile = 0 then
	me@p_pCharacterTile := TILE_BASE_CHARACTER;
    fi;
    TileSetupGui();
corp;

define t_tileGraphics proc TileReEnter(thing theRoom; int row, col)void:
    thing me;

    me := Me();
    me@p_pCurrentRow := row;
    me@p_pCurrentCol := col;
    me -- p_pCurrentRoom;
    if theRoom ~= nil then
	me@p_pCurrentRoom := theRoom;
    fi;
    TileInitialDraw();
    TileAddCharacter();
corp;

define t_tileGraphics proc TileStep(int dir)bool:
    thing me, theField, theCell, oldRoom, newRoom;
    int fieldRows, fieldCols, windowRows, windowCols, currentRow, currentCol;
    int windowRow, windowCol, cursorRow, cursorCol, deltaRow, deltaCol;
    int newRow, newCol, bits, maxRow, maxCol, r, c, height, width;
    bool canMove, blocked, hitEdge, noMove, changedRoom;

    me := Me();
    theField := me@p_pCurrentField;
    fieldRows := theField@p_fRows;
    fieldCols := theField@p_fCols;
    windowRows := DRAW_ROWS / TILE_HEIGHT;
    windowCols := DRAW_COLS / TILE_WIDTH;
    windowRow := me@p_pWindowRow;
    windowCol := me@p_pWindowCol;
    currentRow := me@p_pCurrentRow;
    currentCol := me@p_pCurrentCol;
    cursorRow := currentRow - windowRow;
    cursorCol := currentCol - windowCol;
    height := windowRows * TILE_HEIGHT;
    width := windowCols * TILE_WIDTH;
    canMove := true;

    case dir
    incase D_NORTH:
	deltaRow := -1;
	deltaCol := 0;
    incase D_EAST:
	deltaRow := 0;
	deltaCol := 1;
    incase D_SOUTH:
	deltaRow := 1;
	deltaCol := 0;
    incase D_WEST:
	deltaRow := 0;
	deltaCol := -1;
    default:
	canMove := false;
    esac;

    if canMove then
	newRow := currentRow + deltaRow;
	newCol := currentCol + deltaCol;
	theCell := tileFindCell(theField, newRow, newCol);
	blocked := false;
	hitEdge := false;
	noMove := false;
	if theCell ~= nil and theCell@p_cSpecialAction ~= nil then
	    /* Check this first, so can move off of edge. */
	    if not call(theCell@p_cSpecialAction, bool)(me, newRow, newCol,dir)
	    then
		noMove := true;
	    fi;
	fi;
    fi;

    if canMove and not noMove then
	if deltaRow = -1 and currentRow = 0 then
	    canMove := false;
	    hitEdge := true;
	elif deltaCol = -1 and currentCol = 0 then
	    canMove := false;
	    hitEdge := true;
	elif deltaRow = 1 and currentRow + 1 = fieldRows then
	    canMove := false;
	    hitEdge := true;
	elif deltaCol = 1 and currentCol + 1 = fieldCols then
	    canMove := false;
	    hitEdge := true;
	else
	    bits := tileGetBits(theField, newRow, newCol);
	    if bits & NO_GO_BIT ~= 0 then
		canMove := false;
		blocked := true;
	    elif theCell ~= nil then
		if theCell@p_cOccupants ~= nil then
		    canMove := false;
		    blocked := true;
		fi;
	    fi;
	fi;

	maxRow := windowRows;
	if maxRow > fieldRows then
	    maxRow := fieldRows;
	fi;
	maxCol := windowCols;
	if maxCol > fieldCols then
	    maxCol := fieldCols;
	fi;
    fi;
    
    if noMove then
	true
    elif canMove then
	CheckFollowers(dir);
	/* Undraw the character. */
	TileDelCharacter();

	oldRoom := me@p_pCurrentRoom;
	newRoom := nil;
	if oldRoom ~= nil then
	    changedRoom := false;
	    case dir
	    incase D_NORTH:
		if newRow < oldRoom@p_rRow then
		    newRoom := oldRoom@p_lNorth;
		    changedRoom := true;
		fi;
	    incase D_EAST:
		if newCol >= oldRoom@p_rCol + oldRoom@p_rCols then
		    newRoom := oldRoom@p_lEast;
		    changedRoom := true;
		fi;
	    incase D_SOUTH:
		if newRow >= oldRoom@p_rRow + oldRoom@p_rRows then
		    newRoom := oldRoom@p_lSouth;
		    changedRoom := true;
		fi;
	    incase D_WEST:
		if newCol < oldRoom@p_rCol then
		    newRoom := oldRoom@p_lWest;
		    changedRoom := true;
		fi;
	    esac;
	    if changedRoom then
		ErasePictureButtons(oldRoom);
		me -- p_pCurrentRoom;
		if newRoom = nil then
		    enterRoomStuff();
		fi;
	    fi;
	else
	    if theCell ~= nil and theCell@p_cRoom ~= nil then
		newRoom := theCell@p_cRoom;
	    fi;
	fi;
	if newRoom ~= nil then
	    me@p_pCurrentRoom := newRoom;
	    enterRoomStuff();
	fi;

	/* Scroll the view if appropriate. */
	case dir
	incase D_NORTH:
	    if cursorRow = 1 and windowRow ~= 0 then
		windowRow := windowRow - 1;
		GScrollRectanglePixels(nil, 0, TILE_HEIGHT, 0, 0,
				       width, height);
		for c from 0 upto maxCol - 1 do
		    tileShowCell(me, theField, 0, c, windowRow, windowCol + c);
		od;
	    else
		cursorRow := cursorRow - 1;
	    fi;
	incase D_EAST:
	    if cursorCol = windowCols - 2 and
		windowCol + windowCols ~= fieldCols
	    then
		windowCol := windowCol + 1;
		GScrollRectanglePixels(nil, - TILE_WIDTH, 0, 0, 0,
				       width, height);
		for r from 0 upto maxRow - 1 do
		    tileShowCell(me, theField, r, maxCol - 1, windowRow + r,
				 windowCol + windowCols - 1);
		od;
	    else
		cursorCol := cursorCol + 1;
	    fi;
	incase D_SOUTH:
	    if cursorRow = windowRows - 2 and
		windowRow + windowRows ~= fieldRows
	    then
		windowRow := windowRow + 1;
		GScrollRectanglePixels(nil, 0, - TILE_HEIGHT, 0, 0,
				       width, height);
		for c from 0 upto maxCol - 1 do
		    tileShowCell(me, theField, maxRow - 1, c,
				 windowRow + windowRows - 1, windowCol + c);
		od;
	    else
		cursorRow := cursorRow + 1;
	    fi;
	incase D_WEST:
	    if cursorCol = 1 and windowCol ~= 0 then
		windowCol := windowCol - 1;
		GScrollRectanglePixels(nil, TILE_WIDTH, 0, 0, 0,
				       width, height);
		for r from 0 upto maxRow - 1 do
		    tileShowCell(me, theField, r, 0, windowRow + r, windowCol);
		od;
	    else
		cursorCol := cursorCol - 1;
	    fi;
	esac;
	me@p_pCurrentRow := newRow;
	me@p_pCurrentCol := newCol;
	me@p_pWindowRow := windowRow;
	me@p_pWindowCol := windowCol;
	TileAddCharacter();
	ignore TileShowContents(me, theField, newRow, newCol);
	true
    elif blocked then
	if not GOn(me) then
	    Print("Blocked at " + IntToString(newRow) + "," +
		IntToString(newCol) + ", window at " +
		IntToString(windowRow) + "," + IntToString(windowCol) + "\n");
	fi;
	/* Perhaps just scroll view. */
	case dir
	incase D_NORTH:
	    if cursorRow < windowRows - 2 and windowRow ~= 0 then
		windowRow := windowRow - 1;
		me@p_pWindowRow := windowRow;
		GScrollRectanglePixels(nil, 0, TILE_HEIGHT, 0, 0,
				       width, height);
		for c from 0 upto maxCol - 1 do
		    tileShowCell(me, theField, 0, c, windowRow, windowCol + c);
		od;
	    fi;
	incase D_EAST:
	    if cursorCol > 1 and windowCol + windowCols < fieldCols then
		windowCol := windowCol + 1;
		me@p_pWindowCol := windowCol;
		GScrollRectanglePixels(nil, - TILE_WIDTH, 0, 0, 0,
				       width, height);
		for r from 0 upto maxRow - 1 do
		    tileShowCell(me, theField, r, maxCol - 1, windowRow + r,
				 windowCol + windowCols - 1);
		od;
	    fi;
	incase D_SOUTH:
	    if cursorRow > 1 and windowRow + windowRows < fieldRows then
		windowRow := windowRow + 1;
		me@p_pWindowRow := windowRow;
		GScrollRectanglePixels(nil, 0, - TILE_HEIGHT, 0, 0,
				       width, height);
		for c from 0 upto maxCol - 1 do
		    tileShowCell(me, theField, maxRow - 1, c,
				 windowRow + windowRows - 1, windowCol + c);
		od;
	    fi;
	incase D_WEST:
	    if cursorCol < windowCols - 2 and windowCol ~= 0 then
		windowCol := windowCol - 1;
		me@p_pWindowCol := windowCol;
		GScrollRectanglePixels(nil, TILE_WIDTH, 0, 0, 0,
				       width, height);
		for r from 0 upto maxRow - 1 do
		    tileShowCell(me, theField, r, 0, windowRow + r, windowCol);
		od;
	    fi;
	esac;
	true
    else
	if hitEdge then
	    if not GOn(me) then
		Print("Can't go off edge of world.\n");
	    fi;
	fi;
	false
    fi
corp;

define t_tileGraphics proc TileTryToMove(int dir)bool:
    thing me, theField, theCell;
    int currentRow, currentCol, deltaRow, deltaCol, newRow, newCol, bits;
    bool canMove;

    me := Me();
    theField := me@p_pCurrentField;
    currentRow := me@p_pCurrentRow;
    currentCol := me@p_pCurrentCol;
    canMove := true;

    case dir
    incase D_NORTH:
	deltaRow := -1;
	deltaCol := 0;
    incase D_EAST:
	deltaRow := 0;
	deltaCol := 1;
    incase D_SOUTH:
	deltaRow := 1;
	deltaCol := 0;
    incase D_WEST:
	deltaRow := 0;
	deltaCol := -1;
    default:
	canMove := false;
    esac;

    if canMove then
	newRow := currentRow + deltaRow;
	newCol := currentCol + deltaCol;
	theCell := tileFindCell(theField, newRow, newCol);
	if deltaRow = -1 and currentRow = 0 then
	    canMove := false;
	elif deltaCol = -1 and currentCol = 0 then
	    canMove := false;
	elif deltaRow = 1 and currentRow + 1 = theField@p_fRows then
	    canMove := false;
	elif deltaCol = 1 and currentCol + 1 = theField@p_fCols then
	    canMove := false;
	else
	    bits := tileGetBits(theField, newRow, newCol);
	    if bits & NO_GO_BIT ~= 0 then
		canMove := false;
	    elif theCell ~= nil then
		if theCell@p_cOccupants ~= nil then
		    canMove := false;
		fi;
	    fi;
	fi;
    fi;
    canMove
corp;

define t_tileGraphics proc TileMachineMove(int dir)void:
    thing me, theField, theCell, oldRoom, newRoom;
    int currentRow, currentCol, deltaRow, deltaCol, newRow, newCol, bits;
    bool canMove, noMove, changedRoom;

    me := Me();
    theField := me@p_pCurrentField;
    currentRow := me@p_pCurrentRow;
    currentCol := me@p_pCurrentCol;
    canMove := true;

    case dir
    incase D_NORTH:
	deltaRow := -1;
	deltaCol := 0;
    incase D_EAST:
	deltaRow := 0;
	deltaCol := 1;
    incase D_SOUTH:
	deltaRow := 1;
	deltaCol := 0;
    incase D_WEST:
	deltaRow := 0;
	deltaCol := -1;
    default:
	canMove := false;
    esac;

    if canMove then
	newRow := currentRow + deltaRow;
	newCol := currentCol + deltaCol;
	theCell := tileFindCell(theField, newRow, newCol);
	noMove := false;
	if theCell ~= nil and theCell@p_cSpecialAction ~= nil then
	    if not call(theCell@p_cSpecialAction, bool)(me, newRow, newCol,dir)
	    then
		noMove := true;
		canMove := false;
	    fi;
	fi;
    fi;

    if canMove and not noMove then
	if deltaRow = -1 and currentRow = 0 then
	    canMove := false;
	elif deltaCol = -1 and currentCol = 0 then
	    canMove := false;
	elif deltaRow = 1 and currentRow + 1 = theField@p_fRows then
	    canMove := false;
	elif deltaCol = 1 and currentCol + 1 = theField@p_fCols then
	    canMove := false;
	else
	    bits := tileGetBits(theField, newRow, newCol);
	    if bits & NO_GO_BIT ~= 0 then
		canMove := false;
	    elif theCell ~= nil then
		if theCell@p_cOccupants ~= nil then
		    canMove := false;
		fi;
	    fi;
	fi;
    fi;
    
    if canMove then
	CheckFollowers(dir);
	TileDelCharacter();

	oldRoom := me@p_pCurrentRoom;
	newRoom := nil;
	if oldRoom ~= nil then
	    changedRoom := false;
	    case dir
	    incase D_NORTH:
		if newRow < oldRoom@p_rRow then
		    newRoom := oldRoom@p_lNorth;
		    changedRoom := true;
		fi;
	    incase D_EAST:
		if newCol >= oldRoom@p_rCol + oldRoom@p_rCols then
		    newRoom := oldRoom@p_lEast;
		    changedRoom := true;
		fi;
	    incase D_SOUTH:
		if newRow >= oldRoom@p_rRow + oldRoom@p_rRows then
		    newRoom := oldRoom@p_lSouth;
		    changedRoom := true;
		fi;
	    incase D_WEST:
		if newCol < oldRoom@p_rCol then
		    newRoom := oldRoom@p_lWest;
		    changedRoom := true;
		fi;
	    esac;
	    if changedRoom then
		me -- p_pCurrentRoom;
	    fi;
	else
	    if theCell ~= nil and theCell@p_cRoom ~= nil then
		newRoom := theCell@p_cRoom;
	    fi;
	fi;
	if newRoom ~= nil then
	    me@p_pCurrentRoom := newRoom;
	fi;
	me@p_pCurrentRow := newRow;
	me@p_pCurrentCol := newCol;
	TileAddCharacter();
    fi;
corp;

define t_tileGraphics proc public TileButtonHandler(int whichButton)void:
    string name;
    int dir, saveComponent;

    if whichButton = BUTTON_D_L then
	InsertCommand("look");
    elif whichButton >= BUTTON_P_NW and whichButton <= BUTTON_P_O then
	dir := D_NORTH;
	while dir <= D_HERE and DirPictureButton(dir) ~= whichButton do
	    dir := dir + 1;
	od;
	if dir > D_HERE then
	    Print("Can't find button id?\n");
	else
	    name := Me()@p_pCurrentRoom@(DirPictureProp(dir));
	    if name = "" then
		Print("No image for that direction!\n");
	    else
		saveComponent := DrawComponent(nil, COMPONENT_PICTURE);
		GSetImage(nil, name);
		IfFound(nil);
		    GShowImage(nil, "", 0.0, 0.0, 1.0, 1.0, 0.0, 0.0, 1.0,1.0);
		Else(nil);
		    FailText(nil, "Image not found.");
		Fi(nil);
		ignore DrawComponent(nil, saveComponent);
	    fi;
	fi;
    elif whichButton >= BUTTON_D_NW and whichButton <= BUTTON_D_O then
	dir := D_NORTH;
	while dir <= D_EXIT and DirButton(dir) ~= whichButton do
	    dir := dir + 1;
	od;
	if dir > D_EXIT then
	    Print("Can't find move button id?\n");
	else
	    UnFollow();
	    ignore TileStep(dir);
	fi;
    else
	Print("Unknown button ");
	IPrint(whichButton);
	Print(" hit!\n");
    fi;
corp;

/* a standard region hit handler for the above region */

define t_tileGraphics proc public TileMouseDownHandler(int region, x, y)void:
    thing me;
    int rows, cols, cursorX, cursorY, deltaX, deltaY;

    if region = MOVE_REGION then
	me := Me();
	cursorX := (me@p_pCurrentCol - me@p_pWindowCol) * TILE_WIDTH +
	    TILE_WIDTH / 2;
	cursorY := (me@p_pCurrentRow - me@p_pWindowRow) * TILE_HEIGHT +
	    TILE_HEIGHT / 2;
	if cursorX < x then
	    deltaX := x - cursorX;
	else
	    deltaX := cursorX - x;
	fi;
	if cursorY < y then
	    deltaY := y - cursorY;
	else
	    deltaY := cursorY - y;
	fi;
	UnFollow();
	if deltaY <= deltaX then
	    if x < cursorX then
		ignore TileStep(D_WEST);
	    else
		ignore TileStep(D_EAST);
	    fi;
	else
	    if y < cursorY then
		ignore TileStep(D_NORTH);
	    else
		ignore TileStep(D_SOUTH);
	    fi;
	fi;
    else
	Print("Unknown region ");
	IPrint(region);
	Print(" hit!\n");
    fi;
corp;

unuse tp_tileGraphics
