/*
 * Amiga MUD
 *
 * Copyright (c) 1999 by Chris Gray
 */
 
/*
 * handler.m - define and install the new-player handler. This is done
 *	last in sourcing the scenario, so that it can use anything from
 *	any of the pieces of the scenario, including optional parts.
 */

/*
 * newPlayer - this is the routine which we set up to be called when a
 *	new player is created.
 */

define tp_common proc newPlayer()void:
    string name;
    thing me, lm;
    action a;

    /* Not specific to mode of initial location */
    me := Me();
    name := me@p_pName;
    me@p_pCarrying := CreateThingList();
    me@p_pHiddenList := CreateThingList();
    if me@p_pDesc = "" then
	/* not SysAdmin */
	me@p_pDesc := Capitalize(name) + " is a nondescript adventurer.";
	me@p_pMoney := 75;
    fi;
    /* Do this carefully. We have set SysAdmin up with these lists already,
       and some code, e.g. the build2 code, has added routines to these
       lists, which we want to execute after all normal setup is done. */
    if me@p_pEnterActions = nil then
	me@p_pEnterActions := CreateActionList();
    fi;
    if me@p_pToRoomActions = nil then
	me@p_pToRoomActions := CreateActionList();
    fi;
    if me@p_pExitActions = nil then
	me@p_pExitActions := CreateActionList();
    fi;
    me@p_pVerbose := true;
    me@p_pSuperBrief := false;
    me@p_pEchoPose := true;
    me@p_pStandard := true;
    me@p_pPosition := POS_NONE;
    me@p_pWhere := me;
    ignore SetCharacterInputAction(parseInput);
    ignore SetCharacterRawKeyAction(handleRawKey);
    ignore SetCharacterIdleAction(idleAction);
    ignore SetCharacterActiveAction(activeAction);

    /* Do these only if room-mode is included. */
    me@p_pCursor := MakeCursor();
    me@p_pCursorColour := C_BLUE;
    me@p_pIconColour := C_RED;
    me@p_MapGroup := NO_MAP_GROUP;
    
    if true then
	/* Enter specifically in the Arrivals Room. */
	lm := Room;
	SetLocation(r_arrivals);
	/* Secondary initialization, only if room-mode is included, active. */
	if GOn(nil) then
	    InitRoomGraphics();
	else
	    me@p_pRoomGraphicsDone := false;
	fi;
    else
	/* Enter specifically in the tile-mode town of Tribble. */
	lm := Tile;
	TileWorldEnter();
    fi;
    me@p_pCurrentMode := lm;
    ignore SetCharacterButtonAction(lm@p_lmInitialButtonHandler);
    ignore SetCharacterMouseDownAction(lm@p_lmInitialMouseDownHandler);
    
    /* Not specific to mode of initial location */
    /* This will cause SysAdmin to become a builder. Want to do these
       *after* initializing the mode graphics. */
    DoList(me@p_pEnterActions);

    Print("Welcome to the sample V" + IntToString(ServerVersion() / 10) + "." +
	 IntToString(ServerVersion() % 10) + " MUD world, " + name +
	"! Your character has been created, but it is quite minimal - "
	"you are not carrying anything, and your appearance is dull. "
	"You will soon be able to remedy these conditions. Some of the "
	"commands available: quit, north, n, up, enter, northeast, verbose, "
	"terse, inventory, get, drop, look, examine, etc. Have fun!\n");

    if lm = Room then
	/* Specific to room-mode of initial location (r_arrivals) */
	if GlobalThing@p_FreeBuilding then
	    /* Build facility might not be present! */
	    a := LookupAction(tp_misc, "doMakeBuilder");
	    if a ~= nil then
		ignore call(a, status)();
	    fi;
	fi;
	note - we assume the arrivals room is not dark;
	ForEachAgent(r_arrivals, ShowIconOnce);
	OPrint("New player " + name + " has appeared.\n");
	ignore ShowRoomToMe(true);
    fi;

    ignore ShowClients(false);
corp;

ignore SetNewCharacterAction(newPlayer)$
